#include "qcalc.h"
#include "qwui.h"

using namespace QSAS;

extern qwuiWindow* qsasWindow;

//---------------------------------------------------------------------------------------------------//
//---------------------------------------------------------------------------------------------------//

// QcalcListView class
// ------------------

/*!
    \class QcalcListView
    \brief This class implements a list/tree view. This class specialises the QSAS class QguListView. \n
	It displays 2 column items one for the name of the object and the second is its alias.

	The main signals that are processed in this class are:
	- QcalcListView::SelectionUpdated(): The QcalcListView class emits this signal when processing the mouse movements events that lead to an item
	selection. Its corresponding slot will update the join tab showing the joining options for the newly selected item(s).
	- QcalcListView::listModified: The QcalcListView emits this signal when its content has been changed. Its corresponding signals will
	 respectively update the link for the objects originating from the working list and update the time feed-back details (if objects are
	 time-tagged).

	 The main slots for this class are grouped into 5 categories:
	 - Slots for setting Join tab widgets when the user enters/updates the content of those widgets
	 - Slots for setting calculation tab widgets when the user enters/updates the content of those widgets. The name of the widgets within the
	 calculation tab are set to the label of this tab (alias of the operation created and added to the list view).  Consequently, when setting
	 those widgets with values (entered by the user), their names will be used to identify their corresponding tab page.
	</PRE>
	\endhtmlonly
	 - Slots for deleting and renaming items within its list view
	 - Slots for update the Join tab widgets when a new selection has occurred within its list view
	 - Slots for dealing with signals emitted by the QCalcWindow class (to make sure that the low level resources of this class are only
	 manipulated by its built-in slots).


*/



QcalcListView::QcalcListView( QWidget     *parent,
                              QcalcWindow *QjWindow,
			                  const char  *name )
   : QguListView(parent,name)
{
	this->owner = QjWindow;

	RButtonMenu = new Q_POPUP_MENU( this );

	//RButtonMenu->insertItem( "&New Folder",    this, SLOT( makeGroup() ),                    CTRL+Key_N  );
	RButtonMenu->insertItem( "Re-&Name Object Alias", this, SLOT( renameSelectedItemAlias() ),Qt::CTRL+Qt::Key_N, RENAME_ALIAS_RB_MENU_ITEM );
	RButtonMenu->insertSeparator();

	RButtonMenu->insertItem( "&Copy",          this, SLOT( copy() ),                         Qt::CTRL+Qt::Key_C );
	RButtonMenu->insertItem( "Paste",          this, SLOT( paste() ) ,                       Qt::CTRL+Qt::Key_V);
	RButtonMenu->insertItem( "select &All",    this, SLOT( selectAll() ),                    Qt::CTRL+Qt::Key_A );
	RButtonMenu->insertSeparator();

	RButtonMenu->insertItem( "Copy Join Settings",  QjWindow,  SLOT( keepChoices() ),           Qt::CTRL+Qt::Key_Left   );
	RButtonMenu->insertItem( "Apply Join Settings", QjWindow,  SLOT( applyChoices() ),          Qt::CTRL+Qt::Key_Right  );
	RButtonMenu->insertSeparator();

	RButtonMenu->insertItem( "Delete",         this, SLOT( deleteListItems() ),             Qt::CTRL+Qt::Key_X );
	RButtonMenu->insertItem( "Clear List",     this, SLOT( deleteAllItems() )                             );
	RButtonMenu->insertSeparator();

	RButtonMenu->insertItem( "Move Down",      this, SLOT( moveItemDown() ) ,                Qt::CTRL+Qt::Key_Down);
	RButtonMenu->insertItem( "Move Up",        this, SLOT( moveItemUp() ) ,                  Qt::CTRL+Qt::Key_Up);

	setAcceptDrops( true );
	setAllColumnsShowFocus( true );
	connect( qsasWindow->WorkingList, SIGNAL( listModified() ),   this,  SLOT(validateWLitems()     ) );
	connect( this,  SIGNAL( listModified()     ),                 owner,  SLOT( setTimeFeedback()   ) );


	// Set the private data members
	m_strAliseName   =  QString("");
	m_chChar         = 65;
	alreadySelected  = false;
     mousePressed     = false;
     dragItem         = NULL;
}



QcalcListView::~QcalcListView()
{


}

QdObject_var QcalcListView::getAliasResultObject(QString objName)
{
  QdObject_var nil_obj;
  QcalcListVirtualItem *qItem = 
    dynamic_cast <QcalcListVirtualItem*> ( this->findItem( objName, Alias_) );
  if(qItem == NULL)  return nil_obj;
  return qItem->getResultObject();
}


QguAliasType QcalcListView::getAliasObject(QString objName,QdObject_var dobj)
{
  dobj=0;
  QcalcListVirtualItem *qItem =
    ( QcalcListVirtualItem* )this->findItem( objName, Alias_ );
  if(qItem && qItem->m_eObjType == WORKING_LIST_OBJECT)
  {
    // Is this itself a WL object, else we can't get Metadata
   dobj = qItem->m_pcalcTab->operandStack[0]->slot->getSlotObject();
   if(dobj.is_nil())
     return ALIAS_NOT_FOUND;
   else
     return ALIAS_FOUND;
  }
  else
    return ALIAS_VIRTUAL;
}

void QcalcListView::getAliasArrDim(QString objName,Dim_var &dim)
{
  dim=0;
  QcalcListVirtualItem *qItem =
    ( QcalcListVirtualItem* )this->findItem( objName, Alias_ );
  if(qItem)
    dim =  qItem->getVirtualArrDim();
}

QguAliasObjType QcalcListView::getAliasObjectType(QString objName)
{
  QcalcListVirtualItem *qItem = 
	  ( QcalcListVirtualItem* )this->findItem( objName, Alias_ );
  if(qItem)
  {
    if(qItem->getIsVector())
      return ALIAS_IS_VECTOR;
    else if (qItem->getIsArray())
      return ALIAS_IS_ARRAY;
    else
      return ALIAS_IS_OTHER;
  }
  else
    return ALIAS_IS_UNKNOWN;
}


