/***********************************************************************
*           See Accompanying licence file QSAS_licence                 *
***********************************************************************/
/* PROJECT: Cluster UK CDHF
*
* COMPONENT:    Array Sum
* 
* MODULE:	Array dimension reduction
*
* LANGUAGE:	C++
*
* FUNCTION:
*
* PURPOSE: QSAS Array dimension reduction by summing down one index.
*               Reduces dimension of array by 1
* 
* DESCRIPTION:
*
* ARGUMENTS:
*    IN:
*     MATRIX_SEQUENCE    ts_in           input matrix series or matrix
*     INTEGER            index_i         which dimension index to sum along
*    OUT:
*     ANY_DATA_OBJECT    ts_out
* GLOBALS:
*
* RETURN:
*    TYPE:     QplugReturnStatus
*    VALUES:   see qplug_if.h
*
* FUNCTIONS_USED 
*
* ALGORITHM:   
*              most of the work is in the attributes
*
* HISTORY:		Tony Allen, April 2004
*
***********************************************************************/


#include <stdio.h>
#include <math.h>
#include <string.h>
#include <stdlib.h>
#include "qplug_if.h"
#include "Qdos.h"
#include "qdutil.h"
#include "ArraySum.h"

using namespace QSAS;

QplugReturnStatus  ArraySum  ( QplugArgList *call_list )

{

QdObject_var  ts_in;
QdObject_var  ts_out;
QdRScalar_var  index_i;
 
QdRScalar_var s_out;
QdRMatrix_var m_out;
QdRScalarSeq_var ss_out;
QdRMatrixSeq_var ms_out;

Dim mDim;
int mRank;

char msg_line[100];
char num_str[20];

//	..first item is input Time Series

ts_in = (*call_list)[0];
if ( ts_in.is_nil() ) {	
   QplugAppendTextDisplay ("Null input object!\n");
   return QPLUG_FAILURE; 
}        

// get index to sum along 

index_i = QdRScalar_var::narrow((*call_list)[1]);

if ( index_i.is_nil() ) { 
    QplugAppendTextDisplay (
        "ERROR - index argument not an integer\n");
    return QPLUG_FAILURE; 
}  

int ii = (int)(*index_i)-1;

QdRMatrix_var Mat_in = QdRMatrix_var::narrow(ts_in);
QdRMatrixSeq_var MatSeq_in = QdRMatrixSeq_var::narrow(ts_in);

// handle matrix and matrix sequence separately
// Matrix first
if ( !Mat_in.is_nil()  ) {
  
  // dimension sizes
  mDim = Mat_in->dimdata();
  mRank = mDim.rank();
  for(int ix=0; ix<mRank; ix++){
    sprintf(msg_line, "Index %d has range 1...%d\n", ix+1, (Mat_in->dims())[ix]);
    QplugAppendTextDisplay(msg_line);
  }
  if(ii >= mRank || ii <0){
    QplugAppendTextDisplay ("ERROR - index out of range\n");
    return QPLUG_FAILURE; 
  }
  
  // handle each case separately
  if(mRank == 1){
    s_out = new QdRScalar((*Mat_in)[0]);
    for (int ei=1; ei<(Mat_in->dims())[ii]; ei++){
      *s_out += (*Mat_in)[ei];
    }
    ts_out = (QdObject_var) s_out;
  }
  else if(mRank == 2){
    if(ii == 0){
      m_out = new QdRMatrix((*Mat_in)[0][all]);
      for (int ei=1; ei<(Mat_in->dims())[ii]; ei++){
        *m_out += (*Mat_in)[ei][all];
      }
      ts_out = (QdObject_var) m_out;
    }
    else if(ii == 1){
      m_out = new QdRMatrix((*Mat_in)[all][0]);
      for (int ei=1; ei<(Mat_in->dims())[ii]; ei++){
        *m_out += (*Mat_in)[all][ei];
      }
      ts_out = (QdObject_var) m_out;
    }
    else{
      QplugAppendTextDisplay ("Matrix Rank error - tell csc support\n");
      return QPLUG_FAILURE;
    }
  }
  else if(mRank == 3){
    if(ii == 0){
      m_out = new QdRMatrix((*Mat_in)[0][all][all]);
      for (int ei=1; ei<(Mat_in->dims())[ii]; ei++){
        *m_out += (*Mat_in)[ei][all][all];
      }
      ts_out = (QdObject_var) m_out;
    }
    else if(ii == 1){
      m_out = new QdRMatrix((*Mat_in)[all][0][all]);
      for (int ei=1; ei<(Mat_in->dims())[ii]; ei++){
        *m_out += (*Mat_in)[all][ei][all];
      }
      ts_out = (QdObject_var) m_out;
    }
    else if(ii == 2){
      m_out = new QdRMatrix((*Mat_in)[all][all][0]);
      for (int ei=1; ei<(Mat_in->dims())[ii]; ei++){
        *m_out += (*Mat_in)[all][all][ei];
      }
       ts_out = (QdObject_var) m_out;
    }
    else{
      QplugAppendTextDisplay ("Matrix Rank error - tell csc support\n");
      return QPLUG_FAILURE;
    }
  }
  else{
    QplugAppendTextDisplay ("Sorry, only matrices up to Rank 3 implemented\n");
    return QPLUG_FAILURE; 
  }
}	
else if ( !MatSeq_in.is_nil() ) {
  // sequence size
  int ndata = MatSeq_in->sequence_size();
  sprintf(msg_line, "Input sequence of %d arrays\n", ndata);
  QplugAppendTextDisplay(msg_line);

  // dimension sizes
  mDim = MatSeq_in[0].dimdata();
  mRank = mDim.rank();
  for(int ix=0; ix<mRank; ix++){
    sprintf(msg_line, "Index %d has range 1...%d\n", ix+1, 
               ((MatSeq_in[0].dims())[ix]));
    QplugAppendTextDisplay(msg_line);
  }
  
  if(ii >= mRank || ii<0){
    QplugAppendTextDisplay ("ERROR - index out of range\n");
    return QPLUG_FAILURE; 
  }

  // handle each case separately
  if(mRank == 1){
    ss_out = new QdRScalarSeq((*MatSeq_in)[all][0]);
    for (int ei=1; ei<(MatSeq_in[0].dims())[ii]; ei++){
      *ss_out += (*MatSeq_in)[all][ei];
    }
    ts_out = (QdObject_var) ss_out;
  }
  else if(mRank == 2){
    if(ii == 0){
      ms_out = new QdRMatrixSeq((*MatSeq_in)[all][0][all]);
      for (int ei=1; ei<(MatSeq_in[0].dims())[ii]; ei++){
        *ms_out += (*MatSeq_in)[all][ei][all];
      }
      ts_out = (QdObject_var) ms_out;
    }
    else if(ii == 1){
      ms_out = new QdRMatrixSeq((*MatSeq_in)[all][all][0]);
      for (int ei=1; ei<(MatSeq_in[0].dims())[ii]; ei++){
        *ms_out += (*MatSeq_in)[all][all][ei];
      }
      ts_out = (QdObject_var) ms_out;
    }
    else{
      QplugAppendTextDisplay ("Matrix Rank error - tell csc support\n");
      return QPLUG_FAILURE;
    }
  }
  else if(mRank == 3){
    if(ii == 0){
      ms_out = new QdRMatrixSeq((*MatSeq_in)[all][0][all][all]);
      for (int ei=1; ei<(MatSeq_in[0].dims())[ii]; ei++){
        *ms_out += (*MatSeq_in)[all][ei][all][all];
      }
      ts_out = (QdObject_var) ms_out;
    }
    else if(ii == 1){
      ms_out = new QdRMatrixSeq((*MatSeq_in)[all][all][0][all]);
      for (int ei=1; ei<(MatSeq_in[0].dims())[ii]; ei++){
        *ms_out += (*MatSeq_in)[all][all][ei][all];
      }
      ts_out = (QdObject_var) ms_out;
    }
    else if(ii == 2){
      ms_out = new QdRMatrixSeq((*MatSeq_in)[all][all][all][0]);
      for (int ei=1; ei<(MatSeq_in[0].dims())[ii]; ei++){
        *ms_out += (*MatSeq_in)[all][all][all][ei];
      }
      ts_out = (QdObject_var) ms_out;
    }
    else{
      QplugAppendTextDisplay ("Matrix Rank error - tell csc support\n");
      return QPLUG_FAILURE;
    }
  }
  else{
    QplugAppendTextDisplay ("Sorry, only matrices up to Rank 3 implemented\n");
    return QPLUG_FAILURE; 
  }
  
}
else{	
   QplugAppendTextDisplay ("ERROR - input not an array or array series!\n");
   return QPLUG_FAILURE; 
}        

// copy basic Xrefs across...  
// this will ensure new object points to same time tags.
       
ts_out->copy_xrefs_from(ts_in); 

// create new LABLAXIS 

char * comp=""; 
char * unit="";
string labstr;

if (ts_out->xref_exists(LABLAXIS) ){
  
  // handle case when LABLAXIS is an array 
  QdObject_var xref=ts_out->get_xref(LABLAXIS);
  QdStringSeq_var labi = QdStringSeq_var::narrow(xref);
  if(!labi.is_nil() ){
    // works only for 1-D array, await cef final spec...
    labstr = "Sum ";
    labstr += labi[0];
    for (int ei=1; ei<labi->size(); ei++){
      labstr += ", ";
      labstr += labi[ei];
    }
  }
  else{
    // regular LABLAXIS string
    labstr=string(QuGetAttrText(LABLAXIS, ts_out));
  }
}
else if (ts_out->xref_exists("ObjectName") ){
  labstr = string(QuGetAttrText("ObjectName", ts_out));
}

string labl_attr = "Label_";
if (ii == 0) labl_attr += "1";
if (ii == 1) labl_attr += "2";
if (ii == 2) labl_attr += "3";

string dep_attr = "DEPEND_";
if (ii == 0) dep_attr += "1";
if (ii == 1) dep_attr += "2";
if (ii == 2) dep_attr += "3";

if (ts_out->xref_exists(labl_attr.c_str()) ){
  QdObject_var xref=ts_out->get_xref(labl_attr.c_str());
  QdStringSeq_var labi = QdStringSeq_var::narrow(xref);
  if (!labi.is_nil()){
    labstr += " Sum ";
    labstr += (*labi)[0];  
    for (int ei=1; ei<labi->size(); ei++){
      labstr += ", ";
      labstr += labi[ei];
    }
  }
  else {
    labstr += string(" sum down ");
    labstr += string(QuGetAttrText(labl_attr.c_str(), ts_out));
  }
}
else if (ts_out->xref_exists(dep_attr.c_str())){
   labstr += string(" Sum(");
   QdObject_var dxref=ts_out->get_xref(dep_attr.c_str());
   QdRMatrix_var Dxref=QdRMatrix_var::narrow(dxref);
   QdRMatrixSeq_var DSxref=QdRMatrixSeq_var::narrow(dxref);
   if(!Dxref.is_nil()){
     if (Dxref->xref_exists(LABLAXIS)){
       comp = QuGetAttrText(LABLAXIS, Dxref);
       labstr += string(comp);  
     }
     else if (Dxref->xref_exists(FIELDNAM)){
       comp = QuGetAttrText(FIELDNAM, Dxref);
       labstr += string(comp);   
     }
     if (Dxref->xref_exists(UNITS)){
       labstr += " ";
       unit = QuGetAttrText(UNITS, Dxref);
       labstr += string(unit);
     }
   }
   else if(!DSxref.is_nil()){
     QplugAppendTextDisplay ("CAUTION: Index dependency is RECORD VARYING!\n");
     QplugAppendTextDisplay ("         Sum may not be meaningful!\n");

     if (DSxref->xref_exists(LABLAXIS)){
       comp = QuGetAttrText(LABLAXIS, DSxref);
       labstr += string(comp);  
     }
     else if (DSxref->xref_exists(FIELDNAM)){
       comp = QuGetAttrText(FIELDNAM, DSxref);
       labstr += string(comp);   
     }
     if (DSxref->xref_exists(UNITS)){
       labstr += " ";
       unit = QuGetAttrText(UNITS, DSxref);
       labstr += string(unit);
     }
   }
   labstr += string(")");
}
 QuSetTxtAttr(LABLAXIS, labstr.c_str(), ts_out);
 
 string mods_str = "Array summed over index ";
 sprintf(num_str, "%d", ii+1);
 mods_str += string(num_str);
 mods_str += " (see LABLAXIS)"; 
 QuSetTxtAttr("MODS", mods_str.c_str(), ts_out);

 if (ts_out->xref_exists(UNITS)){
   string units_str = string(QuGetAttrText(UNITS, ts_out));
   units_str += " summed";  
   QuSetTxtAttr("UNITS", units_str.c_str(), ts_out);
 }

if (mRank == 1){
// Some metadata may be string sequences/arrays

// these are not handled properly yet and will be skipped
// until CEF interfacing implemented fully

    if (ts_out->xref_exists(DEPEND_1) ){
      ts_out->delete_xref(DEPEND_1);
    }
    if (ts_out->xref_exists(LABEL_1) ){
      ts_out->delete_xref(LABEL_1);
    }

}
else if (mRank == 2) {
// dimensional metadata must be handled by Depend_i or Label_i
// watch for Depend_i == index i-1

  if( ii == 1) {
    if (ts_out->xref_exists(DEPEND_2) ){
      ts_out->delete_xref(DEPEND_2);
    }
    // leave DEPEND_1 untouched
     if (ts_out->xref_exists(LABEL_2) ){
      ts_out->delete_xref(LABEL_2);
    }
    // leave LABEL_1 untouched
 }
  else if (ii == 0){
    // delete it and shuffle indices down
    if(ts_out->xref_exists(LABEL_1))ts_out->delete_xref(LABEL_1);
    if(ts_out->xref_exists(DEPEND_1))ts_out->delete_xref(DEPEND_1);
    
    if(ts_out->xref_exists(DEPEND_2))
    {
      ts_out->rename_xref(DEPEND_2, DEPEND_1);
    }
    if(ts_out->xref_exists(LABEL_2))
    {
      ts_out->rename_xref(LABEL_2, LABEL_1);
    }
 }
}          // end if on rank 2

else if (mRank == 3){
// dimensional metadata must be handled by Depend_i or Label_i
// watch for Depend_i == index i-1

  if( ii == 2) {
    if(ts_out->xref_exists(DEPEND_3)){
      ts_out->delete_xref(DEPEND_3);
    }
    // leave DEPEND_1,2 untouched
    
    if(ts_out->xref_exists(LABEL_3)){
      ts_out->delete_xref(LABEL_3);
    }
    // leave LABEL_1,2 untouched
  }
  else if (ii == 1){
    // delete it and shuffle indices down
    if(ts_out->xref_exists(LABEL_2))ts_out->delete_xref(LABEL_2);
    if(ts_out->xref_exists(DEPEND_2))ts_out->delete_xref(DEPEND_2);
    
    if(ts_out->xref_exists(DEPEND_3))
    {
      ts_out->rename_xref(DEPEND_3, DEPEND_2);
    }
    if(ts_out->xref_exists(LABEL_3))
    {
      ts_out->rename_xref(LABEL_3, LABEL_2);
    }
  }
  else if (ii == 0){
    // delete it and shuffle indices down
    if(ts_out->xref_exists(LABEL_1))ts_out->delete_xref(LABEL_1);
    if(ts_out->xref_exists(DEPEND_1))ts_out->delete_xref(DEPEND_1);

    if(ts_out->xref_exists(DEPEND_2))
    {
      ts_out->rename_xref(DEPEND_2, DEPEND_1);
    }
    if(ts_out->xref_exists(LABEL_2))
    {
      ts_out->rename_xref(LABEL_2, LABEL_1);
    }
        
    if(ts_out->xref_exists(DEPEND_3))
    {
      ts_out->rename_xref(DEPEND_3, DEPEND_2);
    }

    if(ts_out->xref_exists(LABEL_3))
    {
      ts_out->rename_xref(LABEL_3, LABEL_2);
    }
  }
  
} // end if on rank 3


//	..add  the output series to the list

  call_list->push_back(ts_out);     

QplugAppendTextDisplay ("Array Sum>> data object created - exiting OK\n");

return QPLUG_SUCCESS;

}
