// -----------------------------------------------------------------------------------------
//
// Various utility functions for the generic residue plugin
// 
//
// SEH, 12 Sep 2003
//
// -----------------------------------------------------------------------------------------

#include "qplug_if.h"
#include "qdutil.h"
#include "Qdos.h"
#include "qar.h"
#include "GenericMinVar.h"
#include "GenericResidue.h"



// -----------------------------------------------------------------------------------------
//
// Returns the weights according to Eq 44 in Sonnerup et al, JGR 2005
//
// SEH, Jul 13, 2005 - see Mail from BS, Sat 9 Jul 2005, 22:12:57
// 
// -----------------------------------------------------------------------------------------

double weight_k_calc(QdRMatrix_var Evalues)
{
  if ((double)(*Evalues)[2] == 0.0) return(1.0);
  double lower =  (double)(*Evalues)[0] * sqrt( (double)(*Evalues)[2] * (double)(*Evalues)[1] );
  double w_k = ( (double)(*Evalues)[2] -  (double)(*Evalues)[0] ) * ( (double)(*Evalues)[1] -  (double)(*Evalues)[0] )  / lower;
  return(w_k);
}




double aveTotalPressure(QdRMatrixSeq_var B, QdRMatrixSeq_var P, int nestsize)
{
  int j, k;
  double p,p_ave=0.00;
  double Mu0  = 12.566370614e-07;  

  QdRScalarSeq_var totP = new QdRScalarSeq(nestsize);
  QdRMatrix_var    B_k  = new QdRMatrix(Dim(3), 0.0);
  QdRMatrix_var    p_k  = new QdRMatrix(Dim(3), 0.0);

  for (k=0; k<nestsize; k++) {
    p = 0.0;
    for (j=0; j<3; j++) {
      p += ((double)(*P)[k][j][j])/3.0 * 1.60219E-13;                                // scalar pressure = trace(P)/3 [ kg/ms^2 ]
      (*B_k)[j] =  (*B)[k][j] * 1.0E-9;                                              // B field vec [T]
    }
    p_ave += p/3.0 + vecsum2(B_k)/(2.0*Mu0);
  }
  p_ave /= (double) nestsize;
  return(p_ave);
}
  
  




void Report(QdRMatrix_var Evalues, QdRMatrix_var Evec,  bool outstd, const char *msg, int constrain, double alpha,char *SCc_ID, const bool changeOrder)
{
  char buf[255],extended_msg[255],constrain_msg[255];
  int order[3]={0,1,2};
  if (changeOrder) {
    order[0]=2;
    order[2]=0;
  }

  switch (constrain) {
  case 0 : 
    sprintf(buf,"\n# ------ %s for SC%s   ------\n%s_SC%s \n",msg,SCc_ID,msg,SCc_ID);
    break;
  case 3:
    sprintf(extended_msg, "%sC%d for SC%s #walenslope = %5.3f",msg,constrain,SCc_ID,alpha);
    sprintf(constrain_msg,"%sC%d_SC%s",msg,constrain,SCc_ID);
    sprintf(buf,"\n# ------ %s ------\n%s\n",extended_msg,constrain_msg);
    break;
  default:
    sprintf(extended_msg,"%sC%d",msg,constrain);
    sprintf(buf,"\n# ------ %s for SC%s   ------\n%s_SC%s \n",extended_msg,SCc_ID,extended_msg,SCc_ID);
    break;
  }

  QplugAppendTextDisplay(buf);
  if (outstd)
  {
	printf("%s",buf);
	fflush(stdout);
  }
  for (int i=0; i<3; i++){
    sprintf(buf,"%12.6g  [%10.6g, %10.6g, %10.6g]\n", 
	  (double)(*Evalues)[order[i]],
	  (double)(*Evec)[order[i]][0],
	  (double)(*Evec)[order[i]][1], 
	  (double)(*Evec)[order[i]][2]);
    QplugAppendTextDisplay(buf);
    if (outstd)
	{
		printf("%s",buf);
		fflush(stdout);
	}
  }
}




void ReportSiscoe(QdRMatrixSeq_var q_n, QdRMatrixSeq_var U, 
		 QdRMatrix_var Bdave, QdRMatrix_var Vdave, QdRMatrix_var Vadave, 
		 int nestsize, const char *msg, int constrain, double alpha, char *SCc_ID)
{
  int i,j,k;
  QdRMatrix_var Evec = new QdRMatrix(Dim(3,3), 0.0);  
  QdRMatrix_var Q = new QdRMatrix(Dim(3,3), 0.0);  
  QdRMatrix_var Evalues = new QdRMatrix(Dim(3), 0.0);
  char buf[255];
  char SC_ID_loops[255];

  for (k=0; k<nestsize; k++) {
    for (i=0; i<3; i++) {
      for (j=0; j<3; j++) {
	(*Q)[i][j] += (double) (*q_n)[k][i][j] / (double) nestsize;
      }
    }
  }
  if (constrain ==3) {
    sprintf(SC_ID_loops,"%s (%d) # walenslope = %5.3f",SCc_ID,nestsize,alpha);
  } else sprintf(SC_ID_loops,"%s (%d)",SCc_ID,nestsize);

  sprintf(buf,"%s",msg);
  try{
    GenericMinVar minvar(0,*Q,*Q);                          // unconstrained, so ProjMx does not play any role
    *Evec = minvar.eigenvectors().duplicate(SHALLOW);
    *Evalues = minvar.eigenvalues().duplicate(SHALLOW); 
  }
  catch(Exception& e)     {return; }
 
  QdRMatrix_var N_siscoe = new QdRMatrix(Dim(3), 0.0);
  for (i=0; i<3; i++) (*N_siscoe)[i] = (*Evec)[2][i];
  Report(Evalues, Evec, FALSE, buf, constrain, alpha, SC_ID_loops,true);



  //  -- return average of velocities ----


  QdRMatrix_var U_ave = new QdRMatrix(Dim(3), 0.0);

  for (k=0; k<nestsize; k++) {
    for (i=0; i<3; i++) {
      (*U_ave)[i] += (*U)[k][i] / (1000.0 * nestsize);    // to get U_ave in [km/s]
    }
  }


  double un = DotVec(U_ave,N_siscoe);
  double bn = DotVec(Bdave,N_siscoe);
  double vn = DotVec(Vdave,N_siscoe);
  double van = DotVec(Vadave,N_siscoe);
  
  sprintf(buf,"!<<B>>  = [%10.3f  %10.3f  %10.3f] (%10.3f) nT   (based on %d different nest sizes)\n", (double)(*Bdave)[0],(double)(*Bdave)[1],(double)(*Bdave)[2],bn,nestsize);
  QplugAppendTextDisplay(buf);
  //  printf("%s",buf);

  sprintf(buf,"!<<V>>  = [%10.3f  %10.3f  %10.3f] (%10.3f) km/s \n", (double)(*Vdave)[0],(double)(*Vdave)[1],(double)(*Vdave)[2],vn);
  QplugAppendTextDisplay(buf);
  //  printf("%s",buf);

  sprintf(buf,"!<<Va>> = [%10.3f  %10.3f  %10.3f] (%10.3f) km/s \n", (double)(*Vadave)[0],(double)(*Vadave)[1],(double)(*Vadave)[2],van);
  QplugAppendTextDisplay(buf);
  //  printf("%s",buf);

  sprintf(buf,"!<<U>>  = [%10.3f  %10.3f  %10.3f] (%10.3f) km/s \n", (double)(*U_ave)[0],(double)(*U_ave)[1],(double)(*U_ave)[2],un);
  QplugAppendTextDisplay(buf);

}



QdRMatrix_var AveVec(QdRMatrixSeq_var V, int nestsize)
  //
  // returns the average of a vector time series
  //
{
  int i,k;
  double normfac = (double) nestsize;

  QdRMatrix_var A = new QdRMatrix(Dim(3), 0.0);                   
  
  for (k=0; k<nestsize; k++) {
    for (i=0; i<3; i++) (*A)[i] += (*V)[k][i];
  }

  for (i=0; i<3; i++) (*A)[i] /= normfac;
  return (A);
}




QdRMatrix_var CrossVec(QdRMatrix_var V, QdRMatrix_var R)
  //
  //  Returns the cross product of two vectors
  //
{
  QdRMatrix_var A = new QdRMatrix(Dim(3), 0.0);                   

  (*A)[0] = (*V)[1] * (*R)[2]  - (*V)[2] * (*R)[1];
  (*A)[1] = (*V)[2] * (*R)[0]  - (*V)[0] * (*R)[2];
  (*A)[2] = (*V)[0] * (*R)[1]  - (*V)[1] * (*R)[0];

  return (A);
}


double DotVec(QdRMatrix_var V, QdRMatrix_var R)
  //
  //  Returns the dot (scalar) product of two vectors
  //
{
  double d=0.0;
  for (int i=0; i<3; i++) d += (double)(*V)[i] * (double)(*R)[i];;

  return (d);
}


QdRMatrix_var ScaleVec(double d, QdRMatrix_var V)
  //
  //  Returns the vector V scaled with the double d
  //
{
  QdRMatrix_var A = new QdRMatrix(Dim(3), 0.0);                   
  for (int i=0; i<3; i++) (*A)[i] = d*(double)(*V)[i];
  return (A);
}


double vecsum(QdRMatrix_var Vector)
  //
  // returns the norm of a vector
  //
{
  double sum=0.0;

  for (int i=0; i<3; i++) sum += (*Vector)[i] * (*Vector)[i];

  return(sqrt(sum));
}



double vecsum2(QdRMatrix_var Vector)
  //
  // returns the norm^2 of a vector
  //
{
  double sum=0.0;

  for (int i=0; i<3; i++) sum += (*Vector)[i] * (*Vector)[i];

  return(sum);
}


double unit2SI(QdObject_var Obj)
  // 
  // returns the conversion factor to get to SI units
  // e.g. if SI_conversion = "1.0e-9>T", the double 1E-9 is returned
  //
{
  char *buf,*endptr;
  double cf;

  printf("In units2Si...\n");
  printf("%s|\n",QuGetAttrText("SI_conversion",(QdObject_var) Obj));

  printf("In units2Si...\n");
  sprintf(buf,"%s",QuGetAttrText("SI_conversion",Obj));
  printf("In units2Si...\n");
  printf("In units2Si > buf = %s, cf = %E\n",buf,cf);
  fflush(stdout);

  if (cf == 0) return(1.00);
  return(cf);
}



QdRMatrix_var LeviCivita()
  // 
  // returns a LeviCivita 3x3x3 matrix w
  // 
  //
{
  QdRMatrix_var LC =  new QdRMatrix(Dim(3,3,3), 0.0);

  (*LC)[0][1][2] = 1.0;
  (*LC)[2][0][1] = 1.0;
  (*LC)[1][2][0] = 1.0;
  (*LC)[2][1][0] = -1.0;
  (*LC)[0][2][1] = -1.0;
  (*LC)[1][0][2] = -1.0;

  return(LC);
}
 

void ToSpreadsheet(const char *filename, QdRMatrix_var Normal, QdRMatrix_var U, int row,  bool nl)
  // 
  // writes spreadsheet like output (CSV) for inclusion in spreadsheets
  // 
  //
{

  double un = 0.0;
  for (int i=0; i<3; i++) un += (double)(*U)[i] * (double)(*Normal)[0][row] / 1e3;

  FILE *f1 = fopen(filename,"a");
  fprintf(f1,"%g , %g , %g , ",(double)(*Normal)[row][0], (double)(*Normal)[row][1], (double)(*Normal)[row][2]);
  fprintf(f1,"%g , %g , %g , %g ,",(double)(*U)[0]/1e3,(double)(*U)[1]/1e3,(double)(*U)[2]/1e3, un);
  if (nl) fprintf(f1,"\n");
  fclose(f1);
}
  


int main()
  // do nothing
{
}




// -----------------------------------------------------------------------------------------
//
// Calculate standard deviation of a vector V dotted into a normal 
//
// SEH, 12 Jun 2003
//
// -----------------------------------------------------------------------------------------

double sigma(QdRMatrixSeq_var V, QdRMatrix_var normal, int n)
{
 
  double vn, vnave=0.00, sumv = 0.00;
  int i,j;
  for (i=0; i<n; i++) for (j=0; j<3; j++) vnave += (double)(*V)[i][j] * (double)(*normal)[0][j];
  vnave /= (double) n;

  for (i=0; i<n; i++) {
    vn = 0.00;
    for (j=0; j<3; j++) vn += (double)(*V)[i][j] * (double)(*normal)[0][j];
    sumv += ((vn - vnave) * (vn - vnave));
  }
  sumv /= (double)(n-1);
  return(sqrt(sumv));
}
 

	       
// -----------------------------------------------------------------------------------------
//
// Calculate error according to 8.23. and  8.24 in blue book
//
// SEH, 12 Jun 2003
//
// -----------------------------------------------------------------------------------------

double err_824(QdRMatrix_var B_ave, QdRMatrix_var Evec, QdRMatrix_var Eigenvalues, QdRMatrix_var dYij,  
               QdRMatrix_var M_MVA, double sigma, int nestsize, int modify)
{
  int i, j;
  double Bx1 = 0.00, Bx2 = 0.00, dBx3 = 0.00, l3=0.0;
  
  for (i=0; i<3; i++) {
    for (j=0; j<3; j++) {
      (*dYij)[i][j] = sqrt((double)(*Eigenvalues)[0]/((double)(nestsize-1)) * 
			   ((double)(*Eigenvalues)[i]+ (double)(*Eigenvalues)[j] - (double)(*Eigenvalues)[0]) /
			   (((double)(*Eigenvalues)[i] - (double)(*Eigenvalues)[j]) * 
			    ((double)(*Eigenvalues)[i] - (double)(*Eigenvalues)[j])));
    }
  }
  
  for (i=0; i<3; i++) {
    Bx2 += (double)(*dYij)[0][1] * (double) (*B_ave)[i] * (double) (*Evec)[1][i];
    Bx1 += (double)(*dYij)[0][2] * (double) (*B_ave)[i] * (double) (*Evec)[2][i];
  }

  // how to calculate the first term in 8.24 :

  switch(modify) {
  case 0:
    l3 = (double)(*Eigenvalues)[0] / ((double)(nestsize - 1));
    break;
  case 1:
    for (i=0; i<3; i++) for (j=0; j<3; j++) l3 += (double)(*Evec)[0][i] * (double)(*M_MVA)[i][j] *  (double)(*Evec)[0][i];
     l3 /= ((double)(nestsize - 1));
    break;
  case 2:
    l3 = sigma;
    break;
  case 3:
    l3 = 0.00;
    break;
  default:
    l3 = (double)(*Eigenvalues)[0] / ((double)(nestsize - 1));
    break;
  }

  dBx3 = sqrt(l3  + (Bx1*Bx1) + (Bx2*Bx2));
  return(dBx3);      
}


double jumpvalue(QdRMatrixSeq_var V, const int n)
{
  double before=0.00,after=0.00;
  int j = V->sequence_size()-1; 
  for (int i=0; i<n; i++) {
    before+=sqrt( (double)(*V)[i][0]* (double)(*V)[i][0] + (double)(*V)[i][1]* (double)(*V)[i][1]  + (double)(*V)[i][2]* (double)(*V)[i][2]);
    after +=sqrt( (double)(*V)[j-i][0]* (double)(*V)[j-i][0] + (double)(*V)[j-i][1]* (double)(*V)[j-i][1]  + (double)(*V)[j-i][2]* (double)(*V)[j-i][2]);
  }
  return(before/after);
}

double jumpvalue(QdRScalarSeq_var S, const int n)
{

  if (S.is_nil()) return(-9.9e-31);
  double before=0.00,after=0.00;
  int j = S->sequence_size()-1; 
  for (int i=0; i<n; i++) {
    before+=sqrt( (double)(*S)[i]);
    after +=sqrt( (double)(*S)[j-i]);
  }
  return(before/after);
}


QdRMatrix_var Q_GEN_calc(QdRMatrixSeq_var TS)
{
  int i,j,k;
  QdRMatrix_var Q  = new QdRMatrix(Dim(3,3), 0.00);
  QdRMatrix_var aveTS  = new QdRMatrix(Dim(3), 0.0);

  int nestsize = TS->sequence_size();
  double normfac = (double) nestsize;
 
  QdRMatrixSeq_var dTS  = new QdRMatrixSeq(nestsize,RMatrix(Dim(3)));

  for (k=0; k<nestsize; k++) {              // find <TS>
    for (i=0; i<3; i++) {
      (*aveTS)[i] += (*TS)[k][i] / normfac;;
    }
  }

  for (k=0; k<nestsize; k++) {             // find delta TS
    for (i=0; i<3; i++) {
      (*dTS)[k][i] = (double)(*TS)[k][i] - (double)(*aveTS)[i];
    }
  }

  for (k=0; k<nestsize; k++) {
    for (i=0; i<3; i++) {
      for (j=0; j<3; j++) (*Q)[i][j] += (*dTS)[k][i] * (*dTS)[k][j] / normfac;
    }
  }

  return(Q);
}
