
// -----------------------------------------------------------------------------------------
//
// Calculate Q-matrix for MER - minimum entropy residue
//
// SEH, Sep 7, 2003
//  
// Inputs :
//     B = magnetic field  (n x 3)     [nT]
//     V = plasma velocity (n x 3)     [km/s]
//     P = pressure tensor (n x 3 x 3) [eV/cm^3]
//     N = plasma density  (n)         [1/cm^3]  (proton mass used to get mass density)
//
// For simplification, all units are converted to SI units
//
// -----------------------------------------------------------------------------------------

#include "Qdos.h"
#include "GenericResidue.h"


// -----------------------------------------------------------------------------------------
//
// Returns the Q_matrix for MER (Minimum Energy Residue)
//
// SEH, 19 Sep, 2001
//      15 Oct, 2003, Doxygen DOC
//
// Doxygen info :
/*! 

  \brief Returns the covariance matrix for the Minimum Energy Residue 
  method.

  \param V - vector time series with plasma velocity in units of km/s
  \param B - vector time series with the magnetic field in units of nT
  \param P - matrix time series with the full 3x3 pressure tensor
  \param N - scalar time series with the plasma density in units of 1/cm3
  \param U - vector - holds the calculated velocity in units of km/s on successful return.
  \param nestsize - number of elements in V,B,P,and N.

  \warning If the density is constant, there will be a sigularity in the 
  U calculation. U is then replaced by <V> - the average plasma
  velocity.
*/ 
// -----------------------------------------------------------------------------------------
QdRMatrix_var Q_MER_calc(QdRMatrixSeq_var V, QdRMatrixSeq_var B, QdRMatrixSeq_var P,
			 QdRScalarSeq_var N, QdRMatrix_var U, int nestsize)
{
  int i,j,k,m;
  double mu0 = 12.566370614e-07;             // magnetic constant [N/A^2 == H/m]
  double m_p = 1.67e-27;                     // mass of proton [kg] (we assume ions are == protons)
  double normfac = (double) nestsize;
  double p,rho,ave_eta=0.00,d_eta2=0.00;
  bool PO = true;                           // PrintOut to screen ? 

  QdRMatrix_var    V_k = new QdRMatrix(Dim(3), 0.0);                    // holds the V(k) vector
  QdRMatrixSeq_var q_i = new QdRMatrixSeq(nestsize,RMatrix(Dim(3)));
  QdRMatrix_var    ave_q_i = new QdRMatrix(Dim(3), 0.0);
  QdRScalarSeq_var eta = new QdRScalarSeq(nestsize);
  QdRMatrix_var    Q =  new QdRMatrix(Dim(3,3), 0.0);


  for (k=0; k<nestsize; k++) {
    p = 0.0;
    for (i=0; i<3; i++) {
      (*V_k)[i] = (*V)[k][i] * 1E3;                     
      p += ((double)(*P)[k][i][i])/3.0 * 1.609E-13;                    // scalar pressure [ kg/ms^2 ]
    }
    rho = m_p * (*N)[k] * 1E6;                                        // 1E6 -> to get SI units kg/m^3 
    (*eta)[k] = rho * log(p / pow(rho,(5.0/3.0)));
    ave_eta += (double)(*eta)[k];
    for (i=0; i<3; i++) {                                             // loop over three components
      (*q_i)[k][i] = (*eta)[k] *  (*V_k)[i];             
      (*ave_q_i)[i] += (*q_i)[k][i];
    }
  }
  


  // ----- initialize U, get <q_i> and <eta>--------

  for (i=0; i<3; i++) {
    (*ave_q_i)[i] /= normfac;
    (*U)[i] = 0.0;
  }
  ave_eta /= normfac;

  if (PO) printf("Q_MER:\nq_i vector [kg/(m^2 s)] : \n");
  if (PO) printf("<q_j>     = [%E  %E  %E]; \n",(double)(*ave_q_i)[0], (double)(*ave_q_i)[1], (double)(*ave_q_i)[2]);
  if (PO)
  {
	printf("\n<eta>      = %E [kg/m^3]\n", ave_eta);
	fflush(stdout);
  }

  // ---   get delta_q_i = q_i - <q_i> and delta_eta --------

  for (k=0; k<nestsize; k++) {
    for (i=0; i<3; i++) (*q_i)[k][i] -= (*ave_q_i)[i];
    (*eta)[k] -= ave_eta;
    d_eta2 += ((double) (*eta)[k] * (double) (*eta)[k]);
  }


  if (PO)
  {
	printf("\n<|deta|^2> = %E  [kg^2/(m^2s^4)]\n", d_eta2);
	fflush(stdout);
  }

  // ---- construct U* vector ---------------

  if (d_eta2 != 0.0) {
     for (k=0; k<nestsize; k++) {
       for (i=0; i<3; i++) (*U)[i] += (double)(*eta)[k] * (double)(*q_i)[k][i] / d_eta2; 
      }  
  } else {
     printf("MER : singularity - setting U* = <V_cis>\n");
	 fflush(stdout);
     for (k=0; k<nestsize; k++) {
       for (i=0; i<3; i++) (*U)[i] +=  (double)(*V)[k][i] * 1E3 / normfac;       
      }
  }

  if (PO)
  {
	printf("\nU*         = [%E  %E  %E];  [m/s] (eq 12 in manuscript)\n\n", (double)(*U)[0], (double)(*U)[1], (double)(*U)[2]);
	fflush(stdout);
  }


  // --- construct Q_ij matrix --------------  

  for (k=0; k<nestsize; k++) {
    for (i=0; i<3; i++) {
      for (j=0; j<3; j++) {
	(*Q)[i][j] += ((*q_i)[k][i] - (*U)[i]*(*eta)[k]) * ((*q_i)[k][j] - (*U)[j]*(*eta)[k]);
      }
    }
  }
  

  for (i=0; i<3; i++) {
    for (j=0; j<3; j++) {
      (*Q)[i][j] /= normfac;
    }
  }

  if (PO)printf("Q_ij  [kg/(m^2 s)] (eq 15 in manuscript)\n");
  for (i=0; i<3; i++) {
    if (PO) printf("Q_i%d       = [%E  %E  %E];\n",i, (double)(*Q)[i][0], (double)(*Q)[i][1], (double)(*Q)[i][2]);
	fflush(stdout);
  } 

  return (Q);
}

