// -----------------------------------------------------------------------------------------
//
// Calculate Q-matrix for Minimum Faraday Residue
//
// Inputs :
//     B = magnetic field  (n x 3)     [nT]
//     V = plasma velocity (n x 3)     [km/s]
//     N = plasma density  (n)         [1/cm^3]  (proton mass used to get mass density)
//
// For simplification, all units are converted to SI units
//
//
// SEH, Mar 15, 2003 
//
// -----------------------------------------------------------------------------------------

#include "Qdos.h"
#include "GenericResidue.h"

QdRMatrix_var Q_MFR_calc(QdRMatrixSeq_var B, QdRMatrixSeq_var V, QdRMatrix_var U, int nestsize)
{
 int i,j,k,m;
 double d_eta2=0.00;
 double normfac = (double) nestsize;
 double m_p = 1.67e-27;                     // mass of proton [kg] )we assume ions are == protons)
 bool PO = false;                           // PrintOut to screen ? 

 QdRMatrixSeq_var eta     = new QdRMatrixSeq(nestsize,RMatrix(Dim(3), 0.0));
 QdRMatrix_var    E_k     = new QdRMatrix(Dim(3), 0.0);                    // holds the E(k) vector
 QdRMatrix_var    V_k     = new QdRMatrix(Dim(3), 0.0);                    // holds the V(k) vector
 QdRMatrix_var    B_k     = new QdRMatrix(Dim(3), 0.0);                    // holds the B(k) vector
 QdRMatrix_var    ave_eta = new QdRMatrix(Dim(3), 0.0);
 QdRMatrixSeq_var q_ij    = new QdRMatrixSeq(nestsize,RMatrix(Dim(3,3), 0.00));
 QdRMatrix_var    ave_q_ij = new QdRMatrix(Dim(3,3), 0.0);
 QdRMatrix_var Q   = new QdRMatrix(Dim(3,3), 0.0);
 QdRMatrix_var LC  = LeviCivita();


 // --- construct q_i and eta -----

 for (k=0; k<nestsize; k++) {
   for (i=0; i<3; i++) {
      (*V_k)[i] = (*V)[k][i] * 1.0E3;                                               // V vektor in [m/s]
      (*B_k)[i] = (*B)[k][i] * 1.0E-9;                                              // B field vec [T]  
   }
   E_k = CrossVec(B_k,V_k);
   for (i=0; i<3; i++) {
     (*eta)[k][i] = (*B_k)[i];
     (*ave_eta)[i] += (*eta)[k][i];
     for (j=0; j<3; j++) {
       for (m=0; m<3; m++) {
	 (*q_ij)[k][i][j] += (*LC)[i][j][m] * (*E_k)[m];
       }
       (*ave_q_ij)[i][j] += (*q_ij)[k][i][j];
     }
   }
 }


  // ----- get <q_ij> and <eta>--------

  for (i=0; i<3; i++) {
    for (j=0; j<3; j++) {
      (*ave_q_ij)[i][j] /= normfac;
    }
    (*ave_eta)[i] /= normfac;
    (*U)[i] = 0.0;
  }

  if (PO) printf("In Q_MFR:\nq_ij matrix [km/(m s^2)] : \n");
  if (PO) for (i=0; i<3; i++) {
    printf("<q_%dj>     = [%E  %E  %E];\n",i,(double)(*ave_q_ij)[i][0], (double)(*ave_q_ij)[i][1], (double)(*ave_q_ij)[i][2]);
  } 

  if (PO)
  {
	printf("\n<eta>      = [%E  %E  %E]  [kg/(m^2 s)];\n", (double)(*ave_eta)[0], (double)(*ave_eta)[1], (double)(*ave_eta)[2]);
	fflush(stdout);
  }


  // ---   get delta_q_ij = q_ij - <q_ij> --------

  for (k=0; k<nestsize; k++) {
    for (i=0; i<3; i++) {
      for (j=0; j<3; j++) {
	(*q_ij)[k][i][j] -= (*ave_q_ij)[i][j];
      }
      (*eta)[k][i] -= (*ave_eta)[i];
      d_eta2 += ((double) (*eta)[k][i] * (double) (*eta)[k][i]);
    }
  }

  if (d_eta2 == 0.0) d_eta2 = 1e-99;   // avoid singularity
 
  //  printf("\n<|deta|^2> = [%E  %E  %E];\n", (double)(*d_eta2)[0], (double)(*d_eta2)[1], (double)(*d_eta2)[2]);
 if (PO)
 {
	printf("\n<|deta|^2> = %E  [kg^2/(m^2s^4)]\n", d_eta2);
	fflush(stdout);
 }



  // ---- construct U* vector ---------------

  for (k=0; k<nestsize; k++) {
    for (j=0; j<3; j++) {
      for (i=0; i<3; i++) {
	(*U)[j] += (*eta)[k][i] * (*q_ij)[k][i][j];
      }
    }
  }

  for (i=0; i<3; i++) (*U)[i] /= d_eta2;
  if (PO)
  {
	printf("\nU*         = [%E  %E  %E] [m/s];  (eq 12 in manuscript)\n\n", (double)(*U)[0], (double)(*U)[1], (double)(*U)[2]);
	fflush(stdout);
  }


  // --- construct Q_ij matrix --------------  

  for (k=0; k<nestsize; k++) {
    for (i=0; i<3; i++) {
      for (j=0; j<3; j++) {
	for (m=0; m<3; m++) {
	  (*Q)[i][j] += ((*q_ij)[k][m][i] - (*U)[i]*(*eta)[k][m]) * ((*q_ij)[k][m][j] - (*U)[j]*(*eta)[k][m]);
	}
      }
    }
  }

  for (i=0; i<3; i++) {
    for (j=0; j<3; j++) {
      (*Q)[i][j] /= normfac;
    }
  }

  if (PO) printf("Q_ij [km/(m s^2)] (eq 15 in manuscript)\n");
  if (PO) for (i=0; i<3; i++) {
    printf("Q_i%d       = [%E  %E  %E];\n",i, (double)(*Q)[i][0], (double)(*Q)[i][1], (double)(*Q)[i][2]);
	fflush(stdout);
  } 

  return(Q);
}




