
// -----------------------------------------------------------------------------------------
//
// Calculate Q-matrix for MVA of any vector Quantity
//
// SEH, Mar 27, 2003
//      Sep 13, 2003  - cloned fromm massflow and modified 
//
// Inputs :
//     B = any vector time series, e.g., magnetic field   (n x 3)     
//
// -----------------------------------------------------------------------------------------

#include "Qdos.h"
#include "qplug_if.h"
#include "qdutil.h"

#include "GenericMinVar.h"
#include "GenericResidue.h"

// -----------------------------------------------------------------------------------------
//
// Returns the Q_matrix for MVAB (Minimum Variance of B-field)
//
// SEH, 07 Sep, 2001
//      15 Oct, 2003, Doxygen DOC
//      21 Nov, 2003, Walen Slope + CC as output
//
// Doxygen info :
/*! 
  \brief Calculates the covariance matrix of the magnetic field.
  \param B -  vector time series with the magnetic field in units of nT
  \param V -  vector time series with plasma velocity in units of km/s
  \param U -  holds the calculated velocity on successful return
  \param nestsize - number of elements in B and V.
  \param full - 0 to do MVAB only, 1 to include HT analysis

*/ 

QdRMatrix_var Q_MVAB_calc(QdRMatrixSeq_var B, QdRMatrixSeq_var V, QdRMatrix_var U, QdRMatrixSeq_var Valf,
			  QdRScalarSeq_var N, double *wl, double *cc, int nestsize, bool doHT)
{
  printf("Q_MVAB_calc...nestsize = %d B=NULL : %d   %d\n",nestsize, (int)B.is_nil(), B->sequence_size());
  fflush(stdout);
  int i,j,k;
  double normfac = (double) nestsize;
  QdRMatrix_var Q  = new QdRMatrix(Dim(3,3), 0.00);
  QdRMatrix_var aveB  = new QdRMatrix(Dim(3), 0.0);
  QdRMatrixSeq_var dB  = new QdRMatrixSeq(nestsize,RMatrix(Dim(3)));
  QdTimeSeq_var tt = get_timetags((QdObject_var)B);       // tt of joined V and B
  QdTime timetag;
  bool PO = false;                           // PrintOut to screen ? 

  for (k=0; k<nestsize; k++) {              // find <B>
    for (i=0; i<3; i++) {
      (*aveB)[i] += (*B)[k][i] / normfac;;
    }
  }

  if (PO)
  {
	printf("Q_MVAB : ave B : [%E  %E  %E]\n",(double)(*aveB)[0], (double)(*aveB)[1], (double)(*aveB)[2]);
	fflush(stdout);
  }
  for (k=0; k<nestsize; k++) {             // find delta B
    for (i=0; i<3; i++) {
      (*dB)[k][i] = (double)(*B)[k][i] - (double)(*aveB)[i];
    }
  }

  for (k=0; k<nestsize; k++) {
    for (i=0; i<3; i++) {
      for (j=0; j<3; j++) (*Q)[i][j] += (*dB)[k][i] * (*dB)[k][j] / normfac;
    }
   }

// --- SEH 2004-12-04 ---
//
// This gives a Q-matrix as Q=<Bi Bj> instead of <Bi Bj> - <Bi><Bj>
// suggested by BS, tested by SEH/BS 4 Dec 2004, but only minor improvement.
// -> do not implement
//
//    for (k=0; k<nestsize; k++) {
//     for (i=0; i<3; i++) {
//       for (j=0; j<3; j++) (*Q)[i][j] += (double)(*B)[k][i] * (double)(*B)[k][j] / normfac;
//     }
//   }
// --------------

  if (!doHT) return(Q);

  bool doB = !B.is_nil() && ( B->sequence_size() > 0 );
  bool doV = !V.is_nil() && ( V->sequence_size() > 0 );
  bool doN = !N.is_nil() && ( N->sequence_size() > 0 );
  
  // ----- otherwise, proceed with HT analysis ------

  QdRMatrix_var aveV  = new QdRMatrix(Dim(3), 0.0);
  QdRMatrix_var Vht  = new QdRMatrix(Dim(3), 0.0);
  QdRMatrixSeq_var V_rest   = new QdRMatrixSeq(nestsize,RMatrix(Dim(3)));
  Valf = new QdRMatrixSeq(nestsize,RMatrix(Dim(3)));  

  for (k=0; k<nestsize; k++) {              // find <V>
    for (i=0; i<3; i++) {
      (*aveV)[i] += (*V)[k][i] / normfac;;
    }
  }
  if (PO)printf("Q_MVAB : ave V : [%E  %E  %E]\n",(double)(*aveV)[0], (double)(*aveV)[1], (double)(*aveV)[2]);

  if (PO)printf("Raedy for HT..%d %d %d\n",  B->sequence_size(), V->sequence_size(), nestsize);
  Vht = V_HofTel(B,V,nestsize);
  for (i=0; i<3; i++) (*U)[i] = (*Vht)[i];

  double mu_0 = 1.25664e-06;     // modified mu_0 (due to B in nT and dens in 1/ccm)
  double rho,m_p= 1.67e-27;      // rho = mass dens with proton_mass m_p

  if (doN) {
    for (i=0; i<nestsize; i++) {
      rho = (double)(*N)[i] * 1e06 * m_p;
      for (j=0; j<3; j++) {
	V_rest[i][j] =  (double) (*V)[i][j] - (double)(*Vht)[j]*1e-3;             // 1e-3 to work in km/s
	Valf[i][j] =   1.0e-3 * (double) (*B)[i][j] * 1e-9 / sqrt(mu_0 * rho);   // 1e-3 to work in km/s
      }
      //    printf("Q_MVAB_calc  Vr=[%f   %f   %f]",(double)(*V_rest)[i][0], (double)(*V_rest)[i][1], (double)(*V_rest)[i][2]);
      //    printf("   Valf=[%f   %f   %f]\n",(double)(*Valf)[i][0], (double)(*Valf)[i][1], (double)(*Valf)[i][2]);
    }  
    
    
    double cctmp = correlationCoeff(Valf,V_rest,nestsize);
    double wltmp = slope(Valf,V_rest,nestsize);
    
    if (PO)
	{
		printf("Q_MVAB_calc > WalenSlope, Corrcoeff  = %f  %f, U=[%f   %f   %f]\n",wltmp,cctmp,(double)(*U)[0], (double)(*U)[1], (double)(*U)[2]);
		fflush(stdout);
	}
    
    (*cc) = cctmp;
    (*wl) = wltmp; 
  }
  return (Q);
}






// -----------------------------------------------------------------------------------------
//
// Returns the V_HT frame
//
// SEH, Nov, 2001
//      May, 2002
//      Apr, 2003 (grabbed from HT Plugin, modified here)
//      19 Sep 2003, grabbed from Massflow plugin - simplified and modified here.
//
// Doxygen info :
/*! 
  \brief Returns the DeHoffmann-Teller velocity for the given magnetic 
  field and plasma velocity. The electric field used in the calculation
  is taken from the proxy E = - V x B.
.
  \param B -  vector time series with the magnetic field in units of nT
  \param V -  vector time series with plasma velocity in units of km/s
  \param nestsize - number of elements in B and V.

*/ 
// -----------------------------------------------------------------------------------------

QdRMatrix_var V_HofTel(QdRMatrixSeq_var B, QdRMatrixSeq_var V, int nestsize)
{
  int i,j,k,l;
  double bb, time=0.0, normfac=(double) nestsize;

  //  QdRMatrixSeq_var Vht_x_B = new QdRMatrixSeq(nestsize,RMatrix(Dim(3)));
  //  QdRMatrixSeq_var V_x_B   = new QdRMatrixSeq(nestsize,RMatrix(Dim(3)));
  // QdRMatrixSeq_var V_rest   = new QdRMatrixSeq(nestsize,RMatrix(Dim(3)));
  // QdRMatrixSeq_var V_alf = new QdRMatrixSeq(nestsize,RMatrix(Dim(3)));  
  QdRMatrix_var K = new QdRMatrix(Dim(3,3), 0.00);
  QdRMatrix_var Vht = new QdRMatrix(RMatrix(Dim(3), 0.00));
  QdRMatrix_var B_k = new QdRMatrix(RMatrix(Dim(3)));
  QdRMatrix_var K0 = new QdRMatrix(Dim(3,3), 0.00);
  QdRMatrix_var KV = new  QdRMatrix(RMatrix(Dim(3), 0.00));

  for(k=0; k<nestsize; k++)  {         
    for (j=0; j<3; j++) {
      (*B_k)[j] = (*B)[k][j];
    }
    bb = vecsum2(B_k);

    for (i=0; i<3; i++) {
      for (j=0; j<3; j++) {
	(*K)[i][j] = (double)(i==j) * bb - ((double)(*B_k)[i]*(double)(*B_k)[j]);
      }
    }
     
     for (i=0; i<3; i++) {
       for (j=0; j<3; j++) {
	 (*K0)[i][j] += (*K)[i][j];
       }
     }


     for (i=0; i<3; i++) {
       for (j=0; j<3; j++) {
	 (*KV)[i] += (*K)[i][j] * (*V)[k][j] * 1e3;
       }
     }

  }


  for (i=0; i<3; i++){
    for (j=0; j<3; j++) {
      (*K0)[i][j] /= normfac;                                   // K0 == <K>
    }
    (*KV)[i] /= normfac;
  }

  QdRMatrix_var Kinv = QdRMatrix_var::narrow(invert(K0));


  for  (i=0; i<3; i++) {
    for (j=0; j<3; j++) {
      (*Vht)[i] += (*Kinv)[i][j]*(*KV)[j]; 
    }
  }

  return (Vht);
}





double slope(QdRMatrixSeq_var X_in,QdRMatrixSeq_var Y_in,int n)
// ---------------------------------------------------------------------------
//
//  calculates the regression line, i.e. the coeff b in y(x) = a + bx
//
//
// SEH, 15 Aug, 2002
//      21 Nov 2003, grabbed from HofTel + converted to return a double
//
// ---------------------------------------------------------------------------

{
  int numpts = 3*n;

  ScalarSeq<double> Xc;   Xc.resize(numpts);
  ScalarSeq<double> Yc;   Yc.resize(numpts);
  // = new QdRScalarSeq(numpts);
  //  QdRScalarSeq_var Yc = new QdRScalarSeq(numpts);
  QdRScalarSeq_var Tc = new QdRScalarSeq(numpts);

  for (int i=0; i<n; i++) {
    Xc[i] = (float) (*X_in)[i][0];
    Xc[i+n] = (float) (*X_in)[i][1];
    Xc[i+2*n] = (float) (*X_in)[i][2];
    Yc[i] = (float) (*Y_in)[i][0];
    Yc[i+n] = (float) (*Y_in)[i][1];
    Yc[i+2*n] = (float) (*Y_in)[i][2];
  }
  
  float sum_X = Xc.sum();
  float sum_Y = Yc.sum();
  float ave_X = sum_X / (float) (numpts);
  float ave_Y = sum_Y / (float) (numpts);
  double  b=0.0, tt=0.0;

  for (int i=0; i<numpts; i++) {
    Tc[i] = (float) Xc[i] - ave_X;
    b += (double) Tc[i] * (float)Yc[i];
    tt += (double) (*Tc)[i] * (float) (*Tc)[i];
  }

  b /= tt;
  return(b);
}



double correlationCoeff(QdRMatrixSeq_var X_in,QdRMatrixSeq_var Y_in,int n)
// ---------------------------------------------------------------------------
//
//  calculates the correlation coefficient
//
//
// SEH, 15 Jan 2003 
//      21 Nov 2003, grabbed from HofTel + converted to return a double
//
// ---------------------------------------------------------------------------
{

  double sum_upper=0.0, sum_xx=0.0, sum_yy=0.0;
  int numpts = 3*n;

  ScalarSeq<double> Xc;   Xc.resize(numpts);
  ScalarSeq<double> Yc;   Yc.resize(numpts);
  QdRScalarSeq_var Tc = new QdRScalarSeq(numpts);

  for (int i=0; i<n; i++) {
    Xc[i] = (float) (*X_in)[i][0];
    Xc[i+n] = (float) (*X_in)[i][1];
    Xc[i+2*n] = (float) (*X_in)[i][2];
    Yc[i] = (float) (*Y_in)[i][0];
    Yc[i+n] = (float) (*Y_in)[i][1];
    Yc[i+2*n] = (float) (*Y_in)[i][2];
  }

  double sum_x = Xc.sum() / (double) (numpts);;
  double sum_y = Yc.sum() / (double) (numpts);;

  for (int i=0; i<numpts; i++) {
    sum_upper += (Xc[i] - sum_x) * (Yc[i] - sum_y);
    sum_xx += (Xc[i] - sum_x)*(Xc[i] - sum_x);
    sum_yy += (Yc[i] - sum_y)*(Yc[i] - sum_y);
  }

  return (double) (sum_upper / (sqrt(sum_xx) * sqrt(sum_yy)));
}

