// ------------------------------------------------------------------
//
// ISSI QSAS PlugIn : Constrained MVA
// 
// Performs MVA with or without a Ts * minvec = 0 constraint. Based on
// The MVA PlugIn by S.Schwartz
//
// Inputs :
// 	vts_in;		- vector time series or matrix
//      start_end;      - time interval
//      constraints     - 1 for constrain Bn=0, 0 else
//      pgdev_do        - output device # for PGPLOT
//      filename        - filename for output to *.ps file
//
// Outputs
//      ts_out          - ts_in rotated into the new c'sys
//      rotmx           - transformation matrix form ts_in -> ts_out
//
// SEH, 14 May, 2002
//      25 Aug, 2002, covariance matrix as output to WL
//      25 Aug, 2000, <TS> * minvec= 0 where <TS> may be a separate input
//      26 Feb, 2003, timerange, <TS> and <TS> * minvec as outputs 
//      07 Jan, 2004, QSAS 2.0 version
//      15 Jan, 2004, allow for a matrix as input
//      02 May, 2005, output projection matrix
//      08 Aug, 2005, allow selection of covariance matrix type
//      19 Aug, 2005, corrected minor bug in printout of Bn for Weimer type matrix
//      10 Oct, 2005, removed (redundant) link with lapack
//      03 Mar, 2009, removed PGPLOT calls, 
//      
//
// ------------------------------------------------------------------

#ifndef __cplusplus
#define __cplusplus
#endif

#include <stdio.h>
#include <math.h>
#include <string.h>
#include <stdlib.h>

#include "MinVar.h"
#include "ConstrainedMinVariance.h"


// needed for QSAS 2.0
extern "C" QplugReturnStatus  MinVarB0  ( QplugArgList *);

typedef struct VEC {float x,y,z;} vec;
double vecsum2(QdRMatrix_var V);
QdRMatrix_var Projection(QdRMatrix_var A);

using namespace QSAS;


QplugReturnStatus  MinVarB0  ( QplugArgList *arglist )

{

  // QDOS Objects:

  QdRMatrixSeq_var     vts_in;
  QdRMatrixSeq_var     vts_in2;
  QdRMatrixSeq_var     vts_out;
  QdRMatrixSeq_var     vts_out2;
  QdRMatrix_var	     Rmatx;
  QdRMatrix_var	     Proj_matx;
  QdRMatrix_var        Covar_matx;
  QdRMatrix_var	     Eigenvalues;
  QdTimeInterval_var   start_end;
  QdRScalar_var        pgdev_do,vts_in2i;
  QdRScalar_var        mvatype_do;
  QdString_var         pgname_do;

  const char * filename;
  char attrtxt[150];
  QString str;

  QplugReturnStatus     MV_return;  // used to return from sub-modules

  int    i,j,constrained,nseq=2,triage_cnt=0,Qsel=0;
  double deltat=1.0,avg_width=0.0;
  char   evalsevecs[80];
  char   SC__id[127] = "";

  QTime t;
  t.start();                     // start clock
 
  //	 get the input data objects

  QdObject_var in_0 = (*arglist)[0];
  vts_in = QdRMatrixSeq_var::narrow(in_0);
  strncpy(SC__id,QuGetAttrText("SC_id",(*arglist)[0]),1);		//get SC ID

  //  check whether the input is a covarience matrix - if yes - process here.

  if( vts_in.is_nil() ) {
    Matrix<double> covEigvals;       // Matrix Dim(3) of eigenvalues
    Matrix<double> covEigvecs;       // Matrix Dim(3,3) of eigenvectors (in rows)
    Matrix<double> covMatrix;      // covariance matrix
    printf("..Covariance matrix supplied...\n");
    QplugAppendTextDisplay("Covariance matrix supplied..\n");
    covMatrix.resize(Dim(3,3));
    Rmatx = new QdRMatrix(Dim(3,3), 0.00);  
    Covar_matx = new QdRMatrix(Dim(3,3), 0.00);  
    QdRMatrixSeq_var Empty_TS = new QdRMatrixSeq(1,RMatrix(Dim(3), 0.0));  
    QdRMatrix_var covMx_in = QdRMatrix_var::narrow(in_0);
    for (i=0; i<3; i++) for (j=0; j<3; j++) covMatrix[i][j] = (double) (*covMx_in)[i][j];
    for (i=0; i<3; i++) for (j=0; j<3; j++) covMatrix[i][j] = (double) covMatrix[j][i]; // ensure symmetry
    MatrixEig eig(covMatrix);
    covEigvecs=eig.eigenvectors().seq_to_matrix();
    covEigvals=eig.eigenvalues().seq_to_matrix();
    for (i=0; i<3; i++){
      for (j=0; j<3; j++) {
	(*Rmatx)[i][j] = (double) covEigvecs[i][j];
	(*Covar_matx)[i][j] = (*covMx_in)[i][j];
      }
      sprintf(evalsevecs,"%10.5f  [%10.5g, %10.5g, %10.5g ]\n", 
	      (double) covEigvals[i], (double) covEigvecs[i][0], 
	      (double) covEigvecs[i][1], (double) covEigvecs[i][2]);
      str += evalsevecs;
    }  
    QplugAppendTextDisplay (str.latin1());

    arglist->push_back(Empty_TS);     
    arglist->push_back(Rmatx);   
    arglist->push_back(Covar_matx);   
    arglist->push_back(Covar_matx);   
    return QPLUG_FAILURE;
  }

 
  // ---- check if input_TS is a tensor 

  Dim mDim = vts_in[0].dimdata();
  bool ts_is_tensor = mDim.rank() == 2;
  printf("Tensor input : %d  %d\n", mDim.rank(),ts_is_tensor);



  // check input is cartesian
  const char * frame_ptr;
  try {
    frame_ptr = QuGetRep(vts_in);
    printf("Checking frame is cartesian...\n");
    if(  !ts_is_tensor ){
      if( strstr(frame_ptr, "xyz") == NULL ){
		QplugAppendTextDisplay ("MinVar Failed >> vector is not cartesian\n");
		return QPLUG_FAILURE;
      }
    }
  } catch(Exception &e){
    e.print_msg();
    QplugAppendTextDisplay ("MinVar Failed >> Attribute FRAME not set ?\n");
    return QPLUG_FAILURE;   
  }



  // ---- get time interval ----------------

  printf("Time interval...\n");
  QdObject_var in_1 = (*arglist)[1];
  start_end = get_timeinterval(in_1);
  if( start_end.is_nil() ) return QPLUG_FAILURE;


  // ---- Constrained MVA; calculate so that ts2 * n = 0 + select Covariance matrix type --------

  printf("Checking for constrained...\n");
  constrained = 0;
  Qsel = 0;
  QdObject_var in_2 = (*arglist)[2];
  vts_in2 =  QdRMatrixSeq_var::narrow(in_2);
  if (vts_in2.is_nil() ) {
    vts_in2i =  QdRScalar_var::narrow(in_2);
    constrained = (int) (*vts_in2i);
    Qsel = constrained;
    vts_in2 = QdRMatrixSeq_var::narrow(in_0);                      // if not present, use input ts
    if (constrained == 0) printf("Unconstrained...\n");
    if (constrained == 1) printf("Constrained...\n"); 
  }
  else {
    frame_ptr = QuGetRep(vts_in2);
    
	if( strstr(frame_ptr, "xyz") == NULL ){
		QplugAppendTextDisplay ("MinVar Failed >> constrain vector is not cartesian\n");
		return QPLUG_FAILURE;
	}
    
    printf("Constrained...\n");
    constrained = 1;
  }
  constrained = (constrained == 1) ? 1 : 0;
 
  QString txtbuf("# Requested constrain and MVA type :\n");
  txtbuf += QString("# Constrain = %1\n").arg(constrained);
  txtbuf += QString("# Covariance matrix type = %1 ").arg(Qsel);
  txtbuf += QString(" (Classic / (Constrained PQP / (Weimer type / (Siscoe type / (extreme Weimer /").section('/',Qsel,Qsel);
  txtbuf += QString(")\n#");
  QplugAppendTextDisplay (txtbuf);


  // --- as of QSAS 2.3, no PGplot any more ---


  Rmatx=new QdRMatrix(Dim(3,3));
  Covar_matx=new QdRMatrix(Dim(3,3));
  Proj_matx=new QdRMatrix(Dim(3,3));
  Eigenvalues=new QdRMatrix(Dim(3));


  // ---- join TS and TS2 --------------

  QdObjectSeq_var ttIn    = new QdObjectSeq(nseq);
  QdObjectSeq_var toJoin  = new QdObjectSeq(nseq);
  QdObjectSeq_var joined  = new QdObjectSeq(nseq);
  QdObjectSeq_var restSig = new QdObjectSeq(nseq);
  QdObjectSeq_var joinMethod = new QdObjectSeq(nseq);
  QdObjectSeq_var gapValue   = new QdObjectSeq(nseq);
  QdObjectSeq_var gapHandler = new QdObjectSeq(nseq);

  QdTimeSeq_var ttJoined = get_timetags((*arglist)[0]);      // tt of TS
  if (ttJoined.is_nil()) {
    QplugAppendTextDisplay("Bad or missing time tags in input data\n");
    return QPLUG_FAILURE;
  }

  QuIsTSRegular(ttJoined,&deltat);                    // only used for joining
  avg_width = 1.5 * deltat;

  toJoin[0] = vts_in;
  ttIn[0]   = get_timetags(vts_in);
  joined[0] = toJoin[0]->create();
  toJoin[1] = vts_in2;
  ttIn[1]   = get_timetags(vts_in2);
  joined[1] = toJoin[1]->create();

  for (i=0; i<2; i++) {
    joinMethod[i] = new QdString(LINEAR);
    gapValue[i] = new QdRScalar(avg_width);
    gapHandler[i] = new QdString(LINEAR);   
  }

  try {
    multijoin(ttIn, toJoin, ttJoined, joined, 
	      joinMethod, gapValue, gapHandler, triage_cnt);
  }
  catch(Exception &e) {
    e.print_msg();
    return QPLUG_FAILURE;
  }

  QdRMatrixSeq_var joined_TS  = QdRMatrixSeq_var::narrow(joined[0]);
  QdRMatrixSeq_var masked_TS  = new QdRMatrixSeq();
  QdRMatrixSeq_var joined_TS2 = QdRMatrixSeq_var::narrow(joined[1]);
  QdRMatrixSeq_var masked_TS2 = new QdRMatrixSeq();


  //	...next, subset the time range and data array

  QdTimeSeq_var tt_out = new QdTimeSeq();
  SequenceMask  msk(ttJoined->get_mask(QdTimeSeq::InRange(start_end->start(), start_end->end())));
  ttJoined->get_subsequence(*tt_out, msk);

  joined_TS->get_subsequence(*masked_TS,msk);
  joined_TS2->get_subsequence(*masked_TS2,msk);

  masked_TS->copy_xrefs_from(vts_in);
  set_timetags( (QdObject_var) masked_TS, tt_out);
  QuSetTxtAttr("Frame", "vector>mv_xyz", (QdObject_var) masked_TS);

  masked_TS2->copy_xrefs_from(vts_in2);
  set_timetags( (QdObject_var) masked_TS2, tt_out);
  QuSetTxtAttr("Frame", "vector>mv_xyz", (QdObject_var) masked_TS2);

#ifdef VERBOSE
  printf(">>MinVar.cc Subsetting done successful\n");
#endif
  int nestsize = masked_TS->sequence_size();

  if (nestsize <= 0) {
    QplugAppendTextDisplay("No Data in Interval\n");
    return QPLUG_FAILURE;
  }

  //    ..some output info
  QdTimeSeq_var tt = get_timetags(masked_TS2);
  sprintf(attrtxt,"# Given time interval = %s - %s",tt[0].iso_srep().c_str(),tt[nestsize-1].iso_srep().c_str()+11);
  QplugAppendTextDisplay(attrtxt);
  sprintf(attrtxt,"# Used t = %s -  %s (%d samples)",tt[0].iso_srep().c_str()+11,tt[nestsize-1].iso_srep().c_str()+11,nestsize);
  QplugAppendTextDisplay(attrtxt);

#ifdef VERBOSE
  printf(">>MinVar.cc Subsetting done successful I\n");
#endif

  // --- special treatment for tensor_TS as input (covar matrix == tesnor_TS[i]) -------

  if (ts_is_tensor) {
    printf("TS_is tensor \n");
    Matrix<double> covEigvals;       // Matrix Dim(3) of eigenvalues
    Matrix<double> covEigvecs;       // Matrix Dim(3,3) of eigenvectors (in rows)
    Matrix<double> covMatrix;        // covariance matrix
    Matrix<double> temp;             // temporary storage matrix
    QdRMatrix_var A  = new QdRMatrix(Dim(3), 0.0);
    QdRMatrix_var P  = new QdRMatrix(Dim(3,3), 0.0);
    QdRMatrix_var Q  = new QdRMatrix(Dim(3,3), 0.0);
    QdRMatrix_var T  = new QdRMatrix(Dim(3,3), 0.0);
    QdRMatrix_var C  = new QdRMatrix(Dim(3,3), 0.0);
    QdRMatrix_var V  = new QdRMatrix(Dim(3,3), 0.0);
    QdRMatrixSeq_var OutMatx_TS = new QdRMatrixSeq(nestsize,RMatrix(Dim(3,3),0.0));
    QdRMatrixSeq_var RotMatx_TS = new QdRMatrixSeq(nestsize,RMatrix(Dim(3,3),0.0));
    QdRMatrixSeq_var CovarMatx_TS = new QdRMatrixSeq(nestsize,RMatrix(Dim(3,3),0.0));
    QdRMatrixSeq_var ProjMatx_TS = new QdRMatrixSeq(nestsize,RMatrix(Dim(3,3),0.0));
    for (int k=0; k<nestsize; k++) {
      covMatrix.resize(Dim(3,3));
      for (i=0; i<3; i++) {
	(*A)[i] = (double)(*masked_TS2)[k][i];
	for (j=0; j<3; j++) (*C)[i][j] = (double) (*masked_TS)[k][i][j];
      }
      (*C)[1][0] = (double) (*C)[0][1];     // force symmetry
      (*C)[2][0] = (double) (*C)[0][2];    
      (*C)[2][1] = (double) (*C)[1][2];    
     
#ifdef VERBOSE
  printf(">>MinVar.cc Subsetting done successful a\n");
#endif

      // ------- constrain by projection C = P#C#P 
      P = Projection(A);
      T = QdRMatrix_var::narrow(C * P);
      C = QdRMatrix_var::narrow(P * T);

      for (i=0; i<3; i++) {
	for (j=0; j<3; j++) {
	  covMatrix[i][j] = (double) (*C)[i][j]; 
	  (*ProjMatx_TS)[k][i][j] = (double)(*P)[i][j];
	}
      }

      C->display();

      covMatrix[1][0] = covMatrix[0][1];
      covMatrix[2][0] = covMatrix[0][2];
      covMatrix[2][1] = covMatrix[1][2];

#ifdef VERBOSE
  printf(">>MinVar.cc Subsetting done successful Ib\n");
#endif

      // --- find eigenvals, eigenvecs
  try{
      MatrixEig eig(covMatrix);
      covEigvecs=eig.eigenvectors().seq_to_matrix();
      covEigvals=eig.eigenvalues().seq_to_matrix();
      for (i=0; i<3; i++){
	for (j=0; j<3; j++) {
	  (*RotMatx_TS)[k][i][j] = (double) covEigvecs[i][j];                   // save copy for WL output object
	  (*CovarMatx_TS)[k][i][j] = covMatrix[i][j];                           // projected covariance matrix for WL output
	  (*V)[i][j] = (double) covEigvecs[i][j];                               // V contain eigenvecs = rotation matrix
	  (*P)[i][j] = (double) covEigvecs[j][i];                               // P is the transposed of V
	  (*Q)[i][j] = (double) (*masked_TS)[k][i][j];                          // Q is copy of input matrix
	}
      } 
  } catch (Exception &e) {}
      T = QdRMatrix_var::narrow(Q * P);
      C = QdRMatrix_var::narrow(V * T);
      for (i=0; i<3; i++) for (j=0; j<3; j++) (*OutMatx_TS)[k][i][j] = (double)(*C)[i][j];
    }


    set_timetags( (QdObject_var) RotMatx_TS, tt_out);
    set_timetags( (QdObject_var) ProjMatx_TS, tt_out);
    set_timetags( (QdObject_var) CovarMatx_TS, tt_out);
    set_timetags( (QdObject_var) OutMatx_TS, tt_out);

    OutMatx_TS->copy_xrefs_from(masked_TS);
    ProjMatx_TS->copy_xrefs_from(masked_TS2);
 
    QuSetTxtAttr("CATDESC", "Time series with rotation matrices", RotMatx_TS);
    QuSetTxtAttr("CATDESC", "Time series with covariance matrices", CovarMatx_TS);
    QuSetTxtAttr("CATDESC", "Time series with the rotated input tensor", RotMatx_TS);
    QuSetTxtAttr("CATDESC", "Time series with projection matrices", ProjMatx_TS);

    arglist->push_back(OutMatx_TS);     
    arglist->push_back(RotMatx_TS);   
    arglist->push_back(CovarMatx_TS);   
    arglist->push_back(ProjMatx_TS);   
    return QPLUG_SUCCESS; 
  }



  //      ...next do the minimum variance analysis
  MV_return = MV_analyseB0(masked_TS, masked_TS2, Eigenvalues, Rmatx, Proj_matx, Covar_matx, constrained, nestsize, SC__id, Qsel);

  printf("%d\n",MV_return);
#ifdef VERBOSE
  // ---- print som output info --------------


  printf(">>MinVar.cc Minimum Variance Analysis done \n");
  printf("Eigenvalues, eigenvectors are:\n");
  for (i=0; i<3; i++){
    sprintf(evalsevecs,"%10.5g  [%10.5g, %10.5g, %10.5g ]\n", 
	    (double)(*Eigenvalues)[i],
	    (double)(*Rmatx)[i][0],
	    (double)(*Rmatx)[i][1], 
	    (double)(*Rmatx)[i][2]);
    printf (evalsevecs);
  }   // end for


  // --- 19.6.2006 - normalized covariance matrix --------


  printf("\nNormalized covariance matrix:\n");  
  double l[4] = {0.0, (double)(*Eigenvalues)[2],(double)(*Eigenvalues)[1],(double)(*Eigenvalues)[0]};
  double w = ((l[1] - l[3])*(l[2] - l[3]))  / (l[3] * sqrt(l[2]*l[1]));
  printf("l = %10.4f  %10.4f  %10.4f, w = %10.4f\n", l[1], l[2], l[3], w);
  for (i=0; i<3; i++){
    sprintf(evalsevecs,"[%10.5g, %10.5g, %10.5g ]\n", 
	    (double)(*Covar_matx)[i][0]/w,
	    (double)(*Covar_matx)[i][1]/w, 
	    (double)(*Covar_matx)[i][2]/w);
    printf (evalsevecs);
  }   // end for

  

  vec TS_ave;
  TS_ave.x = 0.0; TS_ave.y = 0.0; TS_ave.z = 0.0;
  for (int j=0; j<nestsize; j++) {
    TS_ave.x += (float) masked_TS[j][0];
    TS_ave.y += (float) masked_TS[j][1];
    TS_ave.z += (float) masked_TS[j][2]; 
  }

  TS_ave.x /= (double) nestsize;    
  TS_ave.y /= (double) nestsize;      
  TS_ave.z /= (double) nestsize; 

#endif

  int compsel = Qsel < 1 ? 0 : 1;
  float avebt = sqrt(TS_ave.x*TS_ave.x + TS_ave.y*TS_ave.y + TS_ave.z*TS_ave.z);
  float b_dot_n = TS_ave.x * (float) (*Rmatx)[compsel][0] + TS_ave.y * (float) (*Rmatx)[compsel][1] + TS_ave.z * (float) (*Rmatx)[compsel][2];
  sprintf(attrtxt,"# <Ts in>  = [%6.3f %6.3f %6.3f],  <|Ts in|> = %7.3f,   <Ts in> * n = %10.4f\n\n",TS_ave.x,TS_ave.y, TS_ave.z, avebt, b_dot_n );
  QplugAppendTextDisplay (attrtxt);

  if (MV_return != QPLUG_SUCCESS) return MV_return; 

  // rotate vector TS into MV frame

  for (int k=0; k<masked_TS->size() ; k++){
    masked_TS[k] =  (*Rmatx) * masked_TS[k];
  }

  //      ...next do the plotting
  MV_return = MV_plot_results(masked_TS, Eigenvalues, Rmatx, constrained);
  if (MV_return != QPLUG_SUCCESS) MV_return; 


  // set attributes

  masked_TS->change_xref("Eigenvalues", Eigenvalues);
  masked_TS->change_xref("Eigenvectors", Rmatx);

  try{
    QdObject_var name_xref = vts_in->get_xref(OBJECT_NAME);
    Rmatx->change_xref("InputSeriesName", name_xref);
    masked_TS->change_xref("InputSeriesName", name_xref);
  }
  catch(Exception &e){
    e.print_msg();
    return QPLUG_FAILURE;   
  }
 
  sprintf(attrtxt, "%d", (int)(masked_TS->size()) );
  QuSetTxtAttr("NumberOfDataValues", attrtxt, Rmatx);
  QuSetTxtAttr("NumberOfDataValues", attrtxt, Covar_matx);
  QuSetTxtAttr("NumberOfDataValues", attrtxt, Eigenvalues);

  QuSetTxtAttr("FIELDNAM", "Rotation matrix", Rmatx);
  QuSetTxtAttr("FIELDNAM", "Covariance matrix", Covar_matx);
  QuSetTxtAttr("CATDESC", "Matrix of Eigenvectors as rows", Rmatx);
  QuSetTxtAttr("FIELDNAM", "Eigenvalues", Eigenvalues);
  QuSetTxtAttr("CATDESC", "Eigenvectors are corresponding rows in Rmatx", Eigenvalues);

  Rmatx->change_xref("TimeInterval", start_end);
  Covar_matx->change_xref("TimeInterval", start_end);
  Eigenvalues->change_xref("TimeInterval", start_end);
  Rmatx->change_xref("Eigenvalues", Eigenvalues);
  Proj_matx->change_xref("Eigenvalues", Eigenvalues);
  Proj_matx->change_xref("TimeInterval", start_end);

  // put new objects on working list

  //return  QPLUG_FAILURE;
  arglist->push_back(masked_TS);     
  arglist->push_back(Rmatx);   
  arglist->push_back(Covar_matx);   
  arglist->push_back(Proj_matx);   

 
  //	..clean up and exit

  QplugAppendTextDisplay ("MinVar >> - exiting successfully..\n");

  QString ts="Time elapsed = "+QString::number(t.elapsed()/1000.0)+"\n";
  QplugAppendTextDisplay (ts);
  return  QPLUG_SUCCESS;
 
}  // end MinVar




// ---------------------------------------------------------
//
//                   MVA analyse 
//
//  Last modified
//  SEH, 26 Aug 2002, - Covariance as output
// ---------------------------------------------------------


QplugReturnStatus  MV_analyseB0(QdRMatrixSeq_var ts_out,
				QdRMatrixSeq_var ts_out2,
				QdRMatrix_var Eigenvalues,
				QdRMatrix_var Rmatx,
				QdRMatrix_var Proj_matx,
				QdRMatrix_var Covar_matx,
				int constrained,int nestsize,
				const char *SC__ID, const int Qsel){

  QString str("# Eigenvalues and eigenvectors are :\n");

  char evalsevecs[80];
  char projmxstr[80];
  char typebuf[80];

  if (constrained) {
    sprintf(typebuf,"MVABC1_SC%s (%d)",SC__ID,nestsize);
  } else {
    sprintf(typebuf,"MVAB_SC%s (%d)",SC__ID,nestsize);
  }   
  str+=typebuf;
  try{
    ConstrainedMinVariance minvar(*ts_out,*ts_out2,constrained,Qsel);   // qdos minvariance call
    double dot=minvar.orthogonality_quality();
    if(dot>0.001){
      QplugAppendTextDisplay(
			     "MinVar >> Eigenvectors failed simple orthonormal test at level 0.001\n");
    }
    *Rmatx = minvar.eigenvectors().duplicate(SHALLOW);
    *Eigenvalues = minvar.eigenvalues().duplicate(SHALLOW); 
    *Covar_matx = minvar.covariance().duplicate(SHALLOW);
    *Proj_matx =  minvar.projection().duplicate(SHALLOW);
    //   if ((*Rmatx)[constrained][0] < 0.0) {
   
    //    (*Rmatx) *= -1.00;    // ensure n_x positive
    //   }
  }
  catch(Exception& e) {
    e.print_msg();
    QplugAppendTextDisplay ("Exception..\n");
    return QPLUG_FAILURE; 
  }


  str += "\n";
  for (int i=0; i<3; i++){
    sprintf(evalsevecs,"%10.5f  [%10.5f, %10.5f, %10.5f ]\n", 
	    (double)(*Eigenvalues)[i], (double)(*Rmatx)[i][0],
	    (double) (*Rmatx)[i][1], (double)(*Rmatx)[i][2]);
    str += evalsevecs;
  }   // end for
  QplugAppendTextDisplay (str.latin1());

  return QPLUG_SUCCESS;
}




// -------------------------------------------------------------------------
//
//  New plot routine - wrapper to PLplot
//
//  SEH, 10 Mar 2009
//
// -------------------------------------------------------------------------
QplugReturnStatus  MV_plot_results(QdRMatrixSeq_var vts_out, 
                                   QdRMatrix_var Eigenvalues, 
				   QdRMatrix_var Rmatx,
				   int constrained)   {

  mvplot (vts_out, Eigenvalues, Rmatx, constrained);
  return QPLUG_SUCCESS;
}



// -------------------------------------------------------------------------
//
//  Returns a projection matrix jfr blue book
//
//  SEH, 27 Jul 2005
//
// -------------------------------------------------------------------------
QdRMatrix_var Projection(QdRMatrix_var A) 
{
  QdRMatrix_var C  = new QdRMatrix(Dim(3,3), 0.0);
  double sz2 = vecsum2(A);
  (*C)[0][0] = 1.0 - ((*A)[0]*(*A)[0]) / sz2;
  (*C)[0][1] = - ((*A)[0]*(*A)[1]) / sz2;
  (*C)[0][2] = - ((*A)[0]*(*A)[2]) / sz2;

  (*C)[1][0] = - ((*A)[1]*(*A)[0]) / sz2;
  (*C)[1][1] = 1.0 - ((*A)[1]*(*A)[1]) / sz2;
  (*C)[1][2] = - ((*A)[1]*(*A)[2]) / sz2;
  
  (*C)[2][0] = - ((*A)[2]*(*A)[0]) / sz2;
  (*C)[2][1] = - ((*A)[2]*(*A)[1]) / sz2;
  (*C)[2][2] = 1.0 - ((*A)[2]*(*A)[2]) / sz2;
  return(C);
}



double vecsum2(QdRMatrix_var Vector)
//
// returns the norm^2 of a vector
//
{
  double sum=0.0;

  for (int i=0; i<3; i++) sum += (*Vector)[i] * (*Vector)[i];

  return(sum);
}




// ---------------------------------------------------------
//
// Does nothing, but there needs to be a main() fuction
//
// ---------------------------------------------------------

int main()
{
  // --- do nothing ---
}
