/***********************************************************************
*           See Accompanying licence file QSAS_licence                 *
***********************************************************************/
/*************************************************************************
* PROJECT: Cluster UK CDHF
*
* COMPONENT:    Plug In library
* 
* MODULE:	ShockNormalCoords.cc
*
* LANGUAGE:	C++
*
* FUNCTION:     QSAS Shock Normal Coordinates Plugin wrapper
*
* PURPOSE:      Calculate Rotation Matrix to transform into Shock
*               Normal Coordinates lmn. n is normal to a model 
*               bow shock, the l direction is the projection of the
*               upstream field onto the surface, and m completes the
*               right handed lmn system..
* 
* DESCRIPTION:  .
*
* ARGUMENTS:
*    IN:      QdCDOList  list of plug in objects containing
*
*             pos_in      Constant vector or Vector TS
*             time_int    Centre time interval for crossing
*             Vsw         Approx solar wind speed to calculate aberration
*             upstream_vec Upstream magnetic field vector or time series
*             time_int_up Upstream time interval to average upstream_vec
*             imodel      Model number from Ch10 of ISSI Book
*             frame_out   GSE->lmn (0) or GSM->lmn (1)
*
*   OUT:
*             frametolmn  Rotation Matrix R. 
*                          R dot vector(frame_out) = vector(lmn)		
*
* GLOBALS:    none
*
* RETURN:
*    TYPE:    QplugReturnStatus
*    VALUES:
*
*
* FUNCTIONS USED: built on qrtn and qbowshock modules
*
* HISTORY: July 2001 Steve Schwartz 
*
***********************************************************************/


#include <stdio.h>
#include <math.h>
#include <string.h>
#include <stdlib.h>
#include "qplug_if.h"

#include "qdutil.h"
#include "Qdos.h"
#include "qrtn.h"
#include "qar.h"
#include "qbowshock.h"

const double Re = 6.3712e6; //Earth radius [m]  - a global	

using namespace QSAS;  

// local prototypes

extern "C" QplugReturnStatus ShockNormalCoords(QplugArgList *);
QplugReturnStatus QSNPosition(QrVector *vec_gsep,
                              QdObject_var pos_in,
			      QdTimeInterval_var time_int);
QplugReturnStatus QSNConstPosition(QrVector *vec_gsep, 
                                    QdRMatrix_var pos_in, 
				    QdTimeInterval_var time_int);
QplugReturnStatus QSNCentrePosition(QrVector *vec_gsep, 
                                    QdRMatrixSeq_var pos_in_seq, 
				    QdTimeInterval_var time_int);
QplugReturnStatus QSNMakeGSERe (QrVector *vecp);
double SIConv_factor(QdObject_var in1_ptr);
QplugReturnStatus QSNmake_upstream_vec(QrVector *vec, QdObject_var vec_obj, 
                            QdTimeInterval_var time_int);
void QrVector_diag_print(QrVector *vecp);
  
/**********************************************************************/
QplugReturnStatus ShockNormalCoords(  QplugArgList *call_list){
/**********************************************************************/

// define some C variables

long  check_rtn;
float Vsw, r_vec[3], n_gse[3];
QBSMP_models   imodel;
int   i, qbs_rtn, qar_status;

double modm;

// call-list arguments
 
QdObject_var         pos_in;       // 0
QdTimeInterval_var   time_int;     // 1
QdRScalar_var        Vsw_var;       // 2
QdObject_var         ups_vec_obj;   // 3
QdTimeInterval_var   time_int_up;   // 4
QdRScalar_var        imodel_var, frame_out_var; // 5, 6
QdRMatrix_var        frametolmn;                         // 7

// define Qdos data objects

QrVector  vec_gse, vec_up, l, m, n;

QplugReturnStatus ret_status;
QplugAppendTextDisplay("\n");

// ...first 2 items are input position vector/vector TS and time interval

pos_in = QdObject_var::narrow((*call_list)[0]);
if (pos_in.is_nil()){
  QplugAppendTextDisplay("SNCoords>> Error getting pos_in object\n");
  return QPLUG_FAILURE;
}

time_int = get_timeinterval((*call_list)[1]);
if( time_int.is_nil() ) {
  QplugAppendTextDisplay ("SNCoords>> Time Interval object invalid.\n");
  return QPLUG_FAILURE;
}

vec_gse.IsPositionVector = 0;     // force so QrDoRotation will accept

ret_status = QSNPosition(&vec_gse,pos_in,time_int);
if (ret_status != QPLUG_SUCCESS){
  QplugAppendTextDisplay("SNCoords>> QBNPosition failed\n");
  return QPLUG_FAILURE;
}

// Whew! vec_gse now has a sequence with 1 element, the gse_xyz position
// of the crossing in Re with one timetag. 

Vsw_var = QdRScalar_var::narrow((*call_list)[2]);
if (Vsw_var.is_nil()){
   QplugAppendTextDisplay("SNCoords>> Input arg Vsw not Scalar\n");
   return QPLUG_FAILURE;
}
imodel_var = QdRScalar_var::narrow((*call_list)[5]);
if(imodel_var.is_nil()){
   QplugAppendTextDisplay("SNCoords>> Input arg imodel not Scalar\n");
   return QPLUG_FAILURE;
}

// unpack inputs for QBOWSHOCK routines and find gse normal;
Vsw = (float) (*Vsw_var);
imodel = (QBSMP_models)(int)*imodel_var;
for (i=0; i<3; i++){r_vec[i] = (float)((vec_gse.ptr)[0][i]);}

qbs_rtn = QBSNormalAtSurfaceGSE(r_vec,Vsw,imodel,n_gse);
if (qbs_rtn != QBS_OK){
   QplugAppendTextDisplay("SNCoords>> QBSNormal returned error\n");
   return QPLUG_FAILURE;
}

// put normal in n

n          = vec_gse;   // copy over n_recs = 1, timetag, ...
n.in_frame = GSE;
n.ptr      = new QdRMatrixSeq(1,RMatrix(Dim(3)));
QuSetFrame( n.ptr, "vector", "gse", "xyz"); // needed for new qrtn
for (i=0; i<3; i++){(n.ptr)[0][i] = n_gse[i];}  // must be a better way

// get upstream field vector from inputs; average over time interval

ups_vec_obj = QdObject_var::narrow((*call_list)[3]);
if (ups_vec_obj.is_nil()){
  QplugAppendTextDisplay("SNCoords>> Error getting upstream vector object\n");
  return QPLUG_FAILURE;
}

time_int_up = get_timeinterval((*call_list)[4]);
if( time_int_up.is_nil() ) {
  QplugAppendTextDisplay ("SNCoords>>Upstr Time Interval object invalid.\n");
  return QPLUG_FAILURE;
}

ret_status = QSNmake_upstream_vec(&vec_up, ups_vec_obj, time_int_up);
if (ret_status != QPLUG_SUCCESS){
  QplugAppendTextDisplay ("SNCoords>>make_upstream_vec failed\n");
  return QPLUG_FAILURE;
}

frame_out_var = QdRScalar_var::narrow((*call_list)[6]);
if(frame_out_var.is_nil()){
   QplugAppendTextDisplay("SNCoords>> Frame_out not a scalar\n");
   return QPLUG_FAILURE;
}

frametolmn = new QdRMatrix(Dim(3,3));

switch ( (int)(*frame_out_var) ){
   case 1:      // gsm
      n.out_frame = GSM;
      vec_up.out_frame = GSM;
      QuSetFrame(frametolmn, "rot", "gsm", "xyz");
      break;
   case 0: default:    // gse
      n.out_frame = GSE;
      vec_up.out_frame = GSE;
      QuSetFrame(frametolmn, "rot", "gse", "xyz");
      break;
}

check_rtn = QrDoRotation(&n);  // put n in correct frame
if (check_rtn != QRTN_OK && check_rtn != SAME_FRAME){
   QplugAppendTextDisplay( "SNCoords>> Failed to put n into out frame\n");
   return QPLUG_FAILURE;
}
n.in_frame = n.out_frame;      // make in_frame consistent with holding
check_rtn = QrDoRotation(&vec_up);  // put vec_up in correct frame
if (check_rtn != QRTN_OK && check_rtn != SAME_FRAME){
   QplugAppendTextDisplay( "SNCoords>> Failed to put vec_up into out frame\n");
   return QPLUG_FAILURE;
}
vec_up.in_frame = vec_up.out_frame; // make in_frame consistent with holding

// find n x B_up 

m.ptr = QdRMatrixSeq_var::narrow(outer(n.ptr, vec_up.ptr));
if ((m.ptr).is_nil()) {
    QplugAppendTextDisplay ("SNCoords>> n cross B_u failed\n");
    return QPLUG_FAILURE;
}

modm = norm(((m.ptr)[0])) ;
if (modm<=0){
    QplugAppendTextDisplay ("SNCoords>> |m| returned invalid number\n");
    return QPLUG_FAILURE;
}

*(m.ptr) /=  modm;
if ((m.ptr).is_nil()) {
    QplugAppendTextDisplay ("SNCoords>> m/|m| failed\n");
    return QPLUG_FAILURE;
}

l.ptr = QdRMatrixSeq_var::narrow(outer(m.ptr,n.ptr));
if ((l.ptr).is_nil()){
    QplugAppendTextDisplay ("SNCoords>> l = m x n failed\n");
    return QPLUG_FAILURE;
}

for (i=0; i<3; i++){
    (*frametolmn)[0][i] = (l.ptr)[0][i];
    (*frametolmn)[1][i] = (m.ptr)[0][i];
    (*frametolmn)[2][i] = (n.ptr)[0][i];
}

// set the attributes for the output rotation matrix

QuSetTxtAttr("FIELDNAM", "RMatrix_to_lmn", frametolmn);
QuSetTxtAttr("SI_conversion", "1>()", frametolmn);
QuSetTxtAttr("UNITS", " ", frametolmn);
QuSetTxtAttr("ToFrame", "vector>lmn_xyz", frametolmn);
char model_name[200];
sprintf(model_name,"Model %d: %s",imodel,BSMP_name[imodel]);
QuSetTxtAttr("Boundary Model", model_name, frametolmn);

QdRMatrixSeq_var crossing = vec_gse.ptr;
char crossing_str[200];
sprintf(crossing_str,"( %9.3g, %9.3g, %9.3g )GSE Re",
   (double)crossing[0][0],(double)crossing[0][1],(double)crossing[0][2]);
QuSetTxtAttr("Crossing",crossing_str,frametolmn);
char crossingt_str[200];
sprintf(crossingt_str,"%s", (vec_gse.t)[0].iso_srep().c_str());
QuSetTxtAttr("Crossing Time",crossingt_str,frametolmn);
char B_up_str[200];
sprintf(B_up_str,"B_up = (%9.3g, %9.3g, %9.3g)", 
  (double)vec_up.ptr[0][0], (double)vec_up.ptr[0][1], (double)vec_up.ptr[0][2]);
QuSetTxtAttr("B_upstream",B_up_str,frametolmn);
char B_up_time_str[300];
sprintf(B_up_time_str,"%s - %s",time_int_up->start().iso_srep().c_str(),
           time_int_up->end().iso_srep().c_str());
QuSetTxtAttr("Upstream Interval", B_up_time_str, frametolmn);
double thetabn = ((vec_up.ptr)[0]).dot((n.ptr)[0]);
thetabn /= (norm((vec_up.ptr)[0]) *  norm((n.ptr)[0]));
thetabn = (45.0/atan(1.0))*acos(thetabn);
char thetabn_str[200];
sprintf(thetabn_str,"Theta_bnu = %11.5g deg", thetabn);
QuSetTxtAttr("Theta_bn",thetabn_str, frametolmn);

call_list->push_back( (QdObject_var) frametolmn);     

// put some results to plug-in text window
char outstr[200];
sprintf(outstr,"SNCoords>> Crossing %s at %s\n", crossing_str, crossingt_str);
QplugAppendTextDisplay(outstr);
sprintf(outstr,"SNCoords>> Normal is (%9.3g, %9.3g, %9.3g) Frame = %s\n",
   (double)(n.ptr)[0][0], (double)(n.ptr)[0][1], (double)(n.ptr)[0][2],
   QuGetAttrText("Frame",frametolmn));
QplugAppendTextDisplay(outstr);
QplugAppendTextDisplay(B_up_str);QplugAppendTextDisplay("\n");
QplugAppendTextDisplay("Upstream Time Interval: ");
QplugAppendTextDisplay(B_up_time_str);QplugAppendTextDisplay("\n");
QplugAppendTextDisplay(thetabn_str);QplugAppendTextDisplay("\n");
QplugAppendTextDisplay("SNCoords>> finished successfully\n");

return QPLUG_SUCCESS;

}

/**********************************************************************/
QplugReturnStatus QSNPosition(QrVector *vec_gsep,
                              QdObject_var pos_in,
			      QdTimeInterval_var time_int){
/**********************************************************************/

long  check_rtn;
QdRMatrixSeq_var pos_in_seq;
QdRMatrix_var    pos_in_const;
QplugReturnStatus ret_status;

// sort out position of crossing. Input can either be a time series or
// a constant vector, so this a bit complicated. Separate routines
// return in vec_gse a sequence of 1 cartesian vector whose timetag
// is the centre of the time interval time_int, and whose frame is
// that of pos_in. This is then rotated into gse. [NB qrtn routines at
// present (July 2001) assume vectors are _xyz]

pos_in_seq       = QdRMatrixSeq_var::narrow(pos_in);
pos_in_const     = QdRMatrix_var::narrow(pos_in);
if( (!pos_in_seq.is_nil()) && (!pos_in_const.is_nil()) ){
  QplugAppendTextDisplay("SNCoords>> pos_in both const and sequence!?\n");
  return QPLUG_FAILURE;
}
if( pos_in_seq.is_nil() && pos_in_const.is_nil()){
  QplugAppendTextDisplay("SNCoords>> pos_in neither const nor sequence!?\n");
  return QPLUG_FAILURE;
}
if( !pos_in_seq.is_nil() ) {
  QplugAppendTextDisplay ("SNCoords>> Vector Sequence. Interpolating ...\n");
  ret_status = QSNCentrePosition(vec_gsep,pos_in_seq,time_int);
  if (ret_status != QPLUG_SUCCESS){
   QplugAppendTextDisplay("SNCoords>> Failed to calculate crossing position\n");
   return QPLUG_FAILURE;
  }
} else if (!pos_in.is_nil()){
  ret_status = QSNConstPosition(vec_gsep, pos_in_const, time_int);
  if (ret_status != QPLUG_SUCCESS){
    QplugAppendTextDisplay("SNCoords>> Failed to calc const. position\n");
    return QPLUG_FAILURE;
  }
} else {
  QplugAppendTextDisplay("SNCoords>> Confused about pos_in object\n");
  return QPLUG_FAILURE;
}

//QrVector_diag_print(vec_gsep);
vec_gsep->out_frame = GSE;
check_rtn = QrDoRotation(vec_gsep);  // make sure vec_gse in gse
if (check_rtn != QRTN_OK && check_rtn != SAME_FRAME){
   QplugAppendTextDisplay( "SNCoords>> Failed to put pos into gse\n");
   return QPLUG_FAILURE;
}
//QrVector_diag_print(vec_gsep);
vec_gsep->in_frame = GSE;        // advertise in GSE so don't double rotate

return QPLUG_SUCCESS;
}

/**********************************************************************/
QplugReturnStatus  QSNConstPosition(QrVector *vec_gsep, 
                                    QdRMatrix_var pos_in, 
				    QdTimeInterval_var time_int){
/**********************************************************************/

QplugReturnStatus ret_status;
const char * xyz_comp[] = {"x", "y", "z"};
int i;

// grab constant vector and fill vec_gsep.

if (!QuIsThreeVector(pos_in)){
   QplugAppendTextDisplay("SNCoords>> Const pos_in not a 3Vector\n");
   return QPLUG_FAILURE;
}

QdTimeSeq_var tt = new QdTimeSeq(1, 0.5*(time_int->start()+time_int->end()));
vec_gsep->ptr = new QdRMatrixSeq(1,RMatrix(Dim(3)));
(vec_gsep->ptr)->copy_xrefs_from(pos_in);
QdRMatrixSeq_var pos_seq = new QdRMatrixSeq(1, (*pos_in) );
pos_seq->copy_xrefs_from(pos_in);
for (i=0; i<3; i++){
  QdRScalarSeq_var comp = 
     QdRScalarSeq_var::narrow(QuGetVectorComponent(pos_seq,xyz_comp[i]));
  if (comp.is_nil()){
     QplugAppendTextDisplay("SNCoords>> Const pos comp narrow failed\n");
     return QPLUG_FAILURE;
  }
  (vec_gsep->ptr)[0][i] = comp[0];
}
vec_gsep->n_recs = 1;
vec_gsep->t = tt;

vec_gsep->in_frame = QrGetFrameFromAttr(pos_in);
if (vec_gsep->in_frame == UNKNOWN){
  QplugAppendTextDisplay("SNCoords>> Const pos_in unknown frame\n");
  return QPLUG_FAILURE;
}

ret_status = QSNMakeGSERe(vec_gsep);
if (ret_status != QPLUG_SUCCESS){
  QplugAppendTextDisplay("SNCoords>> QSNMakeGSERe failed\n");
  return QPLUG_FAILURE;
}

return QPLUG_SUCCESS;
}


/**********************************************************************/
QplugReturnStatus  QSNCentrePosition(QrVector *vec_gsep, 
                                    QdRMatrixSeq_var pos_in_seq, 
				    QdTimeInterval_var time_int){
/**********************************************************************/

int i,nttin,ntrue,il,ih;
QplugReturnStatus ret_status;
QdTime tc;
double cl,ch, frac;
const char * xyz_comp[] = {"x", "y", "z"};

if (!QuIsThreeVector(pos_in_seq)){
   QplugAppendTextDisplay("SNCoords>> Sequence pos_in not a 3Vector\n");
   return QPLUG_FAILURE;
}

// prepare vec_gsep 

tc = 0.5*(time_int->start()+time_int->end());
QdTimeSeq_var tt = new QdTimeSeq(1,tc );
vec_gsep->ptr = new QdRMatrixSeq(1,RMatrix(Dim(3)));
(vec_gsep->ptr)->copy_xrefs_from(pos_in_seq);
vec_gsep->t = tt;
vec_gsep->n_recs = 1;
vec_gsep->in_frame = QrGetFrameFromAttr(pos_in_seq);
if (vec_gsep->in_frame == UNKNOWN){
  QplugAppendTextDisplay("SNCoords>> Const pos_in unknown frame\n");
  return QPLUG_FAILURE;
}

// linearly interpolate crossing position from bracketting time tags
// If any problems, default to 1st vector in pos_in_seq
// There are probably much simpler ways to do this. Note, though, that
// Qrtn routines only give correct answers if given x,y,z representation

QdTimeSeq_var tt_in = get_timetags(pos_in_seq);
if (tt_in.is_nil()){
  QplugAppendTextDisplay(
     "SNCoords>> No timetags for input pos seq. Taking 1st pt\n");
     il = ih = 0;
} else {
  nttin = tt_in->size();
  if (nttin <= 0){
     QplugAppendTextDisplay("SNCoords>> Input seq 0 length\n");
     return QPLUG_FAILURE;
  } else if (nttin == 1){
     il = ih = 0;
  } else {
     SequenceMask msk(tt_in->get_mask( QdTimeSeq::LE( tc)));
     ntrue = msk.num_true();
     if (ntrue <= 0) il = ih = 0;
     else {
       il = ntrue -1;
       ih =  (ntrue < nttin) ?   il + 1 : il; 
     }  
  }
}

if(ih == il) frac = 0.0;   // safe
else frac = (tc-tt_in[il])/(tt_in[ih] - tt_in[il]); // calc lin interpol factor

/* printf("tl: %s\ntc: %s\nth: %s\nfrac = %g\n",
                tt_in[il].iso_srep().c_str(),
		tc.iso_srep().c_str(),
		tt_in[ih].iso_srep().c_str(),
		frac);
*/

for (i=0; i<3; i++){                          // fill crossing position
  QdRScalarSeq_var comp = 
     QdRScalarSeq_var::narrow(QuGetVectorComponent(pos_in_seq,xyz_comp[i]));
  if (comp.is_nil()){
     QplugAppendTextDisplay("SNCoords>> Centre Pos comp narrow failed\n");
     return QPLUG_FAILURE;
  }
  if (ih==il) {
    vec_gsep->ptr[0][i] = (double)comp[il];
  } else {
    cl = (double)comp[il];
    ch = (double)comp[ih];
    vec_gsep->ptr[0][i] = cl + frac*(ch-cl);
  }
}

ret_status = QSNMakeGSERe(vec_gsep);
if (ret_status != QPLUG_SUCCESS){
  QplugAppendTextDisplay("SNCoords>> QSNMakeGSERe failed\n");
  return QPLUG_FAILURE;
}

return QPLUG_SUCCESS;
}

/**********************************************************************/
QplugReturnStatus QSNMakeGSERe (QrVector *vecp){
/**********************************************************************/

// rotate vecp to GSE and scale to Re

int check_rtn;
double factor, SIfactor;

// Rotate

vecp->out_frame = GSE;
//printf("QBNMakeGSEre received:\n");QrVector_diag_print(vecp);
check_rtn = QrDoRotation(vecp);
if (check_rtn != QRTN_OK && check_rtn != SAME_FRAME){
   QplugAppendTextDisplay("SNCoords>> QSNMakeGSERe rotate to gse failed\n");
   printf("QSNMakeGSERe QrDoRotation returned %d\n",check_rtn);
   fflush(stdout);
   return QPLUG_FAILURE;
}
vecp->in_frame = GSE;    // if don't do this, another call to QrDoRot will
                         // rotate again!
// Scale
// Algorithm:
//    Take SI_conversion, verify base unit = m, and use SI_conv/Re as factor
//    If SI_conversion exists but isn't -> m or not positive, assume factor=1.
//    If no SI_conversion, try to make sense of UNITS; default to factor = 1

factor = 1.0;  // assume in Re if all else fails; actually this is done
               // explicitly below, but this is here for safety.

//check whether SI_conversion exists
char *attr = QuGetAttrText("SI_conversion", vecp->ptr);
if(strlen(attr) == 0){                // no SI_conversion
   QplugAppendTextDisplay (
    "SNCoords>>WARNING:Input Vector SI_conversion not defined, trying UNITS.\n");
   attr = QuGetAttrText("UNITS", vecp->ptr);
   if(strlen(attr) == 0) {
     QplugAppendTextDisplay (
     "SNCoords>>WARNING: UNITS not defined, assuming [Re].\n");
     factor = 1.0;
   } else {                           // yes, we have UNITS
     if(!strcmp(attr, "Re")) factor=1.0;
     else if(!strcmp(attr, "RE")) factor=1.0;
     else if(!strcmp(attr, "km")) factor=1000.0/Re;
     else {
       QplugAppendTextDisplay (
         "SNCoords>>WARNING: UNITS not recognized, assuming [Re].\n");
       factor = 1.0;
     }
   }   
} else {      // Found SI_conversion.
  // check whether it is position vector, but don't change vecp->IsPos...
  // so can rotate it
  if(QrIsPositionVector(vecp->ptr)!= 1){  // checks base SI == m
      QplugAppendTextDisplay("SNCoords>> Input Vector not position vector. ");
      QplugAppendTextDisplay("SNCoords>> WARNING: Re will be assumed.\n");
      factor = 1.0;
  } else {
      SIfactor = SIConv_factor(vecp->ptr);  // get SI_conversion
      if(SIfactor <= 0.0) {
	QplugAppendTextDisplay("SNCoords>> Input Vec SI_conversion invalid.\n");
	QplugAppendTextDisplay ("SNCoords>> WARNING: Re will be assumed.\n");
	factor = 1.0;
      } else {
        factor = SIfactor/Re;
      }
  }
}

*(vecp->ptr) *= factor;

//printf("QBNMakeGSEre returned:\n");QrVector_diag_print(vecp);

return QPLUG_SUCCESS;
}

/***********************************************************************/
double SIConv_factor(QdObject_var in1_ptr){ 
/***********************************************************************/

 char * siconv1;
 double value1;
 char * ptr1;
 static char * null_unit = "1 > ()";

  char * txt1 = QuGetAttrText("SI_conversion", in1_ptr);
  
  if(strcmp(txt1, "") == 0) {
    siconv1 = new char[strlen(null_unit)+1] ;
    strcpy(siconv1, null_unit);
  } else {
    siconv1 = new char[strlen(txt1)+1] ;
    strcpy(siconv1, txt1);  
  }
  
  if( (ptr1 = strtok(siconv1, ">")) == NULL) {
     delete[] siconv1;
     return 0.0;
  }
  value1 = (double) atof(ptr1);
 
  delete[] siconv1;
  
  return value1;
  
} // end SIConv_factor


/*********************************************************************/
QplugReturnStatus QSNmake_upstream_vec(QrVector *vec, QdObject_var vec_obj, 
                            QdTimeInterval_var time_int){
/*********************************************************************/

// If ups_vec_obj is a constant, return it in QrVector *vec
// If it is a time series, average it over time_int

int i;
QdRMatrixSeq_var vec_in_seq, vecxyz_subseq;
QdRMatrix_var    vec_in_const;
QplugReturnStatus ret_status;
SequenceMask mask;

// be sure vec_obj in cartesians; returns same obj if already xyz
QdObject_var vecxyz_obj = QuGetVectorXYZ(vec_obj);

// prepare vec
vec->n_recs = 1;
vec->in_frame = QrGetFrameFromAttr(vecxyz_obj);
if (vec->in_frame == UNKNOWN){
  QplugAppendTextDisplay("SNCoords>> Upstream vec in unknown frame\n");
  return QPLUG_FAILURE;
}

vec->ptr = new QdRMatrixSeq(1,RMatrix(Dim(3)));
(vec->ptr)->copy_xrefs_from(vecxyz_obj);

// constant vector used as is; sequence averaged
vec_in_const = QdRMatrix_var::narrow(vecxyz_obj);
vec_in_seq = QdRMatrixSeq_var::narrow(vecxyz_obj);
if ( !vec_in_const.is_nil() ){  // constant - just use it!
   QplugAppendTextDisplay("SNCoords>>Const Upstr Vec; time ignored\n");
   for (i = 0; i<3; i++){ vec->ptr[0][i] = (*vec_in_const)[i]; }
} else if (!vec_in_seq.is_nil()) {  // sequence - average it
   // Get time tags
   QdTimeSeq_var time_tags=get_timetags(vecxyz_obj);
   if (time_tags.is_nil()){
     QplugAppendTextDisplay("SNCoords>>Upstr. Vec not time series; using avg");
     vecxyz_subseq = vec_in_seq;
   } else {
      // Find all points in time interval
      mask= time_tags->get_mask(
                 QdTimeSeq::InRange(time_int->start(),time_int->end()));
      // Create sub-sequence
      vecxyz_subseq=new QdRMatrixSeq;
      vec_in_seq->get_subsequence(*vecxyz_subseq,mask);
   }
   // Calculate average
   int num=vecxyz_subseq->sequence_size();
   if (num <= 0){
      QplugAppendTextDisplay("SNCoords>>Upstr. Vec has no pts in interval\n");
      return QPLUG_FAILURE;
   } else {
      RMatrix ave(vecxyz_subseq->sum() / num);
      for (i = 0; i<3; i++){ vec->ptr[0][i] = ave[i];}
   }
} else {
   QplugAppendTextDisplay("SNCoords>>Upst Vec neither const nor sequence!?\n");
   return QPLUG_FAILURE;
}

QdTimeSeq_var tt = new QdTimeSeq(1);
tt[0] = 0.5*(time_int->start() + time_int->end());
vec->t = tt;
set_timetags( (QdObject_var) vec->ptr, tt);

return QPLUG_SUCCESS;
}



/**************************************************************************/
void QrVector_diag_print(QrVector *vecp){
  printf("IsPositionVector = %d, in_frame = %d, out_frame = %d, n_recs = %d\n",
        vecp->IsPositionVector,vecp->in_frame,vecp->out_frame, vecp->n_recs);
  printf("First Vector: (%9.3g, %9.3g, %9.3g) at %s\n",
     (double)vecp->ptr[0][0], (double)vecp->ptr[0][1], (double)vecp->ptr[0][2],
     vecp->t[0].iso_srep().c_str());
	fflush(stdout);
  return;
}



