/***********************************************************************
*           See Accompanying licence file QSAS_licence                 *
***********************************************************************/
/*************************************************************************
* PROJECT: Cluster UK CDHF
*
* COMPONENT:    Plug In library
* 
* MODULE:	tsyganenko.cc
*
* LANGUAGE:	C++
*
* FUNCTION:     QSAS tsyganenko wrapper
*
* PURPOSE:      Tsyganenko Earth magnetic field calculation.
* 
* DESCRIPTION:  Plug In module checking input parameters and calling c-code.
*
* ARGUMENTS:
*    IN:      QdCDOList  list of plug in objects containing
*
*   OUT:
*         float tsyganenko_n       vector time series of function values		
*
* GLOBALS: see geopack.c
*
* RETURN:
*    TYPE:    QplugReturnStatus
*    VALUES:
*
*
* FUNCTIONS USED: QrGetFrameFromAttr,QrIsPositionVector,SIConv_factor (appended)
*                 from Tsyganenko's GEOPACK.FOR and T96.FOR packages:
*                 external C calcps_,t96_01__,igrf_,recalc_,geogsm_,bspcar_
*                 The external functions are translated by f2c, thus libf2c.a
*                 must be loaded.
*
* HISTORY: created 2001 Markus Fraenz 
*          UNITS checking repaired wjmf 12/07/01
*          rotation into GSM included wjmf 12/07/01
*
***********************************************************************/


#include <stdio.h>
#include <math.h>
#include <string.h>
#include <stdlib.h>
#include "qplug_if.h"

#include "qdutil.h"
#include "Qdos.h"
#include "qrtn.h"

double SIConv_factor(QdObject_var in1_ptr);

extern "C"  int calcps_(long int *iyr, long int *iday, long int *ihour, 
	long int *min__, long int *isec, float *ps);
extern "C"  int t96_01__(long int *iopt, float *parmod, float *ps, float *x, 
	float *y, float *z, float *bx, float *by, float *bz);
extern "C" int recalc_(long int *iyr,long  int *iday,long  int *ihour,
        long  int *min__,long  int *isec);
extern "C" int igrf_(long int *iy, long int *nm, float *r__, float *t, float 
	*f, float *br, float *bt, float *bf);
extern "C" int geogsm_(float *xgeo, float *ygeo, float *zgeo, float *xgsm, 
	float *ygsm, float *zgsm, long int *j);
extern "C" int bspcar_(float *teta, float *phi, float *br, float *btet, 
	float *bphi, float *bx, float *by, float *bz);


extern "C" QplugReturnStatus  Tsyganenko(  QplugArgList *);

using namespace QSAS;    

QplugReturnStatus  Tsyganenko(  QplugArgList *call_list)
{


// define some C variables

long	i,Ndata,check;

int   MJDdate,MJD0;
int   year, month, day, hour, minute, second, msec;
long  iopt=0,iyr,iday,ihour,imin,isec,jswitch;
long int nIGRF = 10;
bool  extset = TRUE,rotateset = FALSE; 

float parmod[10],ps=0.0,GSMx,GSMy,GSMz,bx=0.0,by=0.0,bz=0.0;
float pdyn,dst,byimf,bzimf,xgsm,ygsm,zgsm,bxgeo=0.0,bygeo=0.0,bzgeo=0.0;
float xgeo=0.0,ygeo=0.0,zgeo=0.0,rgeo=0.0,tgeo=0.0,pgeo=0.0,brgeo=0.0,btgeo=0.0,bpgeo=0.0;
double SIfactor,factor=1.0;
const double Re = 6.37104e6; //Earth radius [m]	
char	str_out[150];


// define Qdos data objects

QrVector vec;

QdRScalar_var      pdyn_obj,dst_obj,byimf_obj,bzimf_obj,nIGRF_obj;
QdRMatrixSeq_var   ts_in;
QdRMatrixSeq_var   tsyganenko_out;

QplugAppendTextDisplay ("Tsyganenko Model Plugin V1.1 M.Fraenz 31/07/2001.\n");

// ...first item is input Time Series
ts_in = QdRMatrixSeq_var::narrow((*call_list)[0]);

//  check QSAS datatype

if( ts_in.is_nil() ) {
  QplugAppendTextDisplay ("tsyganenko>> Input not Vector Sequence.\n");
  return QPLUG_FAILURE;
}
// Find original frame
 
vec.in_frame = QrGetFrameFromAttr(ts_in);

if( vec.in_frame == UNKNOWN ) {
  QplugAppendTextDisplay ("tsyganenko>>WARNING: Input Vector Frame not defined, assuming GSM.\n");
}
else if( vec.in_frame != GSM ) {
  rotateset = TRUE;
  //QplugAppendTextDisplay ("tsyganenko>> Input Vector not GSM.\n");
  //return QPLUG_FAILURE;
}

//check whether SI_conversion exists
char *attr = QuGetAttrText("SI_conversion", ts_in);
if(strlen(attr) == 0){
   QplugAppendTextDisplay ("tsyganenko>>WARNING: Input Vector SIconversion not defined, trying UNITS.\n");
   attr = QuGetAttrText("UNITS", ts_in);
   if(strlen(attr) == 0) {
     QplugAppendTextDisplay ("tsyganenko>>WARNING: UNITS not defined, assuming [Re].\n");
   }
   else {
     if(strcmp(attr, "Re") == 0) factor=1.0;
     else if(strcmp(attr, "RE") == 0) factor=1.0;
     else if(strcmp(attr, "km") == 0) factor=1000.0/Re;
     else {
       QplugAppendTextDisplay ("tsyganenko>>WARNING: UNITS not reckognized, assuming [Re].\n");
     }
   }   
}
else {  
  // check whether it is position vector
  vec.IsPositionVector = QrIsPositionVector(ts_in);
  if(vec.IsPositionVector != 1){
   QplugAppendTextDisplay ("tsyganenko>> Input Vector not position vector.\n");
   return QPLUG_FAILURE;
  }
  SIfactor = SIConv_factor(ts_in);  // get SI_conversion
  if(SIfactor <= 0.0) {
    QplugAppendTextDisplay ("tsyganenko>> Input Vector SI_conversion invalid.\n");
    return QPLUG_FAILURE;
  }
  else factor = SIfactor/Re;
}






sprintf(str_out, "tsyganenko>> Converting Input Vector to Re with factor:%g.\n", factor);
QplugAppendTextDisplay (str_out);


// get timetags
vec.t = get_timetags(ts_in);
if( (vec.t).is_nil() ) {
  QplugAppendTextDisplay ("Cannot find time tags, some frames are time dependent\n");
  return QPLUG_FAILURE;
}

Ndata = ts_in->sequence_size();

if (Ndata <= 0) {
	QplugAppendTextDisplay ("tsyganenko>> ERROR - Series must have at least 1 data point.\n");
	return QPLUG_FAILURE; }


//	 option pdyn

pdyn_obj = QdRScalar_var::narrow((* call_list)[1]);

if ( pdyn_obj.is_nil() ) { 
	QplugAppendTextDisplay ("tsyganenko>> ERROR - pdyn not scalar.\n");
	return QPLUG_FAILURE; }

pdyn = (float)(*pdyn_obj);

//	 option dst

dst_obj = QdRScalar_var::narrow((* call_list)[2]);

if ( dst_obj.is_nil() ) { 
	QplugAppendTextDisplay ("tsyganenko>> ERROR - DST not scalar.\n");
	return QPLUG_FAILURE; }

dst = (float)(*dst_obj);

//	 option byimf

byimf_obj = QdRScalar_var::narrow((* call_list)[3]);

if ( byimf_obj.is_nil() ) { 
	QplugAppendTextDisplay ("tsyganenko>> ERROR - byimf not scalar.\n");
	return QPLUG_FAILURE; }

byimf = (float)(*byimf_obj);

//	 option bzimf

bzimf_obj = QdRScalar_var::narrow((* call_list)[4]);

if ( bzimf_obj.is_nil() ) { 
	QplugAppendTextDisplay ("tsyganenko>> ERROR - bzimf not scalar.\n");
	return QPLUG_FAILURE; }

bzimf = (float)(*bzimf_obj);

//	 option nIGRF

nIGRF_obj = QdRScalar_var::narrow((* call_list)[5]);

if ( nIGRF_obj.is_nil() ) { 
	QplugAppendTextDisplay ("tsyganenko>> ERROR - nIGRF undefined.\n");
	return QPLUG_FAILURE; }

nIGRF = (long)(*nIGRF_obj);
if ( nIGRF < 0 ) { 
	QplugAppendTextDisplay ("tsyganenko>> No external field added.\n");
	extset=FALSE;
	nIGRF = -nIGRF;
	}
if ( nIGRF > 10 ) { 
	QplugAppendTextDisplay ("tsyganenko>>  WARNING - nIGRF max=10.\n");
	nIGRF =10;
	}
if ( nIGRF == 0 ) { 
	QplugAppendTextDisplay ("tsyganenko>>  No internal field added.\n");}
else {
    sprintf(str_out, "tsyganenko>> Calculating internal field order:%i.\n", nIGRF);
    QplugAppendTextDisplay (str_out);
}

QdRMatrixSeq_var pos_out = (QdRMatrixSeq_var) Factory::clone(ts_in);
pos_out->copy_xrefs_from(ts_in);
set_timetags(pos_out, vec.t);


// now rotate into GSM if necessary
if (rotateset) {
  QplugAppendTextDisplay ("tsyganenko>>  Rotating into GSM.\n");
  vec.n_recs = Ndata;
  vec.ptr = pos_out;
  vec.out_frame = GSM; 
  vec.IsPositionVector = FALSE;  // this is to avoid rejection
  check = QrDoRotation(&vec);
  printf("Frame_in:%i\n",vec.in_frame);
  printf("Frame_out:%i\n",vec.out_frame);
  fflush(stdout);
  //printf("Check:%i\n",check);
  if (check > QRTN_OK) {
   QplugAppendTextDisplay ("tsyganenko>> ERROR rotating into GSM:\n ");
   QplugAppendTextDisplay (QrErrorMessage(check));
   return QPLUG_FAILURE; 
  }	  
  QrSetFrameAttr(vec.out_frame, pos_out);
}


//	..create the output data series objects



sprintf(str_out, "tsyganenko>> Calculating t96 function for %i data points.\n", Ndata);
QplugAppendTextDisplay (str_out);

tsyganenko_out = new QdRMatrixSeq(Ndata,RMatrix(Dim(3)));


  
  iopt=0;
  for (int i=0; i < 10; i++) parmod[i]=1.0;
  parmod[0] = pdyn;
  parmod[1] = dst;
  parmod[2] = byimf;
  parmod[3] = bzimf;

  for(int i=0; i<Ndata; i++)  {

   cdfEpoch((vec.t)[i]).split_ymd(year, month, day,hour,minute,second, msec);
   MJD0 = QrMJD(year, 1, 1);		     
   MJDdate = QrMJD(year, month, day);
   iday = (long) (MJDdate - MJD0);
   iyr = (long) year;
   ihour = (long) hour;
   imin = (long) minute;
   isec = (long) second;  

     
    xgsm = (float) (pos_out[i][0])*factor;
    ygsm = (float) (pos_out[i][1])*factor;
    zgsm = (float) (pos_out[i][2])*factor;
    //if (i <= 10) printf("xgsm:%e,ygsm:%e,zgsm:%e\n",xgsm,ygsm,zgsm);
    bx=0.0;
    by=0.0;
    bz=0.0;
    
    if (extset) {
     // first calculate dipole angle:
     calcps_(&iyr,&iday,&ihour,&imin,&isec,&ps);
     // then model Bxyz:
     t96_01__(&iopt,&parmod[0],&ps,&xgsm,&ygsm,&zgsm,&bx,&by,&bz);
    }
    
    // now the internal field:
    if (nIGRF > 0) {
    
      recalc_(&iyr,&iday,&ihour,&imin,&isec);
      jswitch = -1;
      geogsm_(&xgeo,&ygeo,&zgeo,&xgsm,&ygsm,&zgsm,&jswitch);
      //if (i <= 10) printf("xgeo:%e,ygeo:%e,zgeo:%e\n",xgeo,ygeo,zgeo);
    
      rgeo = sqrt(xgeo*xgeo + ygeo*ygeo + zgeo*zgeo);
      tgeo = acos(zgeo/rgeo); 
      pgeo = atan2(ygeo,xgeo);
      igrf_(&iyr,&nIGRF,&rgeo,&tgeo,&pgeo,&brgeo,&btgeo,&bpgeo);
      bspcar_(&tgeo,&pgeo,&brgeo,&btgeo,&bpgeo,&bxgeo,&bygeo,&bzgeo);
      /*if (i < 30){
        printf("iyr:%i,iday:%i,ihour:%i,imin:%i,isec:%i\n",iyr,iday,ihour,imin,isec);
        printf("bx:%g,by:%g,bz:%g\n",bx,by,bz);
        printf("rgeo:%g,tgeo:%g,pgeo:%g\n",rgeo,tgeo,pgeo);
        printf("bxgeo:%g,bygeo:%g,bzgeo:%g\n",brgeo,btgeo,bpgeo);
      }*/
      jswitch = 1;
      geogsm_(&bxgeo,&bygeo,&bzgeo,&xgsm,&ygsm,&zgsm,&jswitch);

      bx+=xgsm;
      by+=ygsm;
      bz+=zgsm;
    }
    tsyganenko_out[i][0] = (double) bx;
    tsyganenko_out[i][1] = (double) by;
    tsyganenko_out[i][2] = (double) bz;
  }

//	...set some attributes

  set_timetags(tsyganenko_out, vec.t);
  

  QuSetTxtAttr("SI_conversion", "1.0e-9>T", (QdObject_var) tsyganenko_out);
  sprintf(str_out, "B_xyz_t96");
  QuSetTxtAttr("LABLAXIS",str_out , (QdObject_var) tsyganenko_out);
  sprintf(str_out, "B_xyz Tsyganenko96");
  QuSetTxtAttr("FIELDNAM", str_out, (QdObject_var) tsyganenko_out);
  QuSetTxtAttr("UNITS", "nT", (QdObject_var) tsyganenko_out);
  
  // now rotate back into original frame if necessary
  if (rotateset) {
   QplugAppendTextDisplay ("tsyganenko>>  Rotating into original frame.\n");
   vec.n_recs = Ndata;
   vec.ptr = tsyganenko_out;
   vec.out_frame = vec.in_frame; 
   QrSetFrameAttr(GSM, tsyganenko_out);
   vec.in_frame = GSM;
   vec.IsPositionVector = FALSE;  // this is to avoid rejection
   check = QrDoRotation(&vec);
   //printf("Check:%i\n",check);
   if (check > QRTN_OK) {
    QplugAppendTextDisplay ("tsyganenko>> ERROR rotating into GSM:\n ");
    QplugAppendTextDisplay (QrErrorMessage(check));
    return QPLUG_FAILURE; 
   }	     
  } else QrSetFrameAttr(GSM, tsyganenko_out);




  QplugAppendTextDisplay ("tsyganenko>> finished...\n");



//	..add them to the list

  call_list->push_back( (QdObject_var) tsyganenko_out);     




//	..clean up and exit



return QPLUG_SUCCESS;


}

double SIConv_factor(QdObject_var in1_ptr){

 char * siconv1;
 double value1;
 char * ptr1;
 static char * null_unit = "1 > ()";

  char * txt1 = QuGetAttrText("SI_conversion", in1_ptr);
  
  if(strcmp(txt1, "") == 0) {
    siconv1 = new char[strlen(null_unit)+1] ;
    strcpy(siconv1, null_unit);
  }
  else{
    siconv1 = new char[strlen(txt1)+1] ;
    strcpy(siconv1, txt1);  
  }
  
  if( (ptr1 = strtok(siconv1, ">")) == NULL) return 0.0;
  value1 = (double) atof(ptr1);
 
  delete[] siconv1;
  
  return value1;
  
} // end QarSIConv_factor




