/* ------------------------------------------------------------------

 WaveletDenoise.c - Wavelet conversion & de-noiser for QSAS
 
  
 The wavelet routines are based on the package presented by 
 Torrence & Compo [1998]. FFT routines are taken from NCAR
 (ftp://ftp.ucar.edu/ftp/dsl/lib/fftpack/)

 

 SEH, Jan 22, 2001
      Nov 18, 2001  (allowed for vector input)
--------------------------------------------------------------------*/

#include <stdio.h>
#include <math.h>
#include <string.h>
#include <stdlib.h>
#include "qplug_if.h"
#include "qdutil.h"
#include "Qdos.h"

typedef long int integer;
typedef double doublereal;
typedef struct { doublereal r, i; } doublecomplex;

typedef struct {double dt,s0,dj,wno;  long n,s,base,npad;} wparam;
 
 extern "C"
 {
extern int wavelet_(integer *n, doublereal *y, doublereal *dt, 
			integer *mother, doublereal *param, doublereal *s0, doublereal *dj, 
   integer *jtot, integer *npad, doublecomplex *shwave, doublereal *
		   scale, doublereal *period, doublereal *coi, doublereal *psi0, 
     doublereal *cd, doublecomplex *wave);
	}

double kurtosis(double *x, long n);
void denoise(double *insig, double *outsig, long ndata, double percent, double deltat);


extern "C" QplugReturnStatus  WaveletDenoise (QplugArgList *);

using namespace QSAS;

QplugReturnStatus  WaveletDenoise  (QplugArgList *call_list)
{
  bool vectorset=FALSE;
  long i,k,ndata;
  char dummy[80], *veccomp[3]={"x","y","z"};
  double percent = 0.0, deltat=4.0, kurt=0.0;
  double *insig,*detsig,*stosig;

  QdTimeInterval_var start_end;

  QdObject_var       ts_out;
  QdTimeSeq_var      tt_out;

  QdRScalar_var      threshold;
  QdRScalarSeq_var   det_out, sto_out, ts_todo;
  QdRMatrixSeq_var   det_outV, sto_outV, ts_todoV;

  printf("--> Entering PlugIn wrapper...\n"); fflush(stdout);

  // ------- get input data series and time res -----

  QdObject_var ts_in = (*call_list)[0];
  QdTimeSeq_var tt_in = get_timetags(ts_in);

  if (tt_in.is_nil()) {
    printf(" tt is NULL\n");fflush(stdout);
    QplugAppendTextDisplay(" Failed to get timetags\n");
    return QPLUG_FAILURE;
  }

  if (!QuIsTSRegular(tt_in,&deltat)) {
   QplugAppendTextDisplay ("Warning ! - Irregular time tags.\n");
  }

  vectorset = QuIsThreeVector(ts_in);


  // ----- get timetags and mask according to this --------

  QdObject_var dobj1 = (*call_list)[1];
  start_end = get_timeinterval(dobj1);

  QplugAppendTextDisplay ("Subsetting Time Series...\n");
  SequenceMask msk(tt_in->get_mask(QdTimeSeq::InRange(start_end->start(), start_end->end())));
 
  tt_out = new QdTimeSeq();
  tt_in->get_subsequence(*tt_out, msk);

  if (!vectorset) {
    QdRScalarSeq_var ss = QdRScalarSeq_var::narrow(ts_in);
    QdRScalarSeq_var ss_out = new QdRScalarSeq();
    ss->get_subsequence(*ss_out,msk);
    ts_out = (QdObject_var) ss_out;
  }
  else {
    QdRMatrixSeq_var vs = QdRMatrixSeq_var::narrow(ts_in);
    if (QuIsThreeVector(vs)) {
      QdRMatrixSeq_var vs_out = new QdRMatrixSeq();
      vs->get_subsequence(*vs_out,msk);
      ts_out = (QdObject_var) vs_out;
    }
    else {
      QplugAppendTextDisplay ("ERROR - sequence type not handled\n");
      return QPLUG_FAILURE; 
    }
  }
      

  ndata = ts_out->sequence_size();
  const long NMAX=10000;    //  even 5000 should be OK;

  if (ndata > NMAX) {
    QplugAppendTextDisplay(" Too many recs; try shorter time interval\n\n");
    return QPLUG_FAILURE;
  }
 
 // ------ get threshold level -------

  QdObject_var dobj2 = (*call_list)[2];
  threshold = QdRScalar_var::narrow(dobj2);
  percent = (double) (*threshold);
  if (percent > 100.0 || percent < 0.0) {
    printf(" Threshold level out of range\n");fflush(stdout);
    QplugAppendTextDisplay(" Threshold level out of range\n");
    return QPLUG_FAILURE;
  }



  // ------ allocating memory for plugin arrays -------

   insig  = new double[ndata];
   detsig = new double[ndata];
   stosig = new double[ndata];
   

   // ------ scalar input -> scalar output ------------------

   if (!vectorset) {
     det_out = new QdRScalarSeq(ndata);
     sto_out = new QdRScalarSeq(ndata);
     ts_todo = QdRScalarSeq_var::narrow(ts_out);

     for (i=0; i<ndata; i++) {
        insig[i] = ts_todo[i];
        detsig[i] = ts_todo[i];
     }

     denoise(insig,detsig,ndata,percent,deltat);
     printf("Creating det_out and sto_out...\n");fflush(stdout);
     printf("Transfering to QSAS variable...\n");fflush(stdout);

     for (i=0; i<ndata; i++) {
        det_out[i] = detsig[i];
        sto_out[i] = insig[i] - detsig[i];
        stosig[i] = sto_out[i];
     }
     kurt = kurtosis(stosig,ndata); 
     sprintf(dummy,"Scalar : Kurtosis of stochastic part = %10.3f\n",kurt);
     QplugAppendTextDisplay(dummy);
     printf("Outsig transfered to det_out...%f\n",kurt);fflush(stdout);
     set_timetags(det_out, tt_out);
     set_timetags(sto_out, tt_out);
     QuSetFrame((QdObject_var) det_out, "scalar");
     QuSetTxtAttr("UNITS",QuGetAttrText("UNITS",ts_in),(QdObject_var) det_out);
     QuSetTxtAttr("LABLAXIS",QuGetAttrText("LABLAXIS",ts_in),(QdObject_var) det_out);
     sto_out->copy_xrefs_from(det_out);
     call_list->push_back((QdObject_var) det_out);
     call_list->push_back((QdObject_var) sto_out);
  }

  // -------- vector input -> vector output ----------------------

  else {   
    ts_todoV = QdRMatrixSeq_var::narrow(ts_out);
    det_outV = new QdRMatrixSeq(ndata,RMatrix(Dim(3)));
    sto_outV = new QdRMatrixSeq(ndata,RMatrix(Dim(3)));
    for (k=0; k<3;k++) {
      for (i=0; i<ndata; i++) {
      insig[i] = ts_todoV[i][k];
      detsig[i] = ts_todoV[i][k];
      }
      denoise(insig,detsig,ndata,percent,deltat);
      for (i=0; i<ndata; i++) {
        det_outV[i][k] = detsig[i];
        sto_outV[i][k] = insig[i] - detsig[i];
        stosig[i] = sto_outV[i][k];
      }
      kurt = kurtosis(stosig,ndata); 
      sprintf(dummy,"%s component : Kurtosis of stochastic part = %10.3f\n",veccomp[k],kurt);
      QplugAppendTextDisplay(dummy);
      set_timetags(det_outV, tt_out);
      set_timetags(sto_outV, tt_out);
      QuSetFrame((QdObject_var) det_outV, "vector", QuGetFrame(ts_in), QuGetRep(ts_in));
      QuSetTxtAttr("UNITS",QuGetAttrText("UNITS",ts_in),(QdObject_var) det_outV);
      QuSetTxtAttr("LABLAXIS",QuGetAttrText("LABLAXIS",ts_in),(QdObject_var) det_outV);
      sto_outV->copy_xrefs_from(det_outV);
      call_list->push_back((QdObject_var) det_outV);
      call_list->push_back((QdObject_var) sto_outV);
    }
  }
      
  printf("<-- Returning to QSAS...\n");fflush(stdout);

  return QPLUG_SUCCESS;
}  



long find2n(long n)
/* -----------------------------------------------------------------
  Returns the 2^m number closest to n (2^m >=n
  Header file math.h must be included in source. 

  SEH, 26 Jan 2001
--------------------------------------------------------------------*/
{
  long i,n1=0;
    
  for (i=1; n1<n; i++) n1 = (long) pow(2.0, (double)i);
  return n1;
}



doublecomplex **dcarray(long nrow, long ncol)
/* -----------------------------------------------------------------
  Allocates memory space for a double complex matrix. See also
  NR page 944.

  SEH, 22 Jan 2001
--------------------------------------------------------------------*/
{
  long i,j;
  doublecomplex **m;

  m = (doublecomplex **) malloc((size_t) ((nrow)*sizeof(doublecomplex*)));
  m[0] = (doublecomplex *) malloc((size_t)((nrow*ncol)*sizeof(doublecomplex)));

  for (i=1; i<nrow; i++) m[i] = m[i-1]+ncol;
  return m;
}




double dowave(wparam Ui, doublecomplex **wave, double *y, double *scale, 
	    double *period, double *coi)
/* -----------------------------------------------------------------
  Calls the Wavelet routine and returns the wavelet coefficients

  SEH, 22 Jan 2001
--------------------------------------------------------------------*/
{
  long i,j,k=0;
  double cd=0.0,psi=0.0, cfac,aa,bb;
  doublecomplex *wavesh;

  printf("--> Entering dowave...\n");fflush(stdout);
  
  wavesh = new doublecomplex[Ui.n*Ui.s];
  doublecomplex * temp_wave=new doublecomplex[Ui.n*Ui.s];
  
  wavelet_(&Ui.n,y,&Ui.dt,&Ui.base,&Ui.wno,&Ui.s0,&Ui.dj,&Ui.s,&Ui.npad,wavesh,scale,period,coi,&cd,&psi,temp_wave);
  
  printf("--> Entering dowave...\n");fflush(stdout);
  for (i=0; i<Ui.n; i++) {
    for (j=0; j<Ui.s; j++) {
      wave[i][j].r = wavesh[k].r;
      wave[i][j].i = wavesh[k].i;
      k++;
    }
  }
  delete[] wavesh;
  aa = 1.0 * cd;
  bb = 1.0 * psi;
  cfac = Ui.dj*sqrt(Ui.dt) / (cd * psi);

  printf("<-- Returning from dowave...\n");fflush(stdout);
  return(cfac);
}




double wabs(doublecomplex x)
/* -----------------------------------------------------------------
 Returns the amplitude of the complex number x

 wabs = sqrt(x.real^2 + x.imag^2)

  SEH, 05 Feb 2001
--------------------------------------------------------------------*/
{
  return(sqrt(x.r*x.r + x.i*x.i));
}



double *recon(doublecomplex **wave, double *scale, long ntimes, long nscale, double percent)
/* -----------------------------------------------------------------
  Removes (sets to zero) the 'percent' % lowest coefficients

  Inputs
     wave - array of wavelet coefficients (complex)
     ntimes - number of time steps (columns) in 'wave'
     nscale - number of scales (rows) in 'wave'
     percent - threshold level (range = 0..100 [%])

  SEH, 05 Feb 2001
--------------------------------------------------------------------*/

{
  long i,j,k;
  double wlen,threshold,maxcoeff=-9999.0,mincoeff=9999.0;
  double *denoised;

  printf("--> Entering recon...\n");fflush(stdout);
  denoised = new double[ntimes];
  
  /* find min and max coeff */  
  for (i=0; i<ntimes; i++) {
    for (j=0; j<nscale; j++) {
      wlen = wabs(wave[i][j]);
      if (wlen > maxcoeff) maxcoeff = wlen;
      if (wlen < mincoeff) mincoeff = wlen;
    }
  }
  
  /*calculate threshold */
  threshold = percent * (maxcoeff-mincoeff) / 100.0;

  /* remove (set to 0) coeffs below threshold */
  for (i=0; i<ntimes; i++) {
    denoised[i] = 0.0;
    for (j=0; j<nscale; j++) {
      if (wabs(wave[i][j]) < threshold) {
	wave[i][j].r = 0.0;
	wave[i][j].i = 0.0;
      }
    }
  }


  /* transform back */
  for (i=0; i<ntimes; i++) {
    for (j=0; j<nscale; j++) {
      denoised[i] +=  (wave[i][j].r / sqrt(scale[j]));
    }
  }
  for (i=0; i<10; i++) {printf("%10.4f\n",denoised[i]);fflush(stdout);}
  printf("<-- Returning from recon...\n");  fflush(stdout);
  return (denoised);
  
}



double mean(double *x, long n)
/* -----------------------------------------------------------------
  Returns the mean of the input signal x

  SEH, 6 Feb 2001
--------------------------------------------------------------------*/
{
  long i;
  double sum=0.0;

  for (i=0; i<n; i++) sum += x[i];
  
  return(sum / (double) n);
}



double kurtosis(double *x, long n)
/* -----------------------------------------------------------------
  Returns the kurtosis of the input signal x

  SEH, 8 Jun 2001
--------------------------------------------------------------------*/
{
  long i;
  double part=0.0,sum=0.0, ave=0.0, dev=0.0, var=0.0;
  double skew=0.0,kurt=0.0,sdev=0.0,psum=0.0;

  for (i=0; i<n; i++) sum+=x[i];
  ave = sum/n;

  for (i=0; i<n; i++)  {
      dev  += fabs(sum=x[i]-ave);
      var  += (part = sum*sum);
      skew += (part *= sum);
      kurt += (part *= sum);
      psum += sum;
  }
  dev /= n;
  var  = (var-psum*psum/n)/ (n-1);
  sdev = sqrt(var);
  if (var > 0.0) {
     skew /= (n*var*sdev);
     kurt  = (kurt /(n*var*var))-3.0;
  } else kurt = -999999.0;
  
  return(kurt);
}



void denoise(double *insig, double *outsig, long ndata, double percent, double deltat)
/* ------------------------------------------------------------------------
  Returns a denoised version of 'insig'

  SEH, 6 Feb 2001
  -------------------------------------------------------------------------*/  
{
  long i,j,Zeropad=1,Verbose=1;
  double *signal,*denoised,*scale,*period,*coi,*cd,*psi;
  double sfac=0.0,sigMean=0.0;
  wparam UserIn;

  printf("--> Entering denoise...\n");fflush(stdout);
  UserIn.n = ndata;
  UserIn.s = 46;
  UserIn.dt = deltat;
  UserIn.base = 0;
  UserIn.wno = 6.0;
  UserIn.dj = 0.25;
  UserIn.s0 = 2.0*UserIn.dt;
  UserIn.npad = UserIn.n;
  if (Zeropad)
    UserIn.npad=find2n(UserIn.n);


 if (Verbose) {
    printf("Ndata    %d\n",ndata);
    printf("UserIn.N %d\n",UserIn.n);
    printf("N scale  %d\n",UserIn.s);
    printf("Delta t  %f\n",UserIn.dt);
    printf("WaveBase %d\n",UserIn.base);
    printf("WaveNo   %f\n",UserIn.wno);
    printf("Delta j  %f\n",UserIn.dj);
    printf("Scale s0 %f\n",UserIn.s0);
    printf("Zeropad  %d\n",UserIn.npad);
    printf("\n");
	fflush(stdout);
  }

  // lallocate memory for tables 
  signal = new double[UserIn.n]; 
  denoised = new double[UserIn.n];
  period = new double[UserIn.s]; 
  scale  = new double[UserIn.s]; 
  coi    = new double[UserIn.n];

  doublecomplex **wave;
  wave = new doublecomplex*[UserIn.n];
  for (i=0; i<=UserIn.n-1; i++) wave[i] = new doublecomplex[UserIn.s];

  for (i=0; i<UserIn.n; i++) {
    signal[i] = insig[i];
    sigMean += signal[i];
  }
  sigMean = sigMean / (double) UserIn.n;

  printf("Before call to dowave....\n");fflush(stdout);

  sfac=dowave(UserIn,wave,signal,scale,period,coi);

  // reconstruction 
  if (Verbose) {
    printf("Sfac = %f %f %f\n",sfac,UserIn.dj,UserIn.dt);
	fflush(stdout);
  }

  denoised = recon(wave,scale,UserIn.n,UserIn.s,percent);
  for (i=0; i<UserIn.n; i++) outsig[i] = sfac*denoised[i]+sigMean;
  
  // free memory used by tables
  delete[] signal;
  delete[] denoised;
  delete[] period;
  delete[] scale; 
  delete[] coi; 
  delete [] wave[0];
  delete [] wave;

  

  printf("<-- Returning from denoise ...\n");
	fflush(stdout);
}




