// qpui.cc
//
// Plot Interface Implementation - Plot ListView components
//
// Janet Barnes, AJA
//
// July 2002, July 2007
//
//  Tony Allen
//  Feb 2008

#include "qpui.h"
#include "qwui.h"
#include "qdutil.h"

extern qwuiWindow* qsasWindow;
extern QPalette QPUI_normal_palette;

/// \var  extern vector<QguListItem *> dragList
/// \brief Global variable that contains the list of items being dragged
///
extern vector<QguListItem *> dragList;

/// \var  extern QguListView * dragFromList
/// \brief Global variable that defines the list view from which the items are being dragged
///
extern QguListView * dragFromList;


//     QpPlotItem CLASS
//     ------------------

QpPlotItem::QpPlotItem(QpPanelItem *parent, QString name, QpPlotItem *after, QString WL):
    QpListItem(parent,name,after,ListTypeEntry,WL)
{
	// Constructor for Plot item which is a child of a Panel
	// Trace and Spectrum Items inherit from this class
	I_Panel = parent;
	nameLock = false;

}

QpPlotItem::~QpPlotItem()
{
	I_ListView->removeLink(this);
}

// ----------------------------------

void QpPlotItem::setLinks(list <QpPlotItem*> linkList)
{
	linkedPlots = linkList;
}

// ----------------------------------

void QpPlotItem::removeFromLinks()
{
	I_ListView->removeLink(this);
	linkedPlots.clear();
}

// ----------------------------------

void QpPlotItem::removeLink(QpPlotItem* plot_item)
{
	linkedPlots.remove(plot_item);	
}

// ------------------------------------------

QpFrameType QpPlotItem::frameType()
{
	if(this->I_Panel ) return I_Panel->frameType();

}

// ------------------------------------------


//     QpListView CLASS
//     ------------------

QpListView::QpListView(QWidget *parent, QpWindow *window, const char *name)
   :  QguListView(parent,name)
{
	// Constructor 
  	if(QuSlotDebug())
	{
		printf("QpListView::QpListView %ld\n", this);
		fflush(stdout);
	}

	// New Sub-Menu
	Q_POPUP_MENU* new_sub_menu = new Q_POPUP_MENU(this);
	new_sub_menu->clear();
	new_sub_menu->insertItem("Trace",this,SLOT(makeTraceItem()));
	new_sub_menu->insertItem("SpectroGram",this,SLOT(makeSpectrumItem()));
	new_sub_menu->insertSeparator();
	new_sub_menu->insertItem("Constant X or T Trace",this,SLOT(makeConstXTraceItem()));
	new_sub_menu->insertItem("Constant Y Trace",this,SLOT(makeConstYTraceItem()));
	new_sub_menu->insertSeparator();
	new_sub_menu->insertItem("Event Label",this,SLOT(makeEventItem()));
	new_sub_menu->insertSeparator();
	new_sub_menu->insertItem("Panel",this,SLOT(makePanelItem()));
	new_sub_menu->insertSeparator();
	new_sub_menu->insertItem("Time Series Frame",this,SLOT(makeTSFrameItem()));
	new_sub_menu->insertItem("XY Frame",this,SLOT(makeXYFrameItem()));
	new_sub_menu->insertSeparator();
	new_sub_menu->insertItem("Page",this,SLOT(makePageItem()));

	// Right Button Sub-Menu
	RButtonMenu=new Q_POPUP_MENU(this);
	RButtonMenu->insertItem("&New...", new_sub_menu, Qt::CTRL+Qt::Key_N);
	RButtonMenu->insertSeparator();
	RButtonMenu->insertItem("Rename", this, SLOT(renameSelectedItem()));
	RButtonMenu->insertItem("&Copy", this, SLOT(copy()), Qt::CTRL+Qt::Key_C);
	RButtonMenu->insertItem("P&aste", this, SLOT(paste()), Qt::CTRL+Qt::Key_V);
	RButtonMenu->insertSeparator();
	RButtonMenu->insertItem("Move Up", this, SLOT(moveItemUp()), Qt::CTRL+Qt::Key_Up);
	RButtonMenu->insertItem("Move Down", this, SLOT(moveItemDown()), Qt::CTRL+Qt::Key_Down);
	RButtonMenu->insertSeparator();
	RButtonMenu->insertItem("Plot Page", this, SLOT(plotCurrentItem()));
	RButtonMenu->insertSeparator();
	RButtonMenu->insertItem("Use Default Settings", this, SLOT(setDefaultCurrentItem())); 
	RButtonMenu->insertSeparator();
	RButtonMenu->insertItem("&Delete", this, SLOT(deleteOneItem()));
	RButtonMenu->insertSeparator();
	RButtonMenu->insertItem("Link Selected Traces", this, SLOT( linkPlots() ));
	RButtonMenu->insertItem("Unlink Selected Traces", this, SLOT( unlinkPlots() ));

	setAcceptDrops(true);
	connect(this,SIGNAL(selectionModified()),this, SLOT(updateOnSelection()));
	connect(this,SIGNAL(pageLayoutChanged(QpPageItem*)), this,SLOT(setPageLayout(QpPageItem*)));

	setMinimumHeight(300);
	
	isLink = false; // global available to every DataTab for use in linking
	LinkNumber=0;

	//vectorPlot = getenv("Q_PLOT_VECTOR");
	//array3dPlot = getenv("Q_PLOT_3D_ARRAY");
}

// ---------------------------------

void QpListView::keyPressEvent( QKeyEvent* e )
{
	QpListItem*  selected_item  = NULL;
	setFocus();

	if (    ( e->key() == Qt::Key_Up       )
	     || ( e->key() == Qt::Key_Down     )
		 || ( e->key() == Qt::Key_Home     )
		 || ( e->key() == Qt::Key_End      )
		 || ( e->key() == Qt::Key_PageDown )
		 || ( e->key() == Qt::Key_PageUp   )    )
	{
		Q_LIST_VIEW::keyPressEvent( e );

		selected_item = ( QpListItem * )currentItem();

		if ( selected_item == NULL )
		{
			clearSelectedList();
		}
		else
		{
			updateOnSelection();
		}
	}
	else
	{
		// else pass on to list view
		e->ignore(); // let selection change
	}

}

// ---------------------------------

void QpListView::setPlotOption(int option, bool bChoice)
{
	if(QuSlotDebug()) cout << "QpListView::setPlotOption" << endl;

	// apply Trace option to all selected traces (that belonging to visible tab is always selected)
	for (int i=0; i<localList.size(); i++)
	{
		// only modify plot items
		QpPlotItem * pl_item = dynamic_cast<QpPlotItem*>(localList[i]);
    
		if( !pl_item) continue;
	  
		// don't set option for visible tab - user just did this - avoid infinite loop
		if(pl_item->DataTab->isVisible()) continue;
	  
	  
		switch(option){
			// Data Tab
			default: case USE_TIME_AS_SEC:
			{
				pl_item->DataTab->useTimeAsSec(bChoice);
				break;
			}
		}
	}
}

// ---------------------------------

void QpListView::setTraceOption(int option, int iChoice, float fChoice, QString sChoice, bool bChoice, QColor cChoice)
{
	if(QuSlotDebug()) cout << "QpListView::setTraceOption" << endl;

   // apply Trace option to all selected traces (that belonging to visible tab is always selected)
   for (int i=0; i<localList.size(); i++)
   {

      // only modify trace items
      QpTraceItem * tr_item = dynamic_cast<QpTraceItem*>(localList[i]);
    
      if( !tr_item) continue;
	  
	  // don't set trace for visible tab - user just did this - avoid infinite loop
	  if(currentItem() == tr_item) continue;
	  if(tr_item->TraceTab->isVisible()) continue;
	  if(tr_item->DataTab->isVisible()) continue;
	  
	  
	  switch(option){
		// Trace Tab
		case TRACE_LINE_STYLE:
		{
			tr_item->TraceTab->_line_style_fld->setCurrentItem(iChoice);
			break;
		}
		case TRACE_LINE_WIDTH:
		{
			tr_item->TraceTab->_line_width_fld->setValue(fChoice);
			break;
		}
		case TRACE_CONNECT_STYLE: 
		{
			tr_item->TraceTab->_connect_style_fld->setCurrentItem(iChoice);
			break;
		}
		case TRACE_LINE_COLOUR: 
		{
			QPalette line_palette = QPalette(QPUI_normal_palette);
			line_palette.setColor(QPalette::Active, QColorGroup::ButtonText, cChoice);
			line_palette.setColor(QPalette::Inactive, QColorGroup::ButtonText, cChoice);
			tr_item->TraceTab->_line_colour_fld->setPalette(line_palette);
			tr_item->_line_colour = cChoice;
			break;
		}
		case TRACE_SYMBOL_SHAPE:
		{ 
			tr_item->TraceTab->_symbol_shape_fld->setCurrentItem(iChoice);
			break;
		}
		case TRACE_SYMBOL_SIZE: 
		{
			tr_item->TraceTab->_symbol_size_fld->setValue(fChoice);
		}
		case TRACE_SYMBOL_COLOUR: 
		{
			QPalette line_palette = QPalette(QPUI_normal_palette);
			line_palette.setColor(QPalette::Active, QColorGroup::ButtonText, cChoice);
			line_palette.setColor(QPalette::Inactive, QColorGroup::ButtonText, cChoice);
			tr_item->TraceTab->_symbol_colour_fld->setPalette(line_palette);
			tr_item->_symbol_colour = cChoice;
			break;
		}
		// Data Tab
		case TRACE_LEGEND_TEXT:
		{
			tr_item->DataTab->_legend_text_fld->setText(sChoice);
			break;
		}
		case TRACE_SHOW_LEGEND:
		{
			tr_item->DataTab->_show_legend_fld->setChecked(iChoice);
			break;
		}
		case TRACE_YDATA:
		{
			tr_item->replaceData(sChoice);
			break;
		}
		case TRACE_YCOMP:
		{
			tr_item->replaceComponent(sChoice);
			break;
		}
		case TRACE_XDATA:
		{
			tr_item->replaceXData(sChoice);
			break;
		}
		case TRACE_XCOMP:
		{
			tr_item->replaceXComponent(sChoice);
			break;
		}
		case LEGEND_AUTO_TEXT:
		{
			tr_item->DataTab->_auto_legend_fld->setChecked(bChoice);
			break;
		}

		default:
			  break;
			  
	  }
   } // end for over local list
}


void QpListView::setLegendOption(int option, int iChoice, float fChoice, QString sChoice, bool bChoice, QColor cChoice)
{
	if(QuSlotDebug()) cout << "QpListView::setLegendOption" << endl;

	// Slot called when value changed on legend tab

	// apply Legend option to all selected panels (that belonging to visible tab is always selected)
	for (int i=0; i<localList.size(); i++)
	{
		// only modify panel items
		QpPanelItem * panel_item = dynamic_cast<QpPanelItem*>(localList[i]);
    
		if( !panel_item) continue;
		
		if(panel_item->LegendTab->isVisible()) continue;
	  
		switch(option){
		
			case SHOW_LEGEND:
			{
				panel_item->LegendTab->_show_legend_fld->setChecked(bChoice);
				break;
			}
			case LEGEND_BORDER:
			{
				panel_item->LegendTab->_legend_border_fld->setCurrentItem(iChoice);
				break;
			}
			case LEGEND_NUM_ROWS:
			{
				panel_item->LegendTab->_legend_num_rows_fld->setValue(iChoice);
				int num_traces = panel_item->numberTraces();
				int num_rows = iChoice;
				int num_cols = (int)ceil((float)num_traces / num_rows);
				panel_item->LegendTab->_legend_num_cols_fld->setValue(num_cols);
				break;
			}
			case LEGEND_NUM_COLS:
			{
				panel_item->LegendTab->_legend_num_cols_fld->setValue(iChoice);
				int num_traces = panel_item->numberTraces();
				int num_cols = iChoice;
				int num_rows = (int)ceil((float)num_traces / num_cols);
				panel_item->LegendTab->_legend_num_rows_fld->setValue(num_rows);
				break;
			}
			case LEGEND_CHAR_SIZE:
			{
				panel_item->LegendTab->_legend_char_size_fld->setValue(fChoice);
				break;
			}
			case LEGEND_MAX_CHAR_LENGTH:
			{
				panel_item->LegendTab->_legend_max_char_length_fld->setValue(iChoice);
				break;
			}
			case LEGEND_LINE_LENGTH:
			{
				panel_item->LegendTab->_legend_line_length_fld->setValue(fChoice);
				break;
			}
			case LEGEND_JUST:
			{
				panel_item->LegendTab->_legend_just_fld->setCurrentItem(iChoice);
				break;
			}
			case LEGEND_FONT_FAMILY:
			{
				switch(iChoice)
				{
					case PL_FCI_SERIF:
						panel_item->LegendTab->_legend_font_fld->setSerifFont();
						break;
					case PL_FCI_MONO:
						panel_item->LegendTab->_legend_font_fld->setMonotypeFont();
						break;
					case PL_FCI_SCRIPT:
						panel_item->LegendTab->_legend_font_fld->setDecorativeFont();
						break;
					default:
						panel_item->LegendTab->_legend_font_fld->setSansFont();
						break;
				}
				break;
			}
			case LEGEND_FONT_STYLE:
				panel_item->LegendTab->_legend_font_fld->setItalic(bChoice);
				break;
			case LEGEND_FONT_WEIGHT:
				panel_item->LegendTab->_legend_font_fld->setBold(bChoice);
				break;
			case LEGEND_ALIGN:
			{
				panel_item->LegendTab->_legend_align_fld->setCurrentItem(iChoice);
				break;
			}
			case LEGEND_L_OFFSET:
			{
				panel_item->LegendTab->_legend_loffset_fld->setValue(fChoice);
				break;
			}
			case LEGEND_T_OFFSET:
			{
				panel_item->LegendTab->_legend_toffset_fld->setValue(fChoice);
				break;
			}
			case LEGEND_BACKGROUND:
			{
				panel_item->LegendTab->_legend_background_fld->setCurrentItem(iChoice);
				break;
			}
			case LEGEND_WIDTH:
			{
				panel_item->LegendTab->_legend_width_fld->setValue(fChoice);
#if QT_VERSION < 0x040000
				switch(panel_item->LegendTab->_legend_align_fld->currentItem())
#else
				switch(panel_item->LegendTab->_legend_align_fld->currentIndex())	
#endif
				{
					case QPUI_INSIDE_TOP_RIGHT:
					case QPUI_INSIDE_BOTTOM_RIGHT:
						panel_item->LegendTab->_legend_width_fld->setValue(1.-fChoice);
						break;
						
					case QPUI_ABOVE_TOP_CENTRE:
						panel_item->LegendTab->_legend_width_fld->setValue(0.5-fChoice/2.);
						break;
					
					default:
						break;
				}
				break;
			}
			case LEGEND_HEIGHT:
			{
				panel_item->LegendTab->_legend_height_fld->setValue(fChoice);
#if QT_VERSION < 0x040000
				switch(panel_item->LegendTab->_legend_align_fld->currentItem())
#else
					switch(panel_item->LegendTab->_legend_align_fld->currentIndex())	
#endif
				{
					case QPUI_INSIDE_BOTTOM_LEFT:
					case QPUI_INSIDE_BOTTOM_RIGHT:
						panel_item->LegendTab->_legend_height_fld->setValue(1.-fChoice);
						break;
					
					default:
						break;
				}
				break;
			}
		}
	} // end for over selected list
}

// --------------------------------------------------------

void QpListView::setAxisOption(int option, int axis, int iChoice, float fChoice, QString sChoice, bool bChoice, QColor cChoice)
{
   // Slot called when value changed on axis option tabs
   // axis identifies which of X,Y or T is changed.
	
	if(QuSlotDebug()) cout << "QpListView::setAxisOption" << endl;

	QpAxisTab * axisTab = 0;
	QpListItem * listItem;
   // apply axis option to all selected frames or panels (that belonging to visible tab is always selected)
   for (int i=0; i<localList.size(); i++)
   {
		// if Y axis only modify panel items
		if(axis == Y_AXIS){
			QpPanelItem * panel_item = dynamic_cast<QpPanelItem*>(localList[i]);
			if( !panel_item) continue;
			
			listItem = (QpListItem *) panel_item;
			axisTab = panel_item->yAxisTab;
		}
		else if(axis == X_AXIS){
			QpFrameItem * frame_item = dynamic_cast<QpFrameItem*>(localList[i]);
			if( !frame_item) continue;
			listItem = (QpListItem *) frame_item;

			axisTab = frame_item->xAxisTab;
		}
		else if(axis == T_AXIS){
			QpFrameItem * frame_item = dynamic_cast<QpFrameItem*>(localList[i]);
			if( !frame_item) continue;
			listItem = (QpListItem *) frame_item;
			
			axisTab = frame_item->tAxisTab;
		}
		else break;
		
		// don't set axis for visible tab - user just did this - avoid infinite loop

		if(axisTab->isVisible()) continue;

		switch(option){
			case AXIS_AUTOSCALE:	
			{
				axisTab->_axis_autoscale_fld->setCurrentItem(iChoice);
				axisTab->setAxisMinMax();
				break;
			}
			case AXIS_LINLOG:	
			{
				// only called for X or Y axis	
				axisTab->_axis_linlog_fld->setCurrentItem(iChoice);
				axisTab->_axis_autoscale_fld->setCurrentItem(QP_FROM_USER);

				break;
			}
			case AXIS_MIN:
			{
				axisTab->_axis_autoscale_fld->setCurrentItem(QP_FROM_USER);
				if(axisTab->I_axis!=T_AXIS)
				{
					axisTab->_axis_from_xy_fld->setText(sChoice);
					axisTab->setSlotColour(true);
					axisTab->_axis_from_xy_fld->lineBox->setPalette(axisTab->GUIpalette);
					axisTab->_axis_to_xy_fld->lineBox->setPalette(axisTab->GUIpalette);
				}
				else
				{
					axisTab->_axis_from_time_fld->setTime(Time(sChoice.latin1()));
					axisTab->setSlotColour(true);
					axisTab->_axis_from_time_fld->setPalette(axisTab->GUIpalette);
					axisTab->_axis_to_time_fld->setPalette(axisTab->GUIpalette);
					axisTab->setSlotColour(false);
					axisTab->_t_data_fld->lineBox->setPalette(axisTab->GUIpalette);
				}
				break;
			}
			case AXIS_MAX:
			{
				axisTab->_axis_autoscale_fld->setCurrentItem(QP_FROM_USER);
				if(axisTab->I_axis!=T_AXIS)
				{
					axisTab->_axis_to_xy_fld->setText(sChoice);
					axisTab->setSlotColour(true);
					axisTab->_axis_from_xy_fld->lineBox->setPalette(axisTab->GUIpalette);
					axisTab->_axis_to_xy_fld->lineBox->setPalette(axisTab->GUIpalette);
				}
				else
				{
					axisTab->_axis_to_time_fld->setTime(Time(sChoice.latin1()));
					axisTab->setSlotColour(true);
					axisTab->_axis_from_time_fld->setPalette(axisTab->GUIpalette);
					axisTab->_axis_to_time_fld->setPalette(axisTab->GUIpalette);
					axisTab->setSlotColour(false);
					axisTab->_t_data_fld->lineBox->setPalette(axisTab->GUIpalette);
				}
				break;
			}
			case AXIS_NUM_MAJOR_TICKS:			
			{
				axisTab->_axis_major_ticks_num_fld->setValue(iChoice);
				break;
			}
			case AXIS_NUM_MINOR_TICKS:			
			{
				axisTab->_axis_minor_ticks_num_fld->setValue(iChoice);
				break;
			}
			case AXIS_MAJOR_TICKMARKS:			
			{
				axisTab->_axis_major_tickmarks_fld->setCurrentItem(iChoice);
				break;
			}
			case AXIS_MINOR_TICKMARKS:			
			{
				axisTab->_axis_minor_tickmarks_fld->setCurrentItem(iChoice);
				break;
			}
			case AXIS_LABEL:			
			{
				axisTab->_label_fld->setText(sChoice);
				break;
			}
			case AXIS_LABEL_CHAR_SIZE:			
			{
				axisTab->_label_char_size_fld->setValue(fChoice);
				break;
			}
			case AXIS_LABEL_DISP:			
			{
				axisTab->_label_disp_fld->setValue(fChoice);
				break;
			}
			case AXIS_AUTOLABEL:			
			{
				axisTab->_axis_autolabel_fld->setChecked(bChoice);
				axisTab->setAutoLabelText();
				break;
			}
			case AXIS_LABEL_ROTATION:			
			{
				axisTab->_label_rotation_fld->setValue(fChoice);
				break;
			}
			case AXIS_LABEL_FONT_FAMILY:			
			{
				switch(iChoice)
				{
					case PL_FCI_SERIF:
						axisTab->_label_font_fld->setSerifFont();
						break;
					case PL_FCI_MONO:
						axisTab->_label_font_fld->setMonotypeFont();
						break;
					case PL_FCI_SCRIPT:
						axisTab->_label_font_fld->setDecorativeFont();
						break;
					default:
						axisTab->_label_font_fld->setSansFont();
						break;
				}
				break;
			}
			case AXIS_LABEL_FONT_STYLE:
				axisTab->_label_font_fld->setItalic(bChoice);
				break;
			case AXIS_LABEL_FONT_WEIGHT:
				axisTab->_label_font_fld->setBold(bChoice);
				break;
			case AXIS_LABEL_COLOUR:	
			{
				QPalette label_palette = QPalette(QPUI_normal_palette);
				label_palette.setColor(QPalette::Active, QColorGroup::ButtonText, cChoice);
				label_palette.setColor(QPalette::Inactive, QColorGroup::ButtonText, cChoice);
				axisTab->_label_colour_fld->setPalette(label_palette);
				axisTab->_label_colour = cChoice;
				break;
			}
			case AXIS_T_DATA:	
			{
				// only called for T axis
				axisTab->_t_data_fld->setText(sChoice);
				axisTab->_t_data_fld->reValidate();
				axisTab->setTfromObj();
				break;
			}
			case AXIS_TICK_TIME_FORMAT:
			{
				// only called for T axis
				axisTab->tick_time_fmt_fld->setText(sChoice);
				break;
			}
			case AXIS_TICK_TIME_FORMAT_AUTO:
			{
				// only called for T axis
				axisTab->tick_time_fmt_auto_fld->setChecked(iChoice);
				axisTab->tick_time_fmt_fld->setReadOnly(iChoice);
				axisTab->tick_time_fmt_fld->setEnabled(!iChoice);
				break;
			}
			case AXIS_TIME_LABELS_FORMAT:
			{
				// only called for T axis
				axisTab->time_labels_fmt_fld->setText(sChoice);
				break;
			}
			case AXIS_TIME_LABELS_FORMAT_AUTO:
			{
				// only called for T axis
				axisTab->time_labels_fmt_auto_fld->setChecked(iChoice);
				axisTab->time_labels_fmt_fld->setReadOnly(iChoice);
				axisTab->time_labels_fmt_fld->setEnabled(!iChoice);
				break;
			}
		}
	} // end for over local list
}

// ---------------------------------

void QpListView::setEventOption(int option, int iChoice, float fChoice, QString sChoice,  bool bChoice, QColor cChoice)
{
	if(QuSlotDebug()) cout << "QpListView::setEventOption" << endl;

	// apply Event option to all selected events (that belonging to visible tab is always selected)
	for (int i=0; i<localList.size(); i++)
	{
		// only modify event items
		QpEventItem * ev_item = dynamic_cast<QpEventItem*>(localList[i]);
    
		if( !ev_item) continue;
	  
		// don't set event for visible tab - user just did this - avoid infinite loop
		if(ev_item->eventTab->isVisible()) continue;
	  
		switch(option){
			case EVENT_Y_DATA:
				ev_item->eventTab->_y_data_fld->setText(sChoice);
				break;
			case EVENT_X_DATA:
				ev_item->eventTab->_x_data_fld->setText(sChoice);
				break;
			case EVENT_LABEL:
				ev_item->eventTab->_event_label_fld->setText(sChoice);
				break;
			case EVENT_LABEL_CHAR_SIZE: 
				ev_item->eventTab->_char_size_fld->setValue(fChoice);
			case EVENT_LABEL_FONT_FAMILY:
				switch(iChoice)
				{
					case PL_FCI_SERIF:
						ev_item->eventTab->_event_label_font_fld->setSerifFont();
						break;
					case PL_FCI_MONO:
						ev_item->eventTab->_event_label_font_fld->setMonotypeFont();
						break;
					case PL_FCI_SCRIPT:
						ev_item->eventTab->_event_label_font_fld->setDecorativeFont();
						break;
					default:
						ev_item->eventTab->_event_label_font_fld->setSansFont();
						break;
				}
				break;
			case EVENT_LABEL_FONT_STYLE:
				ev_item->eventTab->_event_label_font_fld->setItalic(bChoice);
				break;
			case EVENT_LABEL_FONT_WEIGHT:
				ev_item->eventTab->_event_label_font_fld->setBold(bChoice);
				break;
			case EVENT_LABEL_COLOUR: 
			{
				QPalette label_palette = QPalette(QPUI_normal_palette);
				label_palette.setColor(QPalette::Active, QColorGroup::ButtonText, cChoice);
				label_palette.setColor(QPalette::Inactive, QColorGroup::ButtonText, cChoice);
				ev_item->eventTab->_event_label_colour_fld->setPalette(label_palette);
				ev_item->_event_label_colour = cChoice;
				break;
			}
			default:
				break;
			  
		}
	} // end for over local list
}

// ---------------------------------

void QpListView::setSpectrumOption(int option, int iChoice, float fChoice, QString sChoice, bool bChoice, QColor cChoice)
{
	if(QuSlotDebug()) cout << "QpListView::setSpectrumOption" << endl;

   // apply Spectrum option to all selected spectra (that belonging to visible tab is always selected)
   for (int i=0; i<localList.size(); i++)
   {
      // only modify trace items
      QpSpectrumItem * sp_item = dynamic_cast<QpSpectrumItem*>(localList[i]);
    
      if( !sp_item) continue;
	  
	  // don't set spectrum for visible tab - user just did this - avoid infinite loop
	  if(sp_item->DataTab->isVisible()) continue;
	  if(sp_item->ColourscaleTab->isVisible()) continue;

	  switch(option){
		// Spectrum Data Tab
		case SPECTROGRAM_ZDATA:

			sp_item->replaceData(sChoice);

			break;
		case SPECTROGRAM_ZCOMP:

			sp_item->replaceComponent(sChoice);

			break;
		case SPECTROGRAM_YDATA_START:
			// sChoice will be either component selection or object base name
			if(sChoice.contains("["))
			{
				// change component
				QString newName = baseObjectText( sp_item->DataTab->_y_start_data_fld->text() );
				newName += sChoice;
				sp_item->DataTab->_y_start_data_fld->setText(newName);
			}
			else
			{
				sp_item->DataTab->_y_start_data_fld->setTextKeepComp(sChoice);
			}			
			break;
		case SPECTROGRAM_YDATA_END:
			// sChoice will be either component selection or object base name
			if(sChoice.contains("["))
			{
				// change component
				QString newName = baseObjectText( sp_item->DataTab->_y_end_data_fld->text() );
				newName += sChoice;
				sp_item->DataTab->_y_end_data_fld->setText(newName);
			}
			else
			{
				sp_item->DataTab->_y_end_data_fld->setTextKeepComp(sChoice);
			}	
			break;
		case SPECTROGRAM_XDATA_START:
			// sChoice will be either component selection or object base name
			if(sChoice.contains("["))
			{
				// change component
				QString newName = baseObjectText( sp_item->DataTab->_x_start_data_fld->text() );
				newName += sChoice;
				sp_item->DataTab->_x_start_data_fld->setText(newName);
			}
			else
			{
				sp_item->DataTab->_x_start_data_fld->setTextKeepComp(sChoice);
			}			
			break;
		case SPECTROGRAM_XDATA_END:
			// sChoice will be either component selection or object base name
			if(sChoice.contains("["))
			{
				// change component
				QString newName = baseObjectText( sp_item->DataTab->_x_end_data_fld->text() );
				newName += sChoice;
				sp_item->DataTab->_x_end_data_fld->setText(newName);
			}
			else
			{
				sp_item->DataTab->_x_end_data_fld->setTextKeepComp(sChoice);
			}	
			break;
		case COLOUR_SCALE_MIN:
			sp_item->ColourscaleTab->_colourscale_min_fld->setDouble(fChoice);
			sp_item->ColourscaleTab->_colourscale_auto_fld->setCurrentItem(QP_FROM_USER);
			sp_item->ColourscaleTab->setSlotColour(true);
			sp_item->ColourscaleTab->_colourscale_min_fld->lineBox->setPalette(sp_item->ColourscaleTab->GUIpalette);
			sp_item->ColourscaleTab->_colourscale_max_fld->lineBox->setPalette(sp_item->ColourscaleTab->GUIpalette);
			break;
		case COLOUR_SCALE_MAX:
			sp_item->ColourscaleTab->_colourscale_max_fld->setDouble(fChoice);
			sp_item->ColourscaleTab->_colourscale_auto_fld->setCurrentItem(QP_FROM_USER);
			sp_item->ColourscaleTab->setSlotColour(true);
			sp_item->ColourscaleTab->_colourscale_min_fld->lineBox->setPalette(sp_item->ColourscaleTab->GUIpalette);
			sp_item->ColourscaleTab->_colourscale_max_fld->lineBox->setPalette(sp_item->ColourscaleTab->GUIpalette);
			break;
		  case COLOUR_SCALE_TICK_LENGTH:
			  sp_item->ColourscaleTab->_colourscale_ticklength_fld->setValue(fChoice);
			  break;
		case COLOUR_SCALE_LABEL:
			sp_item->ColourscaleTab->_colourscale_label_fld->setText(sChoice);
			sp_item->ColourscaleTab->_colourscale_autolabel_fld->setChecked(false);
			break;
		case COLOUR_SCALE_AUTOSCALE:
			sp_item->ColourscaleTab->_colourscale_auto_fld->setCurrentItem(iChoice);
			sp_item->ColourscaleTab->setColourscaleMinMax();
			break;
		case COLOUR_SCALE_AUTOLABEL:
			sp_item->ColourscaleTab->_colourscale_autolabel_fld->setChecked(bChoice);
			sp_item->ColourscaleTab->setColourScaleAutoLabel();
			break;
		case COLOUR_SCALE_LINLOG:
			sp_item->ColourscaleTab->_colourscale_linlog_fld->setCurrentItem(iChoice);
			sp_item->ColourscaleTab->_colourscale_auto_fld->setCurrentItem(QP_FROM_USER);
			break;
		case COLOUR_SCALE_SHOW_COLOUR_BAR:
			sp_item->ColourscaleTab->_colourscale_show_colour_bar_fld->setChecked(bChoice);
			break;
		case COLOUR_SCALE_COLOUR_BAR_WIDTH:
			sp_item->ColourscaleTab->_colourscale_colour_bar_width_fld->setValue(fChoice);
			break;
		case COLOUR_SCALE_COLOUR_BAR_HEIGHT:
			sp_item->ColourscaleTab->_colourscale_colour_bar_height_fld->setValue(fChoice);
			break;
		case COLOUR_SCALE_COLOUR_BAR_VOFFSET:
			sp_item->ColourscaleTab->_colourscale_colour_bar_voffset_fld->setValue(fChoice);
			break;
		case COLOUR_SCALE_COLOUR_BAR_DISP:
			sp_item->ColourscaleTab->_colourscale_colour_bar_disp_fld->setValue(fChoice);
			break;
		case COLOUR_SCALE_TABLE:
			sp_item->ColourscaleTab->_colourscale_table_fld->setCurrentItem(iChoice);
			break;
		default:
			  break;
			  
	  }
   } // end for over local list
}

// -----------------------------------------

void QpListView::setFrameOption(int option, int iChoice, float fChoice, QString sChoice, bool bChoice, QColor cChoice)
{
	if(QuSlotDebug()) cout << "QpListView::setFrameOption" << endl;

   // apply Panel option to all selected panels (that belonging to visible tab is always selected)
   for (int i=0; i<localList.size(); i++)
   {
      // only modify panel items
      QpFrameItem * frame_item = dynamic_cast<QpFrameItem*>(localList[i]);
    
      if( !frame_item) continue;
	  
	  // don't set panel for visible tab - user just did this - avoid infinite loop
	  if(frame_item->PropertiesTab->isVisible()) continue;
	  if(frame_item->DisplayTab->isVisible()) continue;
	  if(frame_item->TitleTab->isVisible()) continue;
	  
	  switch(option){
		// Display Tab
		case PANEL_LINE_WIDTH:
			frame_item->DisplayTab->_line_width_fld->setValue(fChoice);
			break;
		case PANEL_LINE_COLOUR:
		{
			QPalette line_palette = QPalette(QPUI_normal_palette);
			line_palette.setColor(QPalette::Active, QColorGroup::ButtonText, cChoice);
			line_palette.setColor(QPalette::Inactive, QColorGroup::ButtonText, cChoice);
			frame_item->DisplayTab->_line_colour_fld->setPalette(line_palette);
			frame_item->_line_colour = cChoice;
			break;
		}
		case TICKS_LENGTH:
			frame_item->DisplayTab->_ticks_length_fld->setValue(fChoice);
			break;
		case TOP_AXIS:
			frame_item->DisplayTab->_show_top_axis_fld->setChecked(bChoice);
			break;
		case BOTTOM_AXIS:
			frame_item->DisplayTab->_show_bottom_axis_fld->setChecked(bChoice);
			break;
		case LEFT_AXIS:
			frame_item->DisplayTab->_show_left_axis_fld->setChecked(bChoice);
			break;
		case RIGHT_AXIS:
			frame_item->DisplayTab->_show_right_axis_fld->setChecked(bChoice);
			break;
		  case LR_ALTERNATING_AXIS:
			  frame_item->DisplayTab->_show_lr_alternating_axis_fld->setChecked(bChoice);
			  break;
		  case RL_ALTERNATING_AXIS:
			  frame_item->DisplayTab->_show_rl_alternating_axis_fld->setChecked(bChoice);
			  break;
		case TOP_TICKS_LABELS:
			frame_item->DisplayTab->_show_top_ticks_labels_fld->setChecked(bChoice);
			break;
		case BOTTOM_TICKS_LABELS:
			frame_item->DisplayTab->_show_bottom_ticks_labels_fld->setChecked(bChoice);
			break;
		case LEFT_TICKS_LABELS:
			frame_item->DisplayTab->_show_left_ticks_labels_fld->setChecked(bChoice);
			break;
		case RIGHT_TICKS_LABELS:
			frame_item->DisplayTab->_show_right_ticks_labels_fld->setChecked(bChoice);
			break;
		case LR_ALTERNATING_TICKS_LABELS:
			  frame_item->DisplayTab->_show_lr_alternating_ticks_labels_fld->setChecked(bChoice);
			  break;
		  case RL_ALTERNATING_TICKS_LABELS:
			  frame_item->DisplayTab->_show_rl_alternating_ticks_labels_fld->setChecked(bChoice);
			  break;
		case TOP_LABELS:
			frame_item->DisplayTab->_show_top_labels_fld->setChecked(bChoice);
			break;
		case BOTTOM_LABELS:
			frame_item->DisplayTab->_show_bottom_labels_fld->setChecked(bChoice);
			break;
		case LEFT_LABELS:
			frame_item->DisplayTab->_show_left_labels_fld->setChecked(bChoice);
			break;
		case RIGHT_LABELS:
			frame_item->DisplayTab->_show_right_labels_fld->setChecked(bChoice);
			break;
		case LR_ALTERNATING_LABELS:
			  frame_item->DisplayTab->_show_lr_alternating_labels_fld->setChecked(bChoice);
			  break;
		  case RL_ALTERNATING_LABELS:
			  frame_item->DisplayTab->_show_rl_alternating_labels_fld->setChecked(bChoice);
			  break;
		case Y_IS_ZERO:
			frame_item->DisplayTab->_show_yiszero_fld->setChecked(bChoice);
			break;
		case X_IS_ZERO:
			frame_item->DisplayTab->_show_xiszero_fld->setChecked(bChoice);
			break;
		//  Frame Properties Tab
		case LOFFSET:
			frame_item->PropertiesTab->_loffset_fld->setValue(fChoice);
			break;
		case TOFFSET:
			frame_item->PropertiesTab->_toffset_fld->setValue(fChoice);
			break;
		case WIDTH:
			frame_item->PropertiesTab->_width_fld->setValue(fChoice);
			if(frame_item->PropertiesTab->_aspect_ratio_fld->isChecked()) frame_item->PropertiesTab->_height_fld->setValue(fChoice*frame_item->PropertiesTab->_aspect_ratio);
			else frame_item->PropertiesTab->_aspect_ratio = frame_item->PropertiesTab->_height_fld->value()/fChoice;
			break;
		case HEIGHT:
			frame_item->PropertiesTab->_height_fld->setValue(fChoice);
			if(frame_item->PropertiesTab->_aspect_ratio_fld->isChecked()) frame_item->PropertiesTab->_width_fld->setValue(fChoice/frame_item->PropertiesTab->_aspect_ratio);
			else frame_item->PropertiesTab->_aspect_ratio = fChoice/frame_item->PropertiesTab->_width_fld->value();
			break;
		case ASPECT_RATIO:
			frame_item->PropertiesTab->_aspect_ratio_fld->setChecked(bChoice);
			break;
		case FRAME_TYPE:
			frame_item->PropertiesTab->_frame_type_fld->setCurrentItem(iChoice);
			break;
		//  Title Tab
		case AUTOTITLE:
			frame_item->TitleTab->_autotitle_fld->setChecked(bChoice);
			break;
		case TITLE:
			frame_item->TitleTab->_title_fld->setText(sChoice);
			frame_item->TitleTab->_autotitle_fld->setChecked(false);
			break;
		case TITLE_CHAR_SIZE:
			frame_item->TitleTab->_title_char_size_fld->setValue(fChoice);
			break;
		case TITLE_DISP:
			frame_item->TitleTab->_title_disp_fld->setValue(fChoice);
			break;
		case SUBTITLE:
			frame_item->TitleTab->_subtitle_fld->setText(sChoice);
			break;
		case SUBTITLE_CHAR_SIZE:
			frame_item->TitleTab->_subtitle_char_size_fld->setValue(fChoice);
			break;
		case SUBTITLE_DISP:
			frame_item->TitleTab->_subtitle_disp_fld->setValue(fChoice);
			break;
		case TITLE_JUST:
			frame_item->TitleTab->_title_just_fld->setCurrentItem(iChoice);
			break;
		case TITLE_FONT_FAMILY:
			switch(iChoice)
			{
				case PL_FCI_SERIF:
					frame_item->TitleTab->_title_font_fld->setSerifFont();
					break;
				case PL_FCI_MONO:
					frame_item->TitleTab->_title_font_fld->setMonotypeFont();
					break;
				case PL_FCI_SCRIPT:
					frame_item->TitleTab->_title_font_fld->setDecorativeFont();
					break;
				default:
					frame_item->TitleTab->_title_font_fld->setSansFont();
					break;
			}
			break;
		case TITLE_FONT_STYLE:
			frame_item->TitleTab->_title_font_fld->setItalic(bChoice);
			break;
		case TITLE_FONT_WEIGHT:
			frame_item->TitleTab->_title_font_fld->setBold(bChoice);
			break;
		case TITLE_COLOUR:
		{
			QPalette line_palette = QPalette(QPUI_normal_palette);
			line_palette.setColor(QPalette::Active, QColorGroup::ButtonText, cChoice);
			line_palette.setColor(QPalette::Inactive, QColorGroup::ButtonText, cChoice);
			frame_item->TitleTab->_title_colour_fld->setPalette(line_palette);
			frame_item->_title_colour = cChoice;
			break;
		}
		  case TICKS_CHAR_SIZE:
			  frame_item->DisplayTab->_ticks_char_size_fld->setValue(fChoice);
			  break;
		default:
			  break;
			  
	  }
   } // end for over local list
}

// ----------------------------------

void QpListView::selectAll()
{
  // overloaded from QguListView Emits selectionModified signal
  // to ensure currents for selected item are shown

  QguListView::selectAll();
  emit selectionModified();
}

// ----------------------------------

void QpListView::setPageOption(int option, int iChoice, float fChoice, QString sChoice, bool bChoice, QColor cChoice)
{
	if(QuSlotDebug()) cout << "QpListView::setPageOption" << endl;

   // apply Page option to all selected pages (that belonging to visible tab is always selected)
   for (int i=0; i<localList.size(); i++)
   {
      // only modify page items
      QpPageItem * page_item = dynamic_cast<QpPageItem*>(localList[i]);
    
      if( !page_item) continue;
	  
	  // don't set Page of visible tab - user just did this - avoid infinite loop
	  if(page_item->PageTab->isVisible()) continue;
	  if(page_item->LayoutTab->isVisible()) continue;
	  
	  switch(option){
	  
		// Layout Tab
		case AUTOLAYOUT:
			page_item->LayoutTab->_autolayout_fld->setChecked(bChoice);
			page_item->setFramesAuto(bChoice);
			break;
		case NUM_ROWS:
			page_item->LayoutTab->_row_num_fld->setValue(iChoice);
			break;
		case NUM_COLS:
			page_item->LayoutTab->_col_num_fld->setValue(iChoice);
			break;
		case HGAP:
			page_item->LayoutTab->_hgap_fld->setValue(fChoice);
			break;
		case VGAP:
			page_item->LayoutTab->_vgap_fld->setValue(fChoice);
			break;
		case LMARGIN:
			page_item->LayoutTab->_lmargin_fld->setValue(fChoice);
			break;
		case RMARGIN:
			page_item->LayoutTab->_rmargin_fld->setValue(fChoice);
			break;
		case TMARGIN:
			page_item->LayoutTab->_tmargin_fld->setValue(fChoice);
			break;
		case BMARGIN:
			page_item->LayoutTab->_bmargin_fld->setValue(fChoice);
			break;
			
		// Page Tab
		case PAGE_SIZE:
			page_item->PageTab->_page_size_fld->setCurrentItem(iChoice);
			break;
		case PAGE_UNITS:
			page_item->PageTab->_units_fld->setCurrentItem(iChoice);
			break;
		case PAGE_DPI:
			page_item->PageTab->_dpi_fld->setCurrentItem(iChoice);
			break;
		case PAGE_WIDTH:
			page_item->PageTab->_width_fld->setText(sChoice);
			break;
		case PAGE_HEIGHT:
			page_item->PageTab->_height_fld->setText(sChoice);
			break;
		case PAGE_ORIENTATION:
			page_item->PageTab->_orientation_fld->setCurrentItem(iChoice);
			break;
		case XWIN_NUM:
			page_item->PageTab->_xwin_num_fld->setText(tr("%1").arg(iChoice));
			break;
		case IDENT:
			page_item->PageTab->_show_ident_fld->setChecked(bChoice);
			break;
		default:
			  break;
			  
	  }
		page_item->setLayout();

   } // end for over local list
}

// -------------------------------------

QpListItem* QpListView::lastChild() const
{
	// Function to return pointer to lastChild in listview

	Q_LIST_VIEW_ITEM *item=firstChild();
	if(!item)  return 0;
	while(item->nextSibling())  item=item->nextSibling();
	return dynamic_cast<QpListItem*>(item);
}

// --------------------------------------------

void QpListView::setPlotType(int plotType)
{
	// slot invoked when plot type is changed on data tab
	if(QuSlotDebug()) cout << "QpListView::setPlotType" << endl;
	
	// ensure this item is done if there is nothing selected
	QpPlotItem * thisItem = dynamic_cast <QpPlotItem *> (currentItem());

	for(int i=0; i<localList.size(); i++)
	{
		// change all selected traces 
		QpPlotItem * item = dynamic_cast <QpPlotItem *>(localList[i]);
		if(item && item != thisItem) item->I_Panel->changePlotType(item, (QpItemType) plotType);
	}
	// do current item last to ensure it gets selected 
	if(thisItem) thisItem->I_Panel->changePlotType(thisItem, (QpItemType) plotType);

	localList.clear();
	updateOnSelection();
	
}  
	
// --------------------------------------------

void QpListView::setFrameType(int frameType)
{
	// slot invoked when plot type is changed on data tab
	if(QuSlotDebug()) cout << "QpListView::setFrameType" << endl;
	
	// ensure this item is done if there is nothing selected
	QpFrameItem * thisItem = dynamic_cast <QpFrameItem *> (currentItem());
	QpPlotItem * thisPlotItem = dynamic_cast <QpPlotItem *> (currentItem());
	
	for(int i=0; i<localList.size(); i++)
	{
		// change all selected traces 
		QpFrameItem * item = dynamic_cast <QpFrameItem *>(localList[i]);
		if(item && item != thisItem) item->changeFrame(frameType);
		
		QpPlotItem * pItem = dynamic_cast <QpPlotItem *>(localList[i]);
		if(pItem && pItem != thisPlotItem) pItem->I_Panel->I_Frame->changeFrame(frameType);		

	}
	
	// do current item last to ensure it gets selected 
	if(thisItem) thisItem->changeFrame(frameType);
	if(thisPlotItem) thisPlotItem->I_Panel->I_Frame->changeFrame(frameType);
	
	updateOnSelection();
	
}
	

// -------------------------

QString QpListView::nextFrameName()
{
  int count=0;
  QpPageItem * _page = dynamic_cast<QpPageItem*>(this->firstChild());
  while(_page)
  {
	QpFrameItem * _frame = dynamic_cast<QpFrameItem*>(_page->firstChild());
	while(_frame)
	{
		count++;
	}
	_page = dynamic_cast<QpPageItem*>(_page->nextSibling());
  }
    QString name = QString("Frame%1").arg(count);
	return name;
}

// -------------------------

QString QpListView::nextPanelName()
{
  int count=0;
  QpPageItem * _page = dynamic_cast<QpPageItem*>(this->firstChild());
  while(_page)
  {
	QpFrameItem * _frame = dynamic_cast<QpFrameItem*>(_page->firstChild());
	while(_frame)
	{
		QpPanelItem * _panel = dynamic_cast<QpPanelItem*>(_frame->firstChild());
		while(_panel)
		{
			count++;
		}
		_frame = dynamic_cast<QpFrameItem*>(_frame->nextSibling());
	}
	_page = dynamic_cast<QpPageItem*>(_page->nextSibling());
  }
    QString name = QString("Panel%1").arg(count);
	return name;
}


// -------------------------

void QpListView::contentsMouseReleaseEvent(QMouseEvent *event)
{
	// slot invoked when the mouse is pressed and released

	// use this to control tabs visible
	
	QguListView::contentsMouseReleaseEvent(event);
	emit selectionModified();
}

// -------------------------




/// \fn void QpuiListView::contentsMouseDoubleClickEvent(QMouseEvent *event)
/// \brief Method that processes the mouse double-click event. It has been re-implemented from Q_LIST_VIEW
/// \param event of type QMouseEvent * 
/// \exception None
/// \return None
/// 
 void QpListView::contentsMouseDoubleClickEvent(QMouseEvent *event)
{
	// disable mouse down on right and middle button
	// forces launch to be from left button and protects
	// from two mouse down events causing drag problems
  
	if( event->button() == Qt::RightButton) return;   
	if( event->button() == Qt::MidButton) return;
   
	if(event->state() == Qt::ControlButton || event->state() == Qt::ShiftButton) return;

	// clean up localList as double click only applies to single item
	clearSelectedList();
   
	QguListItem *atItem = (QguListItem *) currentItem();  
	if(atItem  == NULL) return;

	if(atItem->getEntryType() == ListTypeGroup) {
		// open/close group view
		Q_LIST_VIEW::contentsMouseDoubleClickEvent(event);
	}  
	else{
		// view linked data object on WL
		// find item on WL from WLcontext and set it to be current item
		QguListItem * wli = qsasWindow->WorkingList->getItemByName(atItem->WLcontext);
		qsasWindow->WorkingList->setCurrentItem(wli);
		// tell WL to browse current item
		
		viewWL_ObjRequest();
	}
}


void QpListView::deleteItem(QpListItem * item)
{
	// remove plot item from selected list
	RemoveFromSelectedList(item);
	
	
	// safe to delete item now
	delete item;

	
}

void QpListView::deleteOneItem()
{
	if(QuSlotDebug())
	{
		printf("QpListView::deleteOneItem %ld\n", this);
		fflush(stdout);
	}

	// slot invoked from Delete menu item on Right Mouse Button Menu
	bool fixLayout = false;
	
	QpListItem *item = dynamic_cast<QpListItem*>(selectedItem());
	if(!item) return;
	
	QpPageItem *pageItem = item->pageItem();
	QpFrameItem *frameItem = dynamic_cast<QpFrameItem*>(item);
	if(frameItem) fixLayout = true;
	
	QpPanelItem *parentPanel = dynamic_cast<QpPanelItem*>(item->parent());
	QpFrameItem *parentFrame = dynamic_cast<QpFrameItem*>(item->parent());
	
	deleteItem(item);
	
	if(parentPanel) parentPanel->I_Frame->updateAxes();
	else if(parentFrame) parentFrame->updateAxes();
	
	if(fixLayout)	
	{
		emit pageLayoutChanged(pageItem);
	}
	selectSomething();
}


void QpListView::deleteSelectedItems()
{
	if(QuSlotDebug())
	{
		printf("QpListView::deleteSelectedItems %ld\n", this);
		fflush(stdout);
	}

	// slot invoked from Delete edit menu item
	// deletes selected items from plot listview

	QpListItem *item=0;
	QpListItem *parent_item=0;
#if QT_VERSION < 0x040000
	QList<QpListItem> delete_lst;
	QList<QpPageItem> page_lst;
	QList<QpFrameItem> frame_lst;
#else
        QList<QpListItem*> delete_lst;
        QList<QpPageItem*> page_lst;
        QList<QpFrameItem*> frame_lst;
#endif
	
	// create list of root items of selected trees
	for(int i=0; i<localList.size(); i++)
	{
		item = dynamic_cast<QpListItem*>(localList[i]);
		if(!item) continue;
		
		// can ignore if parent item is also on selected list
		parent_item = dynamic_cast<QpListItem*>( item->parent() );
		if(isOnSelectedList(parent_item)) continue;
	   
		QpPanelItem *parentPanel;
		QpFrameItem *parentFrame;
		QpPageItem *parentPage;
		
		parentPage = item->pageItem();
		if(0 == page_lst.contains(parentPage)) page_lst.append(parentPage); 

		parentPanel = dynamic_cast<QpPanelItem*>(parent_item);
		if(parentPanel) parentFrame = parentPanel->I_Frame;
		else parentFrame = dynamic_cast<QpFrameItem*>(parent_item);
		
		if(0 == frame_lst.contains(parentFrame)) frame_lst.append(parentFrame);
	   
		// add to list of items to be deleted if not already on list
		if(0 == delete_lst.contains(item)) delete_lst.append(item);
	}

  
	// safe to delete items and their children
	for(uint i=0; i<delete_lst.count(); i++)
	{
		QpListItem *p_item = delete_lst.at(i);
		if(!p_item) continue;
		deleteItem(p_item); // destructors automatically delete child items recursively
	}
	
	// update axes
	for(uint i=0; i<frame_lst.count(); i++)
	{
		QpFrameItem *f_item = frame_lst.at(i);
		if(!f_item) continue;

		f_item->updateAxes(); // update axes for affected
	}
	
	// update page layout
	// update page layout
	for(uint i=0; i<page_lst.count(); i++)
	{
		QpPageItem *p_item = page_lst.at(i);
		if(!p_item) continue;
		setPageLayout(p_item);
	}
			
	clearSelectedList();
	selectSomething();
}

void QpListView::selectSomething()
{
	if(QuSlotDebug())
	{
		printf("QpListView::selectSomething %ld\n", this);
		fflush(stdout);
	}

	// select last thing in listview (ensures sensible tab shown)
	QpPageItem *pageItem = dynamic_cast<QpPageItem*>(this->lastChild());
	if(pageItem)
	{
		QpFrameItem * frameItem = dynamic_cast<QpFrameItem*>(pageItem->lastChild());
		if(frameItem)
		{
			QpPanelItem * panelItem = dynamic_cast<QpPanelItem*>(frameItem->lastChild());
			if(panelItem)
			{
				QpPlotItem * plotItem = dynamic_cast<QpPlotItem*>(panelItem->lastChild());
				if(plotItem) setCurrentItem((QguListItem*)plotItem);
				else setCurrentItem((QguListItem*)panelItem);
			}
			else setCurrentItem((QguListItem*)frameItem);
		}
		else setCurrentItem((QguListItem*)pageItem);
	}
	
	I_LastTab = QP_IGNORE_TAB;
	updateOnSelection();

}

void QpListView::dragEnterEvent(QDragEnterEvent *event)
{
	if(QuSlotDebug()) printf("QpListView::dragEnterEvent %ld\n", this);
#if QT_VERSION >= 0x040000
	if (event->mimeData()->hasText()) event->acceptProposedAction();
#endif
	setCurrentItem(this->lastChild());
	emit raiseWindow();

}

void QpListView::raisePlotWindow()
{
	raiseWindow();
}

void QpListView::dropEvent(QDropEvent *event)
{
	if(QuSlotDebug())
	{
		printf("QpListView::dropEvent %ld\n", this);
		fflush(stdout);
	}
	
	// drop event slot - invoked when item dropped onto plot listview
	// overloaded from QguListView::dropEvent to enable drop to be made onto multiple items

	QString txt;
	QguListItem *dropAtItem = dynamic_cast<QguListItem*>(this->currentItem());
	if (dragList.size() == 0) return;

#if QT_VERSION < 0x040000
	if ( QTextDrag::decode(event, txt) )
#else
	if (event->mimeData()->hasText())
#endif
	{
#if QT_VERSION < 0x040000
		//  copy drag text to clipboard to allow repeat paste
		QApplication::clipboard()->setData( new QTextDrag(txt) );
#else
		txt=event->mimeData()->text();
		QMimeData * clipData=new QMimeData;
		clipData->setText(txt);
		QApplication::clipboard()->setMimeData(clipData);
#endif

				
		// prevent drop onto item in dragList or item with ancestor on list
		if(dragList[0]->listView() == this)
		{
			// Drop may not be on selected list, single target only
			if ( isOnSelectedList(dropAtItem) || findAncestorOnList(dropAtItem)) return;
			
			// special case of single click on item
			if (dropAtItem && txt == dropAtItem->getWLcontext() ) return;

		}

		try{
			insertListItems(dropAtItem); // inserts the drag list
		}
		catch(Exception& e)
		{ 
			cout << e.msg() << endl; 
			return;
		}

	}
}

// -----------------------------------------------------------------

QpListItem * QpListView::getDropTarget(QpListItem *dropAt, const string type)
{
	QpPageItem *dropPage = NULL;
	QpFrameItem *dropFrame = NULL;
	QpPanelItem *dropPanel = NULL;
	QpPlotItem *dropPlot = NULL;

	if(dropAt == NULL) 
	{
		dropPage = dynamic_cast<QpPageItem*>(this->lastChild());
		if(dropPage) dropFrame = dynamic_cast<QpFrameItem*>(dropPage->lastChild());
		if(dropFrame) dropPanel = dynamic_cast<QpPanelItem*>(dropFrame->lastChild());
		if (dropPanel) dropPlot = dynamic_cast<QpPlotItem*>(dropPanel->lastChild());
	}
	else
	{
		dropPage = dynamic_cast<QpPageItem*>(dropAt);
		dropFrame = dynamic_cast<QpFrameItem*>(dropAt);
		dropPanel = dynamic_cast<QpPanelItem*>(dropAt);
		dropPlot = dynamic_cast<QpPlotItem*>(dropAt);
	}

	if(type == "PAGE")
	{
		if(dropPage != NULL) return dropPage;
		else if(dropFrame != NULL) return dropFrame->I_Page;
		else if(dropPanel != NULL) return dropPanel->I_Frame->I_Page;
		else if(dropPlot != NULL) return dropPlot->I_Panel->I_Frame->I_Page;
		
	}
	if(type == "FRAME")
	{
		if(dropFrame != NULL) return dropFrame;
		else if(dropPanel != NULL) return dropPanel->I_Frame;
		else if(dropPlot != NULL) return dropPlot->I_Panel->I_Frame;
	}
	if(type == "PANEL")
	{
		if(dropPanel != NULL) return dropPanel;
		else if(dropPlot != NULL) return dropPlot->I_Panel;
	}
	if(type == "PLOT")
	{
		if(dropPlot != NULL) return dropPlot;
	}
	
	return NULL;
}

// -----------------------------------------

void QpListView::insertDragItem(QguListItem *dropAtItem, QguListItem *item)
{
	// called from QguListView::insertListItems, invoked from a dropEvent
	// drags item from within plot listview

	if(QuSlotDebug())
	{
		printf("QpListView::insertDragItem %ld\n", this);
		fflush(stdout);
	}		

	if(item == NULL) return;

	if(item->listView() == this)
	{
		// drag from this list (so there is always a page)
		// drag of item(s) onto single item (ensured in dropEvent())
		
		QpListItem *dropAt = dynamic_cast<QpListItem*>(dropAtItem);
		if(!dropAt) return;

		QpListItem *list_item = dynamic_cast<QpListItem*>(item);
		if(list_item == NULL) return;
		
		QpPanelItem *parentPanel = dynamic_cast<QpPanelItem*>(list_item->parent());
		QpFrameItem *parentFrame = dynamic_cast<QpFrameItem*>(list_item->parent());
		if(parentPanel) parentFrame = parentPanel->I_Frame;
		
		// if dropAt is descendent of item, do nothing
		if(list_item->isDescendent(dropAt)) return;

		QpPageItem *pageItem = dynamic_cast<QpPageItem*>(list_item);
		QpFrameItem *frameItem = dynamic_cast<QpFrameItem*>(list_item);
		QpPanelItem *panelItem = dynamic_cast<QpPanelItem*>(list_item);
		QpPlotItem *plotItem = dynamic_cast<QpPlotItem*>(list_item);
    
     
		if(pageItem)
		{
			// page item being dragged, drop after page item of selected item
			QpPageItem *after = dynamic_cast<QpPageItem*>(getDropTarget(dropAt, "PAGE"));
			if(after == NULL) return;
			
			pageItem->moveItem(after);
			
		}
		else if(frameItem)
		{
			// Frame item being dragged
			pageItem = frameItem->I_Page;
			QpFrameItem *after = dynamic_cast<QpFrameItem*>(getDropTarget(dropAt, "FRAME"));
			if(after == NULL) 
			{
				QpPageItem *into = dynamic_cast<QpPageItem*>(getDropTarget(dropAt, "PAGE"));
				if(into)
				{
					pageItem->takeItem(frameItem);
					into->insertItem(frameItem);
					frameItem->I_Page = into;
					emit pageLayoutChanged(into);
				}
			}
			else
			{
				pageItem->takeItem(frameItem);
				after->I_Page->insertItem(frameItem);
				frameItem->I_Page = after->I_Page;
				frameItem->moveItem(after);
			}
		}
		else if(panelItem)
		{
			// Panel item being dragged

			frameItem = panelItem->I_Frame;
			pageItem = frameItem->I_Page;
			QpPanelItem *after = dynamic_cast<QpPanelItem*>(getDropTarget(dropAt, "PANEL"));
			if(after == NULL) 
			{
				QpFrameItem *into = dynamic_cast<QpFrameItem*>(getDropTarget(dropAt, "FRAME"));
				if(into)
				{
					frameItem->takeItem(panelItem);
					into->insertItem(panelItem);
					panelItem->I_Frame = into;
					panelItem->I_Frame->updateAxes();
					if(panelItem->I_Frame != parentFrame) parentFrame->updateAxes();
				}
				else
				{
					QpPageItem *intoPage = dynamic_cast<QpPageItem*>(getDropTarget(dropAt, "PAGE"));
					if(intoPage)
					{	
						QpFrameItem *newFrame = new QpFrameItem(intoPage, "Frame 1");
						frameItem->takeItem(panelItem);
						newFrame->insertItem(panelItem);
						panelItem->I_Frame = newFrame;
						newFrame->updateAxes();
						parentFrame->updateAxes();
					}
				}
			}
			else
			{
				frameItem->takeItem(panelItem);
				after->I_Frame->insertItem(panelItem);
				panelItem->I_Frame = after->I_Frame;
				panelItem->moveItem(after);
				panelItem->I_Frame->updateAxes();
				if(panelItem->I_Frame != parentFrame) parentFrame->updateAxes();
			}
		}
		else if(plotItem)
		{
			// plot item being dragged (trace/spectro/event)

			panelItem = plotItem->I_Panel;
			frameItem = panelItem->I_Frame;
			pageItem = frameItem->I_Page;

			QpPlotItem *after = dynamic_cast<QpPlotItem*>(getDropTarget(dropAt, "PLOT"));
			if(after == NULL) 
			{
				QpPanelItem *into = dynamic_cast<QpPanelItem*>(getDropTarget(dropAt, "PANEL"));
				if(into)
				{
					panelItem->takeItem(plotItem);
					into->insertItem(plotItem);
					plotItem->I_Panel = into;				
					into->I_Frame->updateAxes();
					if(into->I_Frame != parentFrame) parentFrame->updateAxes();
				}
				else
				{
					QpFrameItem *intoFrame = dynamic_cast<QpFrameItem*>(getDropTarget(dropAt, "FRAME"));
					if(intoFrame)
					{	
						QpPanelItem *newPanel = new QpPanelItem(intoFrame, "Panel 1");
						panelItem->takeItem(plotItem);
						newPanel->insertItem(plotItem);
						plotItem->I_Panel = newPanel;				
						intoFrame->updateAxes();
						if(intoFrame != parentFrame) parentFrame->updateAxes();
					}
					else
					{
						QpPageItem *intoPage = dynamic_cast<QpPageItem*>(getDropTarget(dropAt, "PAGE"));
						if(intoPage)
						{	
							QpFrameItem *newFrame = new QpFrameItem(intoPage, "Frame 1");
							QpPanelItem *newPanel = new QpPanelItem(newFrame, "Panel 1");
							panelItem->takeItem(plotItem);
							newPanel->insertItem(plotItem);
							plotItem->I_Panel = newPanel;
										
							newFrame->updateAxes();
							parentFrame->updateAxes();
						}
					}
				}
			}
			else
			{
				panelItem->takeItem(plotItem);
				after->I_Panel->insertItem(plotItem);
				plotItem->I_Panel = after->I_Panel;
				plotItem->moveItem(after);
				
				after->I_Panel->I_Frame->updateAxes();
				if(after->I_Panel->I_Frame != parentFrame) parentFrame->updateAxes();
			}
		}
		 
		setCurrentItem(item);

		// rename dragged item if name already in use in new hierarchy
		string itemName(list_item->text(NAME_COL).latin1()); 
		while(list_item->nameAlreadyUsed(list_item->text(NAME_COL).latin1())) 
		{
			itemName += "A";
			list_item->setText(NAME_COL, itemName.c_str());
		}
		// more than one page may be affected if drag between pages
		emit pageLayoutChanged(pageItem); 
		
   }
   else
   {
     // dropping from another listview, so want to make a plot object
		setCurrentItem(dropAtItem); // used by mMultiPanel to place result
		insertEntries(dropAtItem, item);
		
		// look for page to update layout
		if(dropAtItem)
		{
			QpListItem* listItem = dynamic_cast<QpListItem*>(dropAtItem);
			if(listItem) pageLayoutChanged(listItem->pageItem()); 
		}
		else if(this->lastChild()) 
		{
			QpListItem* listItem = dynamic_cast<QpListItem*>(this->lastChild());
			if(listItem) pageLayoutChanged(listItem->pageItem()); 
		}
   }
}

// -----------------------------------------------

void QpListView::insertEntries(QguListItem *dropAt, QguListItem *item)
{
	// function invoked from insertDragItem following a dropEvent
	// creates plot item of appropriate type in plot list hierarchy
	// when dragging an item from another listview

	if(QuSlotDebug())
	{
		printf("QpListView::insertEntries %ld\n", this);
		fflush(stdout);
	}

	clearSelectedList();
	clearSelection();
	
    bool single_drag = (dragList.size() == 1);
   
    if(item->getEntryType() != ListTypeEntry) return; // only drop data objects
	
	// fetch name of dragged item
	QString obj_name = item->getWLcontext();

	if(dropAt == NULL){
		dropAt = this->lastChild(); // use last page
		if(dropAt == NULL)
		{
			// no page so create new page to drop into
			dropAt = makePage();
		}
	}

	QApplication::setOverrideCursor( Qt::waitCursor );
#if QT_VERSION < 0x040000
    qApp->processEvents(2000);
#else
    qApp->processEvents(QEventLoop::AllEvents, 2000);
#endif

	QpPlotItem *dropPlot = dynamic_cast<QpPlotItem*>(dropAt);

	// distinguish between replacing data and creating plot item
	if(dropPlot && single_drag)
	{
		// ensure data tab is visible before changing data to ensure propogation
		I_LastTab == QP_DATA_TAB;
		dropPlot->showTabs();
		
		// replace data in this plot item
		dropPlot->replaceData(obj_name);
		dropPlot->nameLock = false; // allow drop in data slot to rename this
	}
	else
	{
        	QpListItem *newItem = makePlotItem((QpListItem*)dropAt, obj_name);
	}

	QApplication::restoreOverrideCursor();
#if QT_VERSION < 0x040000
        qApp->processEvents(2000);
#else
        qApp->processEvents(QEventLoop::AllEvents, 2000);
#endif
}

void QpListView::paste()
{
	QguListView::paste();
	
	QguListItem *dropAtItem = (QguListItem *)selectedItem();
	
	QpListItem* listItem =dynamic_cast<QpListItem*>(dropAtItem);
	if(listItem) pageLayoutChanged(listItem->pageItem()); 
}

// -----------------------------------------

void QpListView::makeVector_R_X_Y_Z_Item(QpFrameType frameType)
{
	// create panels
	vector <QpTraceItem *> stack;
	vector <int> traces;
	traces.push_back(1);
	traces.push_back(1);
	traces.push_back(1);
	traces.push_back(1);
	makeMultiTraceItem(traces, stack, frameType);
	
	// set component text
	if(stack.size() == 4)
	{
		stack[0]->DataTab->_y_data_fld->setText(R_STR_CMP);
		stack[1]->DataTab->_y_data_fld->setText(X_STR_CMP);
		stack[2]->DataTab->_y_data_fld->setText(Y_STR_CMP);
		stack[3]->DataTab->_y_data_fld->setText(Z_STR_CMP);
	}
	
	// link data slots together
	linkTraceItems(stack, false);

}

// -----------------------------------------

void QpListView::makeVector_X_Y_Z_Item(QpFrameType frameType)
{
	// create panels
	vector <QpTraceItem *> stack;
	vector <int> traces;
	traces.push_back(1);
	traces.push_back(1);
	traces.push_back(1);
	makeMultiTraceItem(traces, stack, frameType);
	
	// set component text
	if(stack.size() == 3)
	{
		stack[0]->DataTab->_y_data_fld->setText(X_STR_CMP);
		stack[1]->DataTab->_y_data_fld->setText(Y_STR_CMP);		
		stack[2]->DataTab->_y_data_fld->setText(Z_STR_CMP);
	}
	
	// link data slots together
	linkTraceItems(stack, false);

}

// -----------------------------------------

void QpListView::makeVector_R_L_P_Item(QpFrameType frameType)
{
	// create panels
	vector <QpTraceItem *> stack;
	vector <int> traces;
	traces.push_back(1);
	traces.push_back(1);
	traces.push_back(1);
	makeMultiTraceItem(traces, stack, frameType);
	
	// set component text
	if(stack.size() == 3)
	{
		stack[0]->DataTab->_y_data_fld->setText(R_STR_CMP);
		stack[1]->DataTab->_y_data_fld->setText(Ld_STR_CMP);
		stack[2]->DataTab->_y_data_fld->setText(Pd_STR_CMP);
	}
	
	// link data slots together
	linkTraceItems(stack, false);

}

// -----------------------------------------

void QpListView::makeVector_R_T_P_Item(QpFrameType frameType)
{
	// create panels
	vector <QpTraceItem *> stack;
	vector <int> traces;
	traces.push_back(1);
	traces.push_back(1);
	traces.push_back(1);
	makeMultiTraceItem(traces, stack, frameType);
	
	// set component text
	if(stack.size() == 3)
	{
		stack[0]->DataTab->_y_data_fld->setText(R_STR_CMP);
		stack[1]->DataTab->_y_data_fld->setText(Td_STR_CMP);
		stack[2]->DataTab->_y_data_fld->setText(Pd_STR_CMP);
	}
	
	// link data slots together
	linkTraceItems(stack, false);

}

// -----------------------------------------

void QpListView::makeVector_RXYZ_Item(QpFrameType frameType)
{
	// create panels
	vector <QpTraceItem *> stack;	vector <int> traces;
	traces.push_back(4);
	makeMultiTraceItem(traces, stack, frameType);
	
	// set component text
	if(stack.size() == 4)
	{
		stack[0]->DataTab->_y_data_fld->setText(R_STR_CMP);
		stack[0]->TraceTab->setLineColour(stack[0]->R_colour);
		stack[0]->userLineColour = true;
		stack[0]->DataTab->_show_legend_fld->setChecked(true);
		stack[0]->I_Panel->LegendTab->_show_legend_fld->setChecked(true);
		
		stack[1]->DataTab->_y_data_fld->setText(X_STR_CMP);
		stack[1]->TraceTab->setLineColour(stack[1]->X_colour);
		stack[1]->userLineColour = true;
		stack[1]->DataTab->_show_legend_fld->setChecked(true);
		stack[2]->DataTab->_y_data_fld->setText(Y_STR_CMP);
		stack[2]->TraceTab->setLineColour(stack[2]->Y_colour);
		stack[2]->userLineColour = true;
		stack[2]->DataTab->_show_legend_fld->setChecked(true);
		stack[3]->DataTab->_y_data_fld->setText(Z_STR_CMP);
		stack[3]->TraceTab->setLineColour(stack[3]->Z_colour);
		stack[3]->userLineColour = true;
		stack[3]->DataTab->_show_legend_fld->setChecked(true);
	}
	
	// link data slots together
	linkTraceItems(stack, false);

}

// -----------------------------------------

void QpListView::makeVector_R_XYZ_Item(QpFrameType frameType)
{
	// create panels
	vector <QpTraceItem *> stack;	vector <int> traces;
	traces.push_back(1);
	traces.push_back(3);
	makeMultiTraceItem(traces, stack, frameType);
	
	// set component text
	if(stack.size() == 4)
	{
		stack[0]->DataTab->_y_data_fld->setText(R_STR_CMP);
		stack[0]->TraceTab->setLineColour(stack[0]->R_colour);
		stack[0]->userLineColour = true;
		stack[0]->DataTab->_show_legend_fld->setChecked(true);
		stack[0]->I_Panel->LegendTab->_show_legend_fld->setChecked(true);
		
		stack[1]->DataTab->_y_data_fld->setText(X_STR_CMP);
		stack[1]->TraceTab->setLineColour(stack[1]->X_colour);
		stack[1]->userLineColour = true;
		stack[1]->DataTab->_show_legend_fld->setChecked(true);
		stack[1]->I_Panel->LegendTab->_show_legend_fld->setChecked(true);
		
		stack[2]->DataTab->_y_data_fld->setText(Y_STR_CMP);
		stack[2]->TraceTab->setLineColour(stack[2]->Y_colour);
		stack[2]->userLineColour = true;
		stack[2]->DataTab->_show_legend_fld->setChecked(true);
		
		stack[3]->DataTab->_y_data_fld->setText(Z_STR_CMP);
		stack[3]->TraceTab->setLineColour(stack[3]->Z_colour);
		stack[3]->userLineColour = true;
		stack[3]->DataTab->_show_legend_fld->setChecked(true);
	}
	
	// link data slots together
	linkTraceItems(stack, false);

}

// -----------------------------------------

void QpListView::makeVector_XYZ_Item(QpFrameType frameType)
{
	// create panels
	vector <QpTraceItem *> stack;	vector <int> traces;
	traces.push_back(3);
	makeMultiTraceItem(traces, stack, frameType);
	
	// set component text
	if(stack.size() == 3)
	{
		stack[0]->DataTab->_y_data_fld->setText(X_STR_CMP);
		stack[0]->TraceTab->setLineColour(stack[0]->X_colour);
		stack[0]->userLineColour = true;
		stack[0]->DataTab->_show_legend_fld->setChecked(true);
		stack[0]->I_Panel->LegendTab->_show_legend_fld->setChecked(true);
		
		stack[1]->DataTab->_y_data_fld->setText(Y_STR_CMP);
		stack[1]->TraceTab->setLineColour(stack[1]->Y_colour);
		stack[1]->userLineColour = true;
		stack[1]->DataTab->_show_legend_fld->setChecked(true);
		
		stack[2]->DataTab->_y_data_fld->setText(Z_STR_CMP);
		stack[2]->TraceTab->setLineColour(stack[2]->Z_colour);
		stack[2]->userLineColour = true;
		stack[2]->DataTab->_show_legend_fld->setChecked(true);
	}
	
	// link data slots together
	linkTraceItems(stack, false);

}

// -----------------------------------------

void QpListView::makeVector_R_LP_Item(QpFrameType frameType)
{
	// create panels
	vector <QpTraceItem *> stack;	vector <int> traces;
	traces.push_back(1);
	traces.push_back(2);
	makeMultiTraceItem(traces, stack, frameType);
	
	// set component text
	if(stack.size() == 3)
	{
		stack[0]->DataTab->_y_data_fld->setText(R_STR_CMP);
		stack[0]->TraceTab->setLineColour(stack[0]->R_colour);
		stack[0]->userLineColour = true;
		stack[0]->DataTab->_show_legend_fld->setChecked(true);
		stack[0]->I_Panel->LegendTab->_show_legend_fld->setChecked(true);
		
		stack[1]->DataTab->_y_data_fld->setText(Ld_STR_CMP);
		stack[1]->TraceTab->setLineColour(stack[1]->L_colour);
		stack[1]->userLineColour = true;
		stack[1]->DataTab->_show_legend_fld->setChecked(true);
		stack[1]->I_Panel->LegendTab->_show_legend_fld->setChecked(true);
		
		stack[2]->DataTab->_y_data_fld->setText(Pd_STR_CMP);
		stack[2]->TraceTab->setLineColour(stack[2]->P_colour);
		stack[2]->userLineColour = true;
		stack[2]->DataTab->_show_legend_fld->setChecked(true);
	}
	
	// link data slots together
	linkTraceItems(stack, false);

}

// -----------------------------------------

void QpListView::makeVector_R_TP_Item(QpFrameType frameType)
{
	// create panels
	vector <QpTraceItem *> stack;
	vector <int> traces;
	traces.push_back(1);
	traces.push_back(2);
	makeMultiTraceItem(traces, stack, frameType);
	
	// set component text
	if(stack.size() == 3)
	{
		stack[0]->DataTab->_y_data_fld->setText(R_STR_CMP);
		stack[0]->TraceTab->setLineColour(stack[0]->R_colour);
		stack[0]->userLineColour = true;
		stack[0]->DataTab->_show_legend_fld->setChecked(true);
		stack[0]->I_Panel->LegendTab->_show_legend_fld->setChecked(true);

		stack[1]->DataTab->_y_data_fld->setText(Td_STR_CMP);
		stack[1]->TraceTab->setLineColour(stack[1]->T_colour);
		stack[1]->userLineColour = true;
		stack[1]->DataTab->_show_legend_fld->setChecked(true);
		stack[1]->I_Panel->LegendTab->_show_legend_fld->setChecked(true);
		
		stack[2]->DataTab->_y_data_fld->setText(Pd_STR_CMP);
		stack[2]->TraceTab->setLineColour(stack[2]->P_colour);
		stack[2]->userLineColour = true;
		stack[2]->DataTab->_show_legend_fld->setChecked(true);
	}
	
	// link data slots together
	linkTraceItems(stack, false);

}

// -----------------------------------------

void QpListView::makeC1_2_3_4_Item()
{
	// create panels
	vector <QpTraceItem *> stack;
	vector <int> traces;
	traces.push_back(1);
	traces.push_back(1);
	traces.push_back(1);
	traces.push_back(1);
	makeMultiTraceItem(traces, stack, QP_TS);
	
	// set component text
	if(stack.size() == 4)
	{
		stack[0]->DataTab->_y_data_fld->setText("?C2=C1? ?C3=C1? ?C4=C1? ?dr2=dr1? ?dr3=dr1? ?dr4=dr1?");
		stack[0]->TraceTab->setLineColour(stack[0]->C1_colour);
		stack[0]->userLineColour = true;
		stack[1]->DataTab->_y_data_fld->setText("?C1=C2? ?C3=C2? ?C4=C2? ?dr1=dr2? ?dr3=dr2? ?dr4=dr2?");
		stack[1]->TraceTab->setLineColour(stack[1]->C2_colour);
		stack[1]->userLineColour = true;
		stack[2]->DataTab->_y_data_fld->setText("?C1=C3? ?C2=C3? ?C4=C3? ?dr1=dr3? ?dr2=dr3? ?dr4=dr3?");
		stack[2]->TraceTab->setLineColour(stack[2]->C3_colour);
		stack[2]->userLineColour = true;
		stack[3]->DataTab->_y_data_fld->setText("?C1=C4? ?C2=C4? ?C3=C4? ?dr1=dr4? ?dr2=dr4? ?dr3=dr4?");
		stack[3]->TraceTab->setLineColour(stack[3]->C4_colour);
		stack[3]->userLineColour = true;
	}
	
	// link data slots together
	linkTraceItems(stack, true);

}
// -----------------------------------------

void QpListView::makeC1234_Item()
{
	// create panels
	vector <QpTraceItem *> stack;
	vector <int> traces;
	traces.push_back(4);
	makeMultiTraceItem(traces, stack, QP_TS);
	
	// set component text
	if(stack.size() == 4)
	{
		stack[0]->DataTab->_y_data_fld->setText("?C2=C1? ?C3=C1? ?C4=C1? ?dr2=dr1? ?dr3=dr1? ?dr4=dr1?");
		stack[0]->TraceTab->setLineColour(stack[0]->C1_colour);
		stack[0]->userLineColour = true;
		stack[1]->DataTab->_y_data_fld->setText("?C1=C2? ?C3=C2? ?C4=C2? ?dr1=dr2? ?dr3=dr2? ?dr4=dr2?");
		stack[1]->TraceTab->setLineColour(stack[1]->C2_colour);
		stack[1]->userLineColour = true;
		stack[2]->DataTab->_y_data_fld->setText("?C1=C3? ?C2=C3? ?C4=C3? ?dr1=dr3? ?dr2=dr3? ?dr4=dr3?");
		stack[2]->TraceTab->setLineColour(stack[2]->C3_colour);
		stack[2]->userLineColour = true;
		stack[3]->DataTab->_y_data_fld->setText("?C1=C4? ?C2=C4? ?C3=C4? ?dr1=dr4? ?dr2=dr4? ?dr3=dr4?");
		stack[3]->TraceTab->setLineColour(stack[3]->C4_colour);
		stack[3]->userLineColour = true;
	}
	
	// link data slots together
	linkTraceItems(stack, true);

}

// -----------------------------------------

void QpListView::make2Panel_Item()
{
	// create panels
	vector <QpTraceItem *> stack;
	vector <int> traces;
	traces.push_back(1);
	traces.push_back(1);
	makeMultiTraceItem(traces, stack, QP_TS);

}

// -----------------------------------------

void QpListView::make3Panel_Item()
{
	// create panels
	vector <QpTraceItem *> stack;
	vector <int> traces;
	traces.push_back(1);
	traces.push_back(1);
	traces.push_back(1);
	makeMultiTraceItem(traces, stack, QP_TS);

}

// -----------------------------------------

void QpListView::make4Panel_Item()
{
	// create panels
	vector <QpTraceItem *> stack;
	vector <int> traces;
	traces.push_back(1);
	traces.push_back(1);
	traces.push_back(1);
	traces.push_back(1);
	makeMultiTraceItem(traces, stack, QP_TS);

}

// -----------------------------------------

void QpListView::makeMultiTraceItem(vector <int> &nTraces, vector <QpTraceItem *> &stack, QpFrameType frameType)
{
	stack.clear();
	clearSelectedList();
	
	QpListItem* target = (QpListItem*) currentItem();
	QpPageItem* tPage = dynamic_cast <QpPageItem*>(target);
	QpFrameItem* tFrame = dynamic_cast <QpFrameItem*>(target);
	QpPanelItem* tPanel = dynamic_cast <QpPanelItem*>(target);
	QpPlotItem* tPlot = dynamic_cast <QpPlotItem*>(target);
	
	QpFrameItem *frameItem;
	if(tPage && frameType == QP_TS) frameItem = makeTSFrame(tPage);
	else if(tPage) frameItem = makeXYFrame(tPage);
	else if(tFrame) frameItem = tFrame;
	else if(tPanel) frameItem = tPanel->I_Frame;
	else 
	{
		makeTSFrameItem();
		frameItem = (QpFrameItem *) currentItem();
	}
	
	for(int i=0; i<nTraces.size(); i++)
	{
		makePanel(frameItem);
		QpPanelItem *panelItem = dynamic_cast <QpPanelItem*>(currentItem());
		if(panelItem) for(int j=0; j<nTraces[i]; j++)
		{
			makeTrace(panelItem, "");
			QpTraceItem * next = dynamic_cast <QpTraceItem *>(currentItem());
			if(next)
			{
				this->AddToSelectedList((QguListItem *) next);
				stack.push_back(next);
			}
		}
	}
}


// -----------------------------------------

QpListItem * QpListView::makePlotItem(QpListItem * target, QString obj_name)
{
	// Make plot item creating Frame and Panel if needed
		if ( QuSlotDebug() )  cout << "QpListView::makePlotItem - " << obj_name.latin1() << endl;
	if(target == NULL) return NULL;
	
	QpPanelItem *panelTarget = dynamic_cast<QpPanelItem*>(target);
	
	clearSelectedList();
	
	// fetch object
	QdObject_var obj = QuGetObjByName(obj_name);
	if(obj.is_nil())  return NULL;
		
	QpFrameType frameType = QP_TS; // default
	if(get_timetags(obj).is_nil()) frameType = QP_XY;

	// default plot type is trace
	QpItemType plot_type = QP_TRACE;
	// check if sequence
	if(!obj->is_sequence())
	{
		QdTimeInterval_var tint = QdTimeInterval_var::narrow(obj);
		QdRScalar_var rscalar_var = QdRScalar_var::narrow(obj);
		QdRMatrix_var rmatrix_var = QdRMatrix_var::narrow(obj);

		if(!tint.is_nil())
		{
			plot_type = QP_CONST_XT;
			frameType = QP_TS;
		}
		else if(!rscalar_var.is_nil()) plot_type = QP_CONST_Y;
		else if(!rmatrix_var.is_nil())
		{
			plot_type = QP_TRACE;
			frameType = QP_XY;
		}
		else
		{
          (void)QMessageBox::warning(this,"","Invalid object");
          return NULL;
		}
	}
	else
	{
		if(obj->sequence_size() == 0)
        {
			(void)QMessageBox::warning(this,"", "Invalid object - empty sequence");
			return NULL;
        }
		QdRScalarSeq_var rscalarseq = QdRScalarSeq_var::narrow(obj);
        QdRMatrixSeq_var rmatrixseq = QdRMatrixSeq_var::narrow(obj);
        
		// if timetags, default plot type is tstrace
		
		if(!rscalarseq.is_nil())
        {
			plot_type = QP_TRACE;
	
		}
		else if(QuIsThreeVector(obj) )
        {
			QString vectorPlot=QSAS_profiles->current()->Vector3D;
			
			if( panelTarget)
			{
				// use existing panel with component
				plot_type = QP_TRACE;
			}
			else if(vectorPlot=="RXYZ")
			{
				makeVector_RXYZ_Item(frameType);
				QpPlotItem *plotItem = dynamic_cast<QpPlotItem*>(currentItem());
				if(plotItem) plotItem->replaceData(obj_name);
				return NULL;
			}
			else if(vectorPlot=="R_XYZ")
			{
				makeVector_R_XYZ_Item(frameType);
				QpPlotItem *plotItem = dynamic_cast<QpPlotItem*>(currentItem());
				if(plotItem) plotItem->replaceData(obj_name);
				return NULL;
			}
			else if(vectorPlot=="XYZ")
			{
				makeVector_XYZ_Item(frameType);
				QpPlotItem *plotItem = dynamic_cast<QpPlotItem*>(currentItem());
				if(plotItem) plotItem->replaceData(obj_name);
				return NULL;
			}
			else if(vectorPlot=="R_TP")
			{
				makeVector_R_TP_Item(frameType);
				QpPlotItem *plotItem = dynamic_cast<QpPlotItem*>(currentItem());
				if(plotItem) plotItem->replaceData(obj_name);
				return NULL;
			}
			else if(vectorPlot=="R_LP")
			{
				makeVector_R_LP_Item(frameType);
				QpPlotItem *plotItem = dynamic_cast<QpPlotItem*>(currentItem());
				if(plotItem) plotItem->replaceData(obj_name);
				return NULL;
			}
			else if(vectorPlot=="R_X_Y_Z")
			{
				makeVector_R_X_Y_Z_Item(frameType);
				QpPlotItem *plotItem = dynamic_cast<QpPlotItem*>(currentItem());
				if(plotItem) plotItem->replaceData(obj_name);
				return NULL;
			}
			else if(vectorPlot=="X_Y_Z")
			{
				makeVector_X_Y_Z_Item(frameType);
				QpPlotItem *plotItem = dynamic_cast<QpPlotItem*>(currentItem());
				if(plotItem) plotItem->replaceData(obj_name);
				return NULL;
			}
			else if(vectorPlot=="R_T_P")
			{
				makeVector_R_T_P_Item(frameType);
				QpPlotItem *plotItem = dynamic_cast<QpPlotItem*>(currentItem());
				if(plotItem) plotItem->replaceData(obj_name);
				return NULL;
			}
			else if(vectorPlot=="R_L_P")
			{
				makeVector_R_L_P_Item(frameType);
				QpPlotItem *plotItem = dynamic_cast<QpPlotItem*>(currentItem());
				if(plotItem) plotItem->replaceData(obj_name);
				return NULL;
			}
			else //"R" if everything goes right
				plot_type = QP_TRACE;
				// continue and create scalar trace
		}
		else if(!rmatrixseq.is_nil())
        {
			if(rmatrixseq->is_homogeneous_seq())
			{
				plot_type = QP_SPECTROGRAM;
			}
			else
			{
				(void)QMessageBox::warning(this,"","Inhomogeneous object type");
				return NULL;
			}
		}
        else
        {
			(void)QMessageBox::warning(this,"","Invalid object type");
			return NULL;
        }
	}
	
	// create object of right type
	QpPlotItem *newPlot = makePlot(target, plot_type, frameType, obj_name);

	return newPlot;
}

// --------------------------------------------------

QpPlotItem* QpListView::makePlot(QpListItem * target, QpItemType plotType, QpFrameType frameType, QString WL)
{
	// creates a plot item of the correct type. 
	// frameType is a hint only if no frame exists
	if ( QuSlotDebug() ) 
	{
		cout << "QpListView::makePlot - '" << WL.latin1();
		if(target) cout << "' in target '" <<  target->text(NAME_COL).latin1() << "'" << endl;
		else cout << endl;
	}
	clearSelectedList();

	// Recursive until target is panel
	QpPageItem *pageItem = dynamic_cast<QpPageItem*>(target);
	QpFrameItem *frameItem = dynamic_cast<QpFrameItem*>(target);
	QpPanelItem *panelItem = dynamic_cast<QpPanelItem*>(target);
	QpPlotItem *plotItem = dynamic_cast<QpPlotItem*>(target);
	
	QpPlotItem* newItem=NULL;
	if( !target)
	{
		QpPageItem *newPage = makePage();
		QpPlotItem *newPlot = makePlot(newPage, plotType, frameType, WL);
		newItem = newPlot;
	}
	else if(pageItem)
	{
		QpFrameItem *newFrame;
		if(frameType == QP_TS) newFrame = makeTSFrame(pageItem);
		else newFrame = makeXYFrame(pageItem);
		
		QpPlotItem *newPlot = makePlot(newFrame, plotType, frameType, WL);
		newItem = newPlot;
	}
	else if(frameItem)
	{
		QpPanelItem *newPanel = makePanel(frameItem);
		QpPlotItem *newPlot = makePlot(newPanel, plotType, frameType, WL);
		newItem = newPlot;
	}
	else if(plotItem)
	{
		QpPlotItem *newPlot = makePlot(plotItem->I_Panel, plotType, frameType, WL);
		newItem = newPlot;
	}
	else if(panelItem)
	{
		// create new item in this panel
		if(plotType == QP_TRACE )
		{
			QpTraceItem *newPlot = makeTrace(panelItem, WL);
			newItem = newPlot;
		}
		else if(plotType == QP_CONST_Y)
		{
			QpTraceItem *newPlot = makeConstYTrace(panelItem, WL);
			newItem = newPlot;
		}
		else if(plotType == QP_CONST_XT)
		{
			QpTraceItem *newPlot = makeConstXTrace(panelItem, WL);
			newItem = newPlot;
		}
		else if(plotType == QP_SPECTROGRAM)
		{
			QpSpectrumItem *newPlot = makeSpectrum(panelItem, WL);
			newItem = newPlot;
		}
		else if(plotType == QP_EVENT)
		{
			QpEventItem *newPlot = makeEvent(panelItem);
			newItem = newPlot;
		}
	}
	else{
		cout << "Unknown target for new plot item" << endl;
	}
	return newItem;
}	  

// ----------------------------------------

void QpListView::makeXYFrameItem()
{
	if ( QuSlotDebug() )  cout << "QpListView::makeXYFrameItem " << endl;

	QpPageItem * pageItem = dynamic_cast<QpPageItem*>(this->currentItem());
	if(pageItem)
	{
		makeXYFrame(pageItem);
	}
	else
	{
		QpFrameItem * frameItem = dynamic_cast<QpFrameItem*>(this->currentItem());
		if(frameItem) makeXYFrame(frameItem->I_Page);
		else 
		{
			pageItem = dynamic_cast<QpPageItem*>(this->firstChild());
			if(pageItem)
			{
				QpPageItem *nextPage = dynamic_cast<QpPageItem*>(pageItem->nextSibling());
				while(nextPage)
				{
					// find last page
					pageItem = nextPage;
					nextPage =  dynamic_cast<QpPageItem*>(pageItem->nextSibling());
				}	
				makeXYFrame(pageItem);

			}	
			else{
				pageItem = makePage();
				makeXYFrame(pageItem);
			}
		}
	}
	
}


// ----------------------------------------

QpFrameItem* QpListView::makeXYFrame(QpPageItem* pageItem)
{
	if ( QuSlotDebug() )  cout << "QpListView::makeXYFrame " << endl;

	clearSelectedList();

	// creates new frame item in page
	if(pageItem == NULL) return NULL;
  
	int frame_cnt = pageItem->childCount();
	string frame_name("Frame "+int_to_string(++frame_cnt));
	QpFrameItem* frameItem = new QpFrameItem(pageItem, QString(frame_name.c_str()), dynamic_cast<QpFrameItem*>(pageItem->lastChild()), QP_XY);
	setCurrentItem(frameItem);
	updateOnSelection();

	emit pageLayoutChanged(pageItem);

	return frameItem;

}	 
	 
// ----------------------------------------

void QpListView::makeTSFrameItem()
{
	if ( QuSlotDebug() )  cout << "QpListView::makeTSFrameItem " << endl;

	QpPageItem * pageItem = dynamic_cast<QpPageItem*>(this->currentItem());
	if(pageItem)
	{
		makeTSFrame(pageItem);
	}
	else
	{
		QpFrameItem * frameItem = dynamic_cast<QpFrameItem*>(this->currentItem());
		if(frameItem) makeTSFrame(frameItem->I_Page);
		else 
		{
			pageItem = dynamic_cast<QpPageItem*>(this->firstChild());
			if(pageItem)
			{
				QpPageItem *nextPage = dynamic_cast<QpPageItem*>(pageItem->nextSibling());
				while(nextPage)
				{
					// find last page
					pageItem = nextPage;
					nextPage =  dynamic_cast<QpPageItem*>(pageItem->nextSibling());
				}	
				makeTSFrame(pageItem);

			}	
			else{
				pageItem = makePage();
				makeTSFrame(pageItem);
			}
		}
	}

}

// ----------------------------------------

QpFrameItem* QpListView::makeTSFrame(QpPageItem* pageItem)
{
	if ( QuSlotDebug() )  cout << "QpListView::makeTSFrame " << endl;

	clearSelectedList();

	// creates new frame item in page
	if(pageItem == NULL) return NULL;
  
	int frame_cnt = pageItem->childCount();
	string frame_name("Frame "+int_to_string(++frame_cnt));
	QpFrameItem* frameItem = new QpFrameItem(pageItem, QString(frame_name.c_str()), dynamic_cast<QpFrameItem*>(pageItem->lastChild()), QP_TS);
	setCurrentItem(frameItem);
	updateOnSelection();
	
	emit pageLayoutChanged(pageItem);

	return frameItem;

}	 
	 
// ----------------------------------------

void QpListView::makePageItem()
{
	this->makePage();	

}


// ----------------------------------------

QpPageItem* QpListView::makePage()
{	
	clearSelectedList();

	// creates new page item in listview
	int page_cnt = childCount();
	string page_name("Page "+int_to_string(++page_cnt));
	QpPageItem* pageItem = new QpPageItem(this, QString(page_name.c_str()), dynamic_cast<QpPageItem*>(this->lastChild()));
	setCurrentItem(pageItem);
	updateOnSelection();

	return pageItem;
}

// ----------------------------------------

void QpListView::makePanelItem()
{
	
	QpFrameItem * frameItem = dynamic_cast<QpFrameItem*>(this->currentItem());
	if(frameItem)
	{
		makePanel(frameItem);
	}
	else
	{
		QpPanelItem * panelItem = dynamic_cast<QpPanelItem*>(this->currentItem());
		if(panelItem) makePanel(panelItem->I_Frame);
		else 
		{
			QpPageItem *pageItem = dynamic_cast<QpPageItem*>(this->firstChild());
			if(pageItem)
			{
				QpPageItem *nextPage = dynamic_cast<QpPageItem*>(pageItem->nextSibling());
				while(nextPage)
				{
					// find last page
					pageItem = nextPage;
					nextPage =  dynamic_cast<QpPageItem*>(pageItem->nextSibling());
				}	
				frameItem = dynamic_cast<QpFrameItem*>(pageItem->firstChild());
				if(frameItem)
				{
					QpFrameItem *nextFrame = dynamic_cast<QpFrameItem*>(pageItem->firstChild());
					while(nextFrame)
					{
						frameItem = nextFrame;
						nextFrame =  dynamic_cast<QpFrameItem*>(frameItem->nextSibling());
					}
					makePanel(frameItem);
				}
				else
				{
					frameItem = makeTSFrame(pageItem);
					makePanel(frameItem);
				}
			}	
			else{
				pageItem = makePage();
				frameItem = makeTSFrame(pageItem);
				makePanel(frameItem);
			}
		}
	}

}

// ----------------------------------------

QpPanelItem* QpListView::makePanel(QpFrameItem* frameItem)
{
	clearSelectedList();

	// creates new panel item in frame
	if(frameItem == NULL) return NULL;
  
	int panel_cnt = frameItem->childCount();
	string panel_name("Panel "+int_to_string(++panel_cnt));
	QpPanelItem* panelItem = new QpPanelItem(frameItem, QString(panel_name.c_str()), dynamic_cast<QpPanelItem*>(frameItem->lastChild()));
	setCurrentItem(panelItem);
	updateOnSelection();

	return panelItem;

}

// ----------------------------------------

void QpListView::makeEmptyItem(QpItemType type)
{
	clearSelectedList();
	
	QpFrameType frameType;
	QpListItem * target = dynamic_cast<QpListItem*>(this->currentItem());
	if(target)
	{
		frameType = target->frameType();
		makePlot(target, type, frameType, "");
	}
	else
	{
		target = dynamic_cast<QpListItem*>(this->lastItem());
		if(target)
		{
			frameType = target->frameType();
		}
		makePlot(target, type, frameType, ""); // if target is NULL new page is created
	}
}

// ----------------------------------------

void QpListView::makeTraceItem()
{
	makeEmptyItem(QP_TRACE);	
}

// ----------------------------------------

QpTraceItem* QpListView::makeTrace(QpPanelItem* panelItem, QString WL)
{
	// creates new trace item in panel
	if(panelItem == NULL) return NULL;

	clearSelectedList();

	QString plot_name;
	if(WL.isEmpty())
	{
		int plot_cnt = panelItem->childCount();
		plot_name = "Plot ";
		plot_name += int_to_string(++plot_cnt).c_str();
		plot_name += ": Trace";
	}
	else 
	{
		if(panelItem->frameType() == QP_TS)
		{
			plot_name = QuGetDefaultComp(WL, QSAS_profiles->current()->Array3DtoScalar);
		}
		else
		{
			plot_name = QuGetDefaultComp(WL, QSAS_profiles->current()->Array3DtoScalarXY);
		}
	}

	QpTraceItem* traceItem = new QpTraceItem(panelItem, plot_name, dynamic_cast<QpPlotItem*>(panelItem->lastChild()), WL, QP_TRACE);
	setCurrentItem(traceItem);
	updateOnSelection();

	panelItem->LegendTab->updateNumTraces();
	
	return traceItem;

}

// ----------------------------------------

void QpListView::makeConstXTraceItem()
{
	clearSelectedList();

	makeEmptyItem(QP_CONST_XT);

}

// ----------------------------------------

QpTraceItem* QpListView::makeConstXTrace(QpPanelItem* panelItem, QString WL)
{
	// creates new trace item in panel
	if(panelItem == NULL) return NULL;
	
	clearSelectedList();

	QString plot_name;
	if(WL.isEmpty())
	{
		int plot_cnt = panelItem->childCount();
		
		if(panelItem->frameType() == QP_TS)
		{
			plot_name = "Plot ";
			plot_name += int_to_string(++plot_cnt).c_str();
			plot_name += ": Const T";
		}
		else
		{
			plot_name = "Plot ";
			plot_name += int_to_string(++plot_cnt).c_str();
			plot_name += ": Const X";
		}
	}
	else plot_name = WL;

	QpTraceItem* traceItem = new QpTraceItem(panelItem, plot_name, dynamic_cast<QpPlotItem*>(panelItem->lastChild()), WL, QP_CONST_XT);
	setCurrentItem(traceItem);
	updateOnSelection();

	return traceItem;

}

// ----------------------------------------

void QpListView::makeConstYTraceItem()
{
	
	makeEmptyItem(QP_CONST_Y);

}

// ----------------------------------------

QpTraceItem* QpListView::makeConstYTrace(QpPanelItem* panelItem, QString WL)
{
	// creates new trace item in panel
	if(panelItem == NULL) return NULL;
  	clearSelectedList();

	QString plot_name;
	if(WL.isEmpty())
	{
		int plot_cnt = panelItem->childCount();
		plot_name = "Plot ";
		plot_name += int_to_string(++plot_cnt).c_str();
		plot_name == ": Const Y";
	}
	else plot_name = WL;
	
	QpTraceItem* traceItem = new QpTraceItem(panelItem, plot_name, dynamic_cast<QpPlotItem*>(panelItem->lastChild()), WL, QP_CONST_Y);
	setCurrentItem(traceItem);
	updateOnSelection();
	
	return traceItem;

}

// ----------------------------------------

void QpListView::makeSpectrumItem()
{
	
	makeEmptyItem(QP_SPECTROGRAM);		
}

// ----------------------------------------

QpSpectrumItem* QpListView::makeSpectrum(QpPanelItem* panelItem, QString WL)
{
	// creates new trace item in panel
	if(panelItem == NULL) return NULL;
	clearSelectedList();

	QString plot_name;
	if(WL.isEmpty())
	{
		int plot_cnt = panelItem->childCount();
		plot_name = "Plot ";
		plot_name += int_to_string(++plot_cnt).c_str();
		plot_name += ": Spect";
	}
	else 
	{
		if(panelItem->frameType() == QP_TS)
		{
			plot_name = QuGetDefaultComp(WL, QSAS_profiles->current()->Array3D);
		}
		else
		{
			plot_name = QuGetDefaultComp(WL, QSAS_profiles->current()->Array3DXY);
		}
	}
	
	QpSpectrumItem* spectrumItem = new QpSpectrumItem(panelItem, plot_name, dynamic_cast<QpPlotItem*>(panelItem->lastChild()), "");
	spectrumItem->DataTab->_z_data_fld->setText(plot_name);
	spectrumItem->DataTab->_z_data_fld->reValidate();
	
	setCurrentItem(spectrumItem);
	updateOnSelection();

	return spectrumItem;

}

// ----------------------------------------

void QpListView::makeEventItem()
{
	
	makeEmptyItem(QP_EVENT);		
}

// ----------------------------------------

QpEventItem* QpListView::makeEvent(QpPanelItem* panelItem)
{
	// creates new trace item in panel
	if(panelItem == NULL) return NULL;
  	clearSelectedList();

	int plot_cnt = panelItem->childCount();
	string plot_name("Plot "+int_to_string(++plot_cnt)+": Event");
	QpEventItem* eventItem = new QpEventItem(panelItem, QString(plot_name.c_str()), dynamic_cast<QpPlotItem*>(panelItem->lastChild()));
	setCurrentItem(eventItem);
	updateOnSelection();
	return eventItem;

}


// ----------------------------------------

void QpListView::moveItemDown()
{
	// slot invoked from Move Down menu item on Right Mouse Button menu
	// moves item down in the listview hierarchy, so below its sibling

	// fetch current item
	QpListItem *item = dynamic_cast<QpListItem*>(selectedItem());
	if(!item) return;
  
	QpListItem *after = NULL;
	
	// fetch item below selected item in hierarchy
	QpListItem *item_below = dynamic_cast<QpListItem*>(item->itemBelow());
	if(!item_below) return;
  
	// keep going while item_below's parent is not item's parent
	while(item_below )
	{
		if(item_below->parent() == item->parent() ) 
		{
			after = item_below;
			break;
		}
		item_below = dynamic_cast<QpListItem*>(item_below->itemBelow());
	}
	// found items younger sibling so move item below item_below in hierarchy
	if(after) {
		item->moveItem(after);
		emit pageLayoutChanged(item->pageItem());
	}
}

// ----------------------------------------

void QpListView::moveItemUp()
{
	// slot invoked from Move Up menu item on Right Mouse Button menu
	// moves item above its sibling

	// fetch current item
	QpListItem *item = dynamic_cast<QpListItem*>(selectedItem());
	if(!item) return;
  
	QpListItem *above = NULL;
  
	// get item immediately above item in hierarchy
	QpListItem *item_above = dynamic_cast<QpListItem*>(item->itemAbove());
	if(!item_above) return;
  
	// keep going while item_above's parent is not item's parent and
	while (item_above )
	{
		if(item_above->parent() == item->parent())
		{
			above = item_above;
			break;
		}
		item_above = dynamic_cast<QpListItem*>(item_above->itemAbove());
	}
  
	// found item's older sibling - move item before item_above
	if(above) 
	{
		above->moveItem(item);
		emit pageLayoutChanged(item->pageItem());
	}
}

// ----------------------------------------

void QpListView::plotCurrentItem()
{
	// slot invoked from plot menu item on Right Mouse Button menu
	// plots page for current item
	// fetch current item
	QpListItem *item = dynamic_cast<QpListItem*>(currentItem());

	if(!item)
	{
		(void)QMessageBox::warning(this,"Plot",("No Items Selected..."));
		return;
	}
	QpPageItem *page_item = item->pageItem();
	if(!page_item)
	{
		(void)QMessageBox::warning(this,"Plot Current Item",("No Pages Selected..."));
		return;
	}

	_plot_page_list.clear();
	_plot_page_list.push_back(page_item);

	plot();
}

// ----------------------------------------

void QpListView::plotSelectedItems()
{
	// slot invoked from plot menu item on File Menu
	// plots page for selected items

	QpListItem *plot_item = 0;
	QpPageItem *page_item = 0;

	_plot_page_list.clear();
	for(int i=0; i<localList.size(); i++)
	{
		plot_item = dynamic_cast<QpListItem*>(localList[i]);
		if(!plot_item) continue;
		// fetch page and add to list of pages to be plotted if not already included
	 
		page_item = plot_item->pageItem();
		if(!page_item) continue;
		bool found = false;
		for(int i=0; i<_plot_page_list.size(); i++)
		{
			if(page_item == _plot_page_list[i]) found = true;
		}
		if(!found) _plot_page_list.push_back(page_item);
	}

	plot();
}

// ----------------------------------------

void QpListView::plot()
{
	QpPageItem* page_item;
	
	if(_plot_page_list.size() == 0)
	{
		(void)QMessageBox::warning(this,"Plot",("No Pages Selected..."));
		return;
	}
		
	QApplication::setOverrideCursor( Qt::waitCursor );
	#if QT_VERSION < 0x040000
	qApp->processEvents(2000);
	#else
	qApp->processEvents(QEventLoop::AllEvents, 2000);
	#endif
	
	try{
		for(uint i=0; i<_plot_page_list.size(); i++)
		{
			page_item = _plot_page_list[i];
			if(!page_item) continue;
			
			double width, height;
			double conv;
			
			width = page_item->PageTab->_width_fld->text().toDouble();
			height =page_item->PageTab->_height_fld->text().toDouble();
			
			int _units = page_item->PageTab->_units_fld->currentItem();
			conv=page_item->_conversion_factors[_units];
			width=width/conv; //conversion to inches
			height=height/conv; //conversion to inches
			
			conv=page_item->_conversion_factors[0]; // conv factor to pixels
			width=width*conv; //px
			height=height*conv; //px
			
			if(width<=0. || height<=0.)
			{
				width=800.;
				height=600.;
			}
			
		
			if(page_item->PageTab->_orientation_fld->currentItem() == QPUI_LANDSCAPE && width<height || page_item->PageTab->_orientation_fld->currentItem() == QPUI_PORTRAIT && width>height)
			{
				double temp=width;
				width=height;
				height=temp;
			}
			
			PlotWindow* win;
			int plot_id=page_item->PageTab->_xwin_num_fld->text().toInt();
			QPrinter::Orientation orientation=(page_item->PageTab->_orientation_fld->currentItem() == QPUI_LANDSCAPE ? QPrinter::Landscape : QPrinter::Portrait);
			QPrinter::PageSize pageSize=QPrinter::A4;
			int resolution=72;
			switch(page_item->PageTab->_page_size_fld->currentItem())
			{
				case QPUI_EXECUTIVE:
					pageSize=QPrinter::Executive;
					break;
				case QPUI_LEGAL:
					pageSize=QPrinter::Legal;
					break;
				case QPUI_LETTER:
					pageSize=QPrinter::Letter;
					break;
				default: break;
			}
			
			switch(page_item->PageTab->_dpi_fld->currentItem())
			{
				case QPUI_150_DPI: resolution=150; break;
				case QPUI_300_DPI: resolution=300; break;
				case QPUI_600_DPI: resolution=600; break;
				default: resolution=72; break;
			}
			
			if(plot_id < 0) // create new window
			{
				win=page_item->newWindow(width, height, &(plot_id), resolution, pageSize, orientation);
				page_item->PageTab->setXwinNum(QString::number(plot_id));
				page_item->PageTab->_xwin_num_fld->setText(QString::number(plot_id));
			}
			else
			{
				win=page_item->focusWindow(plot_id, width, height, resolution, pageSize, orientation);
			}
			
			win->disconnect(SIGNAL(clicked(double, double, int, int))); // disconnect possibly pre-existing connections
			
			connect(win, SIGNAL(clicked(double, double, int, int)), page_item, SLOT(displayIntervalWidget(double, double, int, int)));
			
			if(QPUI_FAILURE == page_item->plot(win))
			{
				(void)QMessageBox::warning(this,"Plot", ("Problem plotting page "+page_item->text(NAME_COL)));
			}
		}
	}
	catch(...)
	{;}
	
	QApplication::restoreOverrideCursor();
	#if QT_VERSION < 0x040000
	qApp->processEvents(2000);
	#else
	qApp->processEvents(QEventLoop::AllEvents, 2000);
	#endif
}



// ----------------------------------------

bool QpListView::reNameItem(QpListItem *item)
{
	// function to rename item in listview
  
	if(!item) return false;
	else
	{
		bool result = QguListView::reNameItem((QguListItem*)item);
		item->nameLock = true; // lock against object drop if user set
		QpPageItem * page_item=dynamic_cast<QpPageItem*>(item);
		
		if(page_item!=NULL)
		{
			page_item->rename(item->text(0));
		}
		
		return result;
	}
}

// ----------------------------------------

void QpListView::renameSelectedItem()
{
	// slot invoked from Rename menu item on Right Mouse Button menu
	// renames selected item

	QpListItem *item = dynamic_cast<QpListItem*>(selectedItem());
	if(!item) return;
	reNameItem(item);
}

// ----------------------------------------

void QpListView::rightButtonPressEvent(Q_LIST_VIEW_ITEM *item, const QPoint &P, int c)
{
	//  RtButton menu item gets launched
    setSelectedList(dynamic_cast<QpListItem*>(item));
    qApp->processEvents();
    RButtonMenu->exec(P);
}

// ----------------------------------------

void QpListView::setDefaultSelectedItems()
{
	// slot invoked from Use Default Settings file menu item

	for(int i=0; i<localList.size(); i++)
	{
		QpListItem *plot_item = dynamic_cast<QpListItem*>(localList[i]);
		if(!plot_item) continue;
		plot_item->setDefaultOptions();
	}
}

// ----------------------------------------

void QpListView::linkTraceItems(vector <QpTraceItem *> stack, bool onceOnly)
{
	//  invoked to link traces in multi-panel creation
	QString LinkText;
	
	LinkNumber++;
	LinkText = QString("*%1").arg(LinkNumber);
	if(onceOnly) LinkText.insert(0, "*"); // extra * for searching on link once only
	list <QpPlotItem*> linkedPlots;
	for(int i=0; i<stack.size(); i++)
	{
		QpPlotItem *plot_item = dynamic_cast<QpPlotItem*>(stack[i]);
		if(!plot_item) continue;
		linkedPlots.push_back(plot_item);
		plot_item->setText(LINK_COL, LinkText);
	}
	// now tell each trace
	for(int i=0; i<stack.size(); i++)
	{
		QpPlotItem *plot_item = dynamic_cast<QpPlotItem*>(stack[i]);
		if(!plot_item) continue;
		plot_item->setLinks(linkedPlots);
	}

}

// ----------------------------------------

void QpListView::linkPlots()
{
	// slot invoked from link plots edit menu
	QString LinkText;
	
	LinkNumber++;
	LinkText = QString("*%1").arg(LinkNumber);

	list <QpPlotItem*> linkedPlots;
	for(int i=0; i<localList.size(); i++)
	{
		QpPlotItem *plot_item = dynamic_cast<QpPlotItem*>(localList[i]);
		if(!plot_item) continue;
		linkedPlots.push_back(plot_item);
		plot_item->setText(LINK_COL, LinkText);
	}
	// now tell each trace
	for(int i=0; i<localList.size(); i++)
	{
		QpPlotItem *plot_item = dynamic_cast<QpPlotItem*>(localList[i]);
		if(!plot_item) continue;
		plot_item->setLinks(linkedPlots);
	}	
	clearSelectedList();
	clearSelection();
}


// ----------------------------------------

void QpListView::removeLink(QpPlotItem* plot_item)
{
	// go through each plot item unlinking traces
	QpPageItem *pageItem = dynamic_cast<QpPageItem*> (this->firstChild());
	while(pageItem)
	{
		QpFrameItem *frameItem = dynamic_cast<QpFrameItem*> (pageItem->firstChild());
		while(frameItem)
		{
			QpPanelItem *panelItem = dynamic_cast<QpPanelItem*> (frameItem->firstChild());
			while(panelItem)
			{
				QpPlotItem *plotItem = dynamic_cast<QpPlotItem*> (panelItem->firstChild());
				while(plotItem)
				{
					plotItem->removeLink(plot_item);
					
					
					plotItem = dynamic_cast<QpPlotItem*> (plotItem->nextSibling());
				}
				panelItem = dynamic_cast<QpPanelItem*> (panelItem->nextSibling());
			}
			frameItem = dynamic_cast<QpFrameItem*> (frameItem->nextSibling());
		}
		pageItem = dynamic_cast<QpPageItem*> (pageItem->nextSibling());
	}


}
// ----------------------------------------

void QpListView::unlinkPlots()
{
	// slot invoked from unlink traces edit menu

	list <QpPlotItem*> linkedPlots;
	for(int i=0; i<localList.size(); i++)
	{
		QpPlotItem *plot_item = dynamic_cast<QpPlotItem*>(localList[i]);
		if(!plot_item) continue;
		linkedPlots.push_back(plot_item);
		plot_item->setText(LINK_COL, "");
	}
	
	list <QpPlotItem*>::iterator it;
	for( it=linkedPlots.begin(); it != linkedPlots.end(); it++)
	{
		(*it)->removeFromLinks();
	}
	
}

// ----------------------------------------

void QpListView::setDefaultCurrentItem()
{
	// slot invoked from Use Default Settings file menu item

	QpListItem *plot_item = dynamic_cast<QpListItem*>(currentItem());
	if(!plot_item) return;
	plot_item->setDefaultOptions();

}

// ----------------------------------------

void QpListView::setPageLayout(QpPageItem *pageItem)
{
	// slot invoked when pageLayoutChanged signal emitted
	if(QuSlotDebug())
	{
		printf("QpListView::setPageLayout %ld %ld\n", this, pageItem);
		fflush(stdout);
	}
	
	// tell pages to redo layout
	if(pageItem)
	{
		pageItem->LayoutTab->updateRowCol(pageItem->numberOfChildren());
	}
	
}

// ----------------------------------------

void QpListView::setSelectedList(QguListItem *item)
{
	// overload function to add item to local list of selected items

	this->QguListView::setSelectedList(item);
	updateOnSelection();
	
}

// ----------------------------------------

void QpListView::updateOnSelection()
{
	QWidget * oldPage = this->tabWidget->currentPage();
	if(oldPage)
	{
		QpTab * oldTab = dynamic_cast <QpTab*>(oldPage);
		if(this->I_LastTab == QP_IGNORE_TAB) this->I_LastTab = QP_DATA_TAB;
		else this->I_LastTab = oldTab->TabType;
	}

	// sets tabs appropriate when selection is changed
	QpListItem *item = dynamic_cast<QpListItem*>(this->currentItem());
	if(QuSlotDebug())
	{
		printf("QpListView::updateOnSelection %ld %ld\n", this, item);
		fflush(stdout);
	}

	
	// show tabs for different item types via virtual method
	if(item) item->showTabs(); 
	this->tabWidget->adjustSize();
}

// ----------------------------------------

void QpListView::refreshWLitems()
{
	if(QuSlotDebug()) cout << "QpListView::refreshWLitems" << endl;

	// go through each plot item checking if data object has changed
	QpPageItem *pageItem = dynamic_cast<QpPageItem*> (this->firstChild());
	while(pageItem)
	{
		QpFrameItem *frameItem = dynamic_cast<QpFrameItem*> (pageItem->firstChild());
		while(frameItem)
		{
			QpPanelItem *panelItem = dynamic_cast<QpPanelItem*> (frameItem->firstChild());
			while(panelItem)
			{
				QpPlotItem *plotItem = dynamic_cast<QpPlotItem*> (panelItem->firstChild());
				while(plotItem)
				{
					QpTraceItem * traceItem = dynamic_cast<QpTraceItem*> (plotItem);
					if(traceItem) traceItem->refreshTabs();
					
					QpSpectrumItem * spectrumItem = dynamic_cast<QpSpectrumItem*> (plotItem);
					if(spectrumItem) spectrumItem->refreshTabs();
					
					plotItem = dynamic_cast<QpPlotItem*> (plotItem->nextSibling());
				}
				panelItem = dynamic_cast<QpPanelItem*> (panelItem->nextSibling());
			}
			frameItem = dynamic_cast<QpFrameItem*> (frameItem->nextSibling());
		}
		pageItem = dynamic_cast<QpPageItem*> (pageItem->nextSibling());
	}
	
}


// ----------------------------------------

bool QpListView::findPItem(QpListItem *item)
{
	// find if QpListItem is anywhere in list view
	// used to trap when item has been deleted but is still known about
    Q_LIST_VIEW_ITEM_ITERATOR it( this );
    while ( it.current() ) {
        if ( it.current() == item) return true;
        ++it;
    }
	return false;
}



// ----------------------------------------

Vistream& operator>>(Vistream& v_is, QpListView& lv)
{
	if(QuSlotDebug()) cout <<  "QpListView restore (>>)" << endl;

	// streams from virtual input stream v_is into item
	lv.clearSelectedList();
	int cnt;
	v_is >> cnt;
	for(int i=0; i<cnt; i++)
	{
		QpPageItem *item = new QpPageItem(&lv, "", dynamic_cast<QpPageItem*>(lv.lastChild()));
		v_is >> *item;
		QString name = ((Q_LIST_VIEW_ITEM*)item)->text(NAME_COL);
		while(item->nameAlreadyUsed(name)) name += "A";
		item->setText(NAME_COL, name);
	}
	
	// relink items now everything is restored
	vector <QString> linkFlags;
	linkFlags.clear();
	Q_LIST_VIEW_ITEM_ITERATOR it(&lv);
	while(it.current())
	{
		QString thisText = it.current()->text(LINK_COL);
		if(!thisText.isEmpty() && thisText.contains("S")) 
		{
			bool found = false;
			// found restored linked item, assemble list of unique link texts
			for (int i=0; i< linkFlags.size(); i++)
			{
				if(linkFlags[i] == thisText) 
				{
					found = true;
					break;
				}
			}
			if (!found) linkFlags.push_back(thisText);
		}
		++it;
	}
	// we now have a list of known link text strings, link them
	for(int i=linkFlags.size()-1; i>=0; i--)
	{
		lv.clearSelectedList();
		Q_LIST_VIEW_ITEM_ITERATOR it1(&lv);
		while(it1.current())
		{
			if(it1.current()->text(LINK_COL) == linkFlags[i])  lv.AddToSelectedList((QpListItem *) it1.current());
			it1++;
		}
		lv.linkPlots();
	}
	lv.clearSelectedList();
	lv.clearSelection();		// unselect everything

		
	QpPageItem *pageOne = dynamic_cast<QpPageItem*>(lv.firstChild());
	if(pageOne) 
	{
		lv.setCurrentItem(pageOne);	// set first page as current item
		lv.updateOnSelection();		// show tabs for current item
	}
	
	return v_is;
}

// ----------------------------------------

Vostream& operator<<(Vostream& v_os, QpListView& lv)
{
	if(QuSlotDebug()) cout << "QpListView save (<<)" << endl;

	// streams item to virtual output stream v_os

#if QT_VERSION < 0x040000
	QList<QpPageItem> lst;
#else
        QList<QpPageItem*> lst;
#endif

	QpPageItem *item = dynamic_cast<QpPageItem*>(lv.firstChild());
	int cnt=0;
	while(item)
	{
		lst.append(item);
		cnt++;
		item = dynamic_cast<QpPageItem*>(item->nextSibling());
	}
	v_os << cnt;
	for(int i=0; i<cnt; i++)
	{
		QpPageItem *item = lst.at(i);
		v_os << *item;
	}

	return v_os;
}


//      QpListItem CLASS
//      -----------------

QpListItem::QpListItem(QpListView *parent,  QString txt, QpListItem *after, QguListType type, QString WL )
   :  QguListItem(parent, txt, type, after)
{
	// Constructor

	if(type == ListTypeEntry) WLcontext = QString(WL);
	I_Type = QP_NONE;
	tabWidget = parent->tabWidget;  
	I_ListView = parent;
}

//     ----------------------

QpListItem::QpListItem(QpListItem *parent,  QString txt,
   QpListItem *after, QguListType type, QString WL )
   :  QguListItem(parent, txt, type, after)
{
	// Constructor

	if(type == ListTypeEntry) WLcontext = QString(WL);
	I_Type = QP_NONE;
	tabWidget = parent->tabWidget;  
	I_ListView = parent->I_ListView;
}

//     ----------------------

QpListItem* QpListItem::lastChild() const
{
	// return youngest child of current item in listview hierarchy

	Q_LIST_VIEW_ITEM *item = firstChild();
	if(!item) return 0;

	while(item->nextSibling()) item = item->nextSibling();
	return dynamic_cast<QpListItem*>(item);
}


int QpListItem::numberOfChildren() const
{	
	int cnt = 0;
	if( !this->firstChild() ) return cnt;
	
	QpListItem *listItem = dynamic_cast<QpListItem*>(this->firstChild());
	while (listItem)
	{
		cnt++;
		listItem = dynamic_cast<QpListItem*>(listItem->nextSibling());
	}
	return cnt;
}

//     ----------------------

QpPageItem* QpListItem::pageItem()
{
	// returns current item's page item

	QpPageItem *page_item = dynamic_cast<QpPageItem*>(this);
	if(page_item) return page_item;
  
	QpFrameItem *frame_item = dynamic_cast<QpFrameItem*>(this);
	if(frame_item) return frame_item->I_Page;
  
	QpPanelItem *panel_item = dynamic_cast<QpPanelItem*>(this);
	if(panel_item) return panel_item->I_Frame->I_Page;

	QpPlotItem *plot_item = dynamic_cast<QpPlotItem*>(this);
	if(plot_item) return plot_item->I_Panel->I_Frame->I_Page;

	return NULL;
}

//     ----------------------

QpListItem* QpListItem::parent()
{
   // return current item's parent in listview hierarchy

   Q_LIST_VIEW_ITEM *item = this;
   if(!item) return NULL;
   return dynamic_cast<QpListItem*>(item->parent());
}


//     ----------------------

Vistream& operator>>(Vistream& v_is, QpListItem& item)
{
	// streams from virtual input stream v_is into item

	QguListItem& qgu_item = item;
	v_is >> qgu_item;
	
	int type;
	v_is >> type;
	item.I_Type = (QpItemType)type;
	
	return v_is;
}

Vostream& operator<<(Vostream& v_os, QpListItem& item)
{
   // streams item to virtual output stream v_os

   QguListItem& qgu_item = item;
   qgu_item.with = 0; // === ensure no data written by old code
   v_os << qgu_item;

   v_os << item.I_Type;
	
	return v_os;
}

