//  DataAvailability.cc
//
//  Qdos - data interfacing
//
//  David Burgess
//  March 2000
//

#include "DataAvailability.h"
#include "StringTokenizer.h"
#include <iostream>
#include <stdio.h> // for sprintf

namespace QDOS {

DataAvailability::DataAvailability(
   const string& data_name, TimeInterval time_interval,const double& resolution, 
   const string& data_availability_string ): _data_name(data_name),\
   _time_interval(time_interval), _resolution(resolution)

{
  set_num_quanta();
  set_data_availability(data_availability_string);
}

DataAvailability::DataAvailability(
   const string& data_name, TimeInterval time_interval,
   const string& data_availability_string ): _data_name(data_name),\
    _time_interval(time_interval)
{ 
  char delim[]=":";
  StringTokenizer da_st(data_availability_string.c_str(),delim);
  if(da_st.get_n_tokens()!=2)
    return;
  
  char *resolution_str=da_st[0];
  char *encoded_avail_str=da_st[1];

  istrstream ist(resolution_str);
  ist >> _resolution;
  delete[] resolution_str;
  resolution_str=NULL;
  
  set_num_quanta();
  
  set_data_availability(encoded_avail_str); 
  
  delete[] encoded_avail_str;
  encoded_avail_str=NULL;

}

void DataAvailability::set_data_availability(const string& \
  data_availability_string)
{
  static const int bitmasks[] = { 32, 16, 8, 4, 2, 1 };
  
  int bpw = 6;
  int len=data_availability_string.size();
  int avail_sz=_available.size();
  
  for( register int i=0; i < _num_quanta; i+=bpw )
  {
    int bitcount = bpw;
    if( (i+bpw) > avail_sz )
      bitcount = avail_sz - i;
    int cpos = i/bpw;
    // if run out of characters in string, then exit loop
    //  effectively leaves rest of availability as false
    if( cpos >= len )
      break;
    int ival = data_availability_string[ cpos ];
    
    if(ival>=97) ival -= 6;
    if(ival>=63) ival -= 5;
    ival -= 48;
    
    for( register int j=0; j < bitcount; j++ )
      _available[ i+j ] = ival<<(bpw-bitcount) & bitmasks[j];
  }
}

void DataAvailability::set_num_quanta(void)
{
  if(fabs(_resolution)<0.001)
    _num_quanta=1;
  else
  {
    _num_quanta =  static_cast<int>(
                 _time_interval.duration_abs().get_time_rep()/fabs(_resolution))
                 + 1;
  }
  _available.assign(_num_quanta,false);
}

void DataAvailability::set_resolution( double resolution )
{
  _resolution = resolution;
  if( _time_interval.duration_abs().get_time_rep() != 0 )
    set_num_quanta();
}

void DataAvailability::set_data_available( Time t )
{
  if( !_time_interval.contains(t) ) return;
  int i;
  if(fabs(_resolution)<0.001)
    i=0;
  else
    i = static_cast<int>((t - _time_interval.start()).get_time_rep()
                            /fabs(_resolution) );
  _available[i] = true;
}

bool DataAvailability::is_data_available( Time t ) const
{
  if( !_time_interval.contains(t) ) return false;
  int i;
  if(fabs(_resolution)<0.001)
    i=0;
  else
    i = static_cast<int>((t - _time_interval.start()).get_time_rep()
                            /fabs(_resolution) );
  return _available[i];
}

bool DataAvailability::is_data_available( TimeInterval tintvl ) const
{
  bool ok = false; // return value
  tintvl.force_sense();
  if( tintvl.end() < _time_interval.start()
      || tintvl.start() > _time_interval.end() ) return false;
// limit requested time interval to that of data availability
  if( tintvl.start() < _time_interval.start() )
    tintvl.set_start( _time_interval.start() );
  if( tintvl.end() > _time_interval.end() )
    tintvl.set_end( _time_interval.end() );
  int i,j;
  if(fabs(_resolution)<0.001)
    i=j=0;
  else
  {
    i = static_cast<int>
      ((tintvl.start() - _time_interval.start()).get_time_rep()
      /fabs(_resolution) );
    j = static_cast<int>
      ((tintvl.end() - _time_interval.start()).get_time_rep()
      /fabs(_resolution) );
  }
  for( int k = i; k <= j; k++ )
    if( _available[k] ) { ok=true; break; }
  return ok;
}

string DataAvailability::srep(void) const
{
  string srep_string; // return value

  char number_buff[100];

  if(ceil(_resolution)==floor(_resolution))
    sprintf(number_buff,"%g.0:",_resolution);
  else
    sprintf(number_buff,"%g:",_resolution);

  srep_string = number_buff;

// convert the bool vector into a sequence of 6bit integers
// and map to printable chars

  int bpw = 6; // bits per word
  for( int i=0; i < _available.size(); i+=bpw )
  {
    int bitcount = bpw;
    if( (i+bpw) > _available.size() )
      bitcount = _available.size() - i;
    int ival = 0;
    for( int j=0; j<bitcount; j++ )
      ival = ival*2 + ( _available[i+j] ? 1 : 0 );

//    cout << ival << " ";

// need 64 characters for 6 bits per word
// 0 - 9  ...  48 - 57
// ?@     ...  63 - 64
// A - Z  ...  65 - 90
// a - z  ...  97 - 122
   ival += 48;
   if( ival > 57 ) ival += 5;
   if( ival > 90 ) ival += 6;

   char c = ival;
   srep_string += c;
  }

//  cout << endl;
  return srep_string;
}

void DataAvailability::diag_print(void) const
{
  cout << "DataAvailability for : "
       << _data_name
       << " time period: " << _time_interval
       << " resolution: " << _resolution
       << " num quanta: " << _num_quanta << endl;
  for( int i=0; i< _available.size(); i++ )
    cout << _available[i];
  cout << endl;  
  cout << srep() << endl; 
}

} // end namespace QDOS
